
'EZP a wide string document viewer/printer inspired by DDOC and Virtual Print Engine
'Compilable in 32 and 64 bit by FreeBasic 
'by James Klutho

' Released into the public domain for private and public use without restriction.
' This code is provided as is without warranty of any kind, either expressed or implied,
' including and not limited to any implied warranties, merchantability, or fitness for
' a particular purpose.

'072317  - made changes to the dialog box control fonts


#INCLUDE ONCE "windows.bi"
#INCLUDE ONCE "win/winspool.bi"
#INCLUDE ONCE "win/commdlg.bi"

#ifdef __FB_64BIT__
    #inclib "gdiplus"
    #include once "win/gdiplus-c.bi"
#else
    #include once "win/ddraw.bi"
    #include once "win/gdiplus.bi"
    using gdiplus
#endif

#define RGBA_R( c ) ( CUInt( c ) Shr 16 And 255 )
#define RGBA_G( c ) ( CUInt( c ) Shr  8 And 255 )
#define RGBA_B( c ) ( CUInt( c )        And 255 )
#define RGBA_A( c ) ( CUInt( c ) Shr 24         )

#define ezt(r,c) EZP_G(r,c,0)
#define ezl(r,c) EZP_G(r,c,1)
#define ezb(r,c) EZP_G(r,c,2)
#define ezr(r,c) EZP_G(r,c,3)


#define GetFilePointer(hFile) SetFilePointer(hFile, 0, NULL, FILE_CURRENT)
#define EZP_VERSION 1
#define EZP_MAX_PAGES 1000
#define EZP_MAX_BOOKMARKS 50
#define EZP_MAX_GRAPHICS 50
#define EZP_DEFAULTPAPERSIZE  1 'Letter
#define EZP_MAX_PAPERS 35
#define EZP_MAXTEXTLEN 1024
#define EZP_DIAGVIEWTOP 50
#define EZP_FONT_NAME "Arial"  'Default Font Name
#define EZP_FONT_SIZE 10       'Default Font Size4
#define EZP_MAXNUMCOLS 300
#define EZP_MAXNUMROWS 1000
#define EZP_GRIDCOLDEFAULT 0.25      'or 0.63 cm
#define EZP_GRIDROWDEFAULT 0.25      'or 0.63 cm
#define EZP_GRIDMARGINSDEFAULT 0.75  'or 1.91 cm
#define DegreesToRadians(Deg) = Deg * 0.0174532925199432957

Const PI As Double = 3.1415926535897932
const nWidthToArrowRatio as integer = 7
const fSliceAngle as  double = pi/180*(45)
Const HalfPI as Double = 1.5707963267948965
                                                            


'Dialog Child Controls
Const MarginsMenu      = 1
Const HeadersMenu      = 2
Const GridMenu         = 3
Const UserMarginsMenu  = 4

const MyCloseBut       = 101
const MyFirstBut       = 102
const MyBackBut        = 103
const MyPageStatic     = 104
const MyNextBut        = 105
const MyLastBut        = 106
const MyBookmarkCombo  = 107
const MyPrintBut       = 108
CONST MyZoomCombo      = 109

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Function Return Constants for some function
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
const EZP_FAILED          = -1
const EZP_SUCCESS         =  0

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Unit of measure constants as passed to EZP_DocBegin
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
const EZP_INCH           =  0
const EZP_CM             =  1

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  EZP_DoDoc Option constants
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
const EZP_END_VIEW             =   0
const EZP_END_PRINT            =   1    'With Dialogs
const EZP_END_PRINT_NODIALOG   =   2
const EZP_END_DELETE           =   4    'Delete File

const EZP_VIEWER_GRID            =   1   
const EZP_VIEWER_MARGINS         =   2
const EZP_VIEWER_HEADERFOOTER    =   4    '
const EZP_VIEWER_ZOOMFIT         =   8
const EZP_IGNOREFILEEND          =  16


const EZP_ZOOM100        =   0
const EZP_ZOOMFIT        =   1
const EZP_ZOOMWIDTH      =   2
const EZP_ZOOM125        =   3
const EZP_ZOOM75         =   4



'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Orientation Constants for use with EZP_DocBegin
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
const EZP_PORTRAIT       =  0   'DMORIENT_PORTRAIT = 1
const EZP_LANDSCAPE      =  1   'DMORIENT_LANDSCAPE = 2

'const EZP_DUPLEX_UNKNOWN    =  0
const EZP_DUPLEX_NONE       =  0 ' Normal nonduplex Printer  default is actually DMDUP_SIMPLEX = 1
const EZP_DUPLEX_VERTICAL   =  2 ' Long Edge binding - the long edge of the page is vertical 
const EZP_DUPLEX_HORIZONTAL =  3 ' Short Edge binding - the long edge of the page is horizontal

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Paper Bin Constants used in EZP_DocBegin and EZP_DocNewPage
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

' Paper sources.
' DMBIN_UPPER = 1; DMBIN_LOWER = 2; DMBIN_MIDDLE = 3;
' DMBIN_MANUAL = 4; DMBIN_ENVELOPE = 5; DMBIN_ENVMANUAL = 6; DMBIN_AUTO = 7;
' DMBIN_TRACTOR = 8; DMBIN_SMALLFMT = 9; DMBIN_LARGEFMT = 10; DMBIN_LARGECAPACITY = 11;
' DMBIN_CASSETTE = 14; DMBIN_FORMSOURCE = 15

CONST EZP_BIN_UPPER = 1
CONST EZP_BIN_FIRST = EZP_BIN_UPPER
CONST EZP_BIN_ONLYONE = 1
CONST EZP_BIN_LOWER = 2
CONST EZP_BIN_MIDDLE = 3
CONST EZP_BIN_MANUAL = 4
CONST EZP_BIN_ENVELOPE = 5
CONST EZP_BIN_ENVMANUAL = 6
CONST EZP_BIN_AUTO = 7
CONST EZP_BIN_TRACTOR = 8
CONST EZP_BIN_SMALLFMT = 9
CONST EZP_BIN_LARGEFMT = 10
CONST EZP_BIN_LARGECAPACITY = 11
CONST EZP_BIN_CASSETTE = 14
CONST EZP_BIN_FORMSOURCE = 15

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Paper Size Constants used in EZP_DocBegin and EZP_DocNewPage
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

' Paper.
' DMPAPER_LETTER = 1; DMPAPER_FIRST = DMPAPER_LETTER; DMPAPER_LETTERSMALL = 2; DMPAPER_TABLOID = 3;
' DMPAPER_LEDGER = 4; DMPAPER_LEGAL = 5; DMPAPER_STATEMENT = 6; DMPAPER_EXECUTIVE = 7; DMPAPER_A3 = 8;
' DMPAPER_A4 = 9; DMPAPER_A4SMALL = 10; DMPAPER_A5 = 11; DMPAPER_B4 = 12; DMPAPER_B5 = 13;
' DMPAPER_FOLIO = 14; DMPAPER_QUARTO = 15; DMPAPER_10X14 = 16; DMPAPER_11X17 = 17; DMPAPER_NOTE = 18;
' DMPAPER_ENV_9 = 19; DMPAPER_ENV_10 = 20; DMPAPER_ENV_11 = 21; DMPAPER_ENV_12 = 22; DMPAPER_ENV_14 = 23;
' DMPAPER_CSHEET = 24; DMPAPER_DSHEET = 25; DMPAPER_ESHEET = 26; DMPAPER_ENV_DL = 27; DMPAPER_ENV_C5 = 28;
' DMPAPER_ENV_C3 = 29; DMPAPER_ENV_C4 = 30; DMPAPER_ENV_C6 = 31; DMPAPER_ENV_C65 = 32; DMPAPER_ENV_B4 = 33;
' DMPAPER_ENV_B5 = 34; DMPAPER_ENV_B6 = 35

CONST EZP_PAPER_LETTER = 1           '  Letter 8 1/2 x 11 in
CONST EZP_PAPER_LETTERSMALL = 2      '  Letter Small 8 1/2 x 11 in
CONST EZP_PAPER_TABLOID = 3          '  Tabloid 11 x 17 in
CONST EZP_PAPER_LEDGER = 4           '  Ledger 17 x 11 in
CONST EZP_PAPER_LEGAL = 5            '  Legal 8 1/2 x 14 in
CONST EZP_PAPER_STATEMENT = 6        '  Statement 5 1/2 x 8 1/2 in
CONST EZP_PAPER_EXECUTIVE = 7        '  Executive"7 1/2 x 10 in
CONST EZP_PAPER_A3 = 8               '  A3 297 x 420 mm
CONST EZP_PAPER_A4 = 9               '  A4 210 x 297 mm
CONST EZP_PAPER_A4SMALL = 10         '  A4 Small 210 x 297 mm
CONST EZP_PAPER_A5 = 11              '  A5 148 x 210 mm
CONST EZP_PAPER_B4 = 12              '  B4 250 x 354
CONST EZP_PAPER_B5 = 13              '  B5 182 x 257 mm
CONST EZP_PAPER_FOLIO = 14           '  Folio 8 1/2 x 13 in
CONST EZP_PAPER_QUARTO = 15          '  Quarto 215 x 275 mm
CONST EZP_PAPER_10x14 = 16           '  10x14 in
CONST EZP_PAPER_11X17 = 17           '  11x17 in
CONST EZP_PAPER_NOTE = 18            '  Note 8 1/2 x 11 in
CONST EZP_ENV_9 = 19                 '  Envelope #9 3 7/8 x 8 7/8
CONST EZP_ENV_10 = 20                '  Envelope #10 4 1/8 x 9 1/2
CONST EZP_ENV_11 = 21                '  Envelope #11 4 1/2 x 10 3/8
CONST EZP_ENV_12 = 22                '  Envelope #12 4 \276 x 11
CONST EZP_ENV_14 = 23                '  Envelope #14 5 x 11 1/2
CONST EZP_ENV_DL = 27                '  Envelope DL 110 x 220mm
CONST EZP_ENV_C5 = 28                '  Envelope C5 162 x 229 mm
CONST EZP_ENV_C3 = 29                '  Envelope C3  324 x 458 mm
CONST EZP_ENV_C4 = 30                '  Envelope C4  229 x 324 mm
CONST EZP_ENV_C6 = 31                '  Envelope C6  114 x 162 mm
CONST EZP_ENV_C65 = 32               '  Envelope C65 114 x 229 mm
CONST EZP_ENV_B4 = 33                '  Envelope B4  250 x 353 mm
CONST EZP_ENV_B5 = 34                '  Envelope B5  176 x 250 mm
CONST EZP_ENV_B6 = 35                '  Envelope B6  176 x 125 mm

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Bitmap Constants
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'const EZP_NOTRANS        =  12345678
'const EZP_UPPERLEFT      =  23456789


'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Text alignment constants for EZP_DocText in the style for multi-line text
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
const EZP_LEFT           =  0
const EZP_CENTER         =  1
const EZP_RIGHT          =  2

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Grid Positioning
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
const EZP_GRIDNEW        =  0
const EZP_GRIDMOD        =  1
const EZP_GRIDMARGINS    =  2

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Grid Positioning
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
const EZP_t        =  0
const EZP_l        =  1
const EZP_b        =  2
const EZP_r        =  3

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Font Styles Contants
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
const EZP_NORMAL      = 0
const EZP_BOLD        = 1
const EZP_ITALIC      = 2
const EZP_UNDER       = 4
const EZP_STRIKE      = 8

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  
'  EZP Document Block Contants  (Internal)
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Const EZP_TEXT  = 1
Const EZP_RECT  = 2
Const EZP_LINE  = 3
Const EZP_ELLP  = 4
Const EZP_FONT  = 5
Const EZP_NWPG  = 6
Const EZP_PGNM  = 7
Const EZP_BKMK  = 8
Const EZP_ENDD  = 9
Const EZP_NWDC  = 10
Const EZP_IMGA  = 11
Const EZP_IMGR  = 13
Const EZP_HEAD  = 14
Const EZP_FOOT  = 15
Const EZP_TXTX  = 16
Const EZP_TXTR  = 17
Const EZP_ARC1  = 18
Const EZP_PIE1  = 19
Const EZP_NAME  = 20
Const EZP_DATE  = 21
Const EZP_GRID  = 22    
Const EZP_POLY  = 23
Const EZP_TABL  = 24
Const EZP_SHAP  = 25  'Arrow, Balloon, Star
Const EZP_SMBL  = 26  'Triangles, Rect, Diamond    

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Polygon Pre-Define Shapes
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

const EZP_SHAPE_ARROW   = 1
const EZP_SHAPE_STAR    = 2
const EZP_SHAPE_BALLOON = 3
'const EZP_SHAPE_OTHER   = 4

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Symbol Shapes that are centered on a point
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

const EZP_SYMBOL_RECT       = 1
const EZP_SYMBOL_DIAMOND    = 2
const EZP_SYMBOL_TRIUP      = 3  'Triangle pointing up
const EZP_SYMBOL_TRIDOWN    = 4  'Triangle pointing down
const EZP_SYMBOL_TRILEFT    = 5  'Triangle pointing left
const EZP_SYMBOL_TRIRIGHT   = 6  'Triangle pointing right
const EZP_SYMBOL_CIRCLE     = 7
const EZP_SYMBOL_STAR       = 8  

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Pre-Defined Color Contants
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 Const EZP_WHITE      = BGR(255,255,255) ' Default White
 Const EZP_BLACK      = BGR(0,0,0) 'Black
 Const EZP_LTRED      = BGR(250,128,114) 'Light Red Salmon
 Const EZP_GREEN      = BGR(0,128,0) 'Green
 Const EZP_LTGREEN    = BGR(0,255,127) 'Light Green
 Const EZP_BLUE       = BGR(0,0,128) 'Blue
 Const EZP_LTBLUE     = BGR(175,238,238) 'Light Blue
 Const EZP_MAGENTA    = BGR(255,0,255) 'Magenta
 Const EZP_BEIGE      = BGR(139,0,139) 'Dark white
 Const EZP_CYAN       = BGR(0,255,255) 'Cyan
 Const EZP_AQUA       = BGR(127,255,212) 'Light Cyan Aquamarine
 Const EZP_RED        = BGR(255,0,0) 'Red
 Const EZP_BROWN      = BGR(244,164,96) 'Brown
 Const EZP_YELLOW     = BGR(255,255,0) 'Yellow
 Const EZP_ORANGE     = BGR(255,127,80) 'Orange
 Const EZP_GAINSBORO  = &HDCDCDC
 Const EZP_LIGHTGRAY  = &HD3D3D3
 Const EZP_SILVER     = &HC0C0C0
 Const EZP_DARKGRAY   = &HA9A9A9
 Const EZP_GRAY       = &H808080
 Const EZP_DIMGRAY    = &H696969
 Const EZP_LIGHTSLATEGRAY = &H998877 
 Const EZP_SLATEGRAY  = &H908070 
 Const EZP_DARKSLATEGRAY  = &H4F4F2F
 
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Pen Styles Contants
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
const EZP_PSOLID      = PS_SOLID 
const EZP_PDASH       = PS_DASH 
const EZP_PDOT        = PS_DOT 
const EZP_PDASHDOT    = PS_DASHDOT 
const EZP_PDASHDOTDOT = PS_DASHDOTDOT 

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Brush Styles Contants
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
const EZP_BHATCHED    = BS_HATCHED 
const EZP_BHOLLOW     = BS_HOLLOW
const EZP_BSOLID      = BS_SOLID 

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Arrow Styles Contants
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
const EZP_ARROWNONE   = 0
const EZP_ARROWEND    = 1
const EZP_ARROWSTART  = 2
const EZP_ARROWBOTH   = 3
const EZP_ARROWAUTO   = 4

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Control Words Contants
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
const EZP_CLOSE       = 0 
const EZP_PRINT       = 1
const EZP_FITWIN      = 2
const EZP_FITWIDTH    = 3

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Header and Footer Contants
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 

const EZP_PAGENUMLEFT   = 1
const EZP_PAGENUMRIGHT  = 2
const EZP_DATELEFT      = 4
const EZP_DATERIGHT     = 8


'Document Header Block
Type DocHeader
  DIM MyAction    as LONG
  Dim MyVersion   as long   'MyVersion = MAKELONG(EZP_VERSION,EZP.InchesOrCm)  ' shares with CM INCHES toggle
  Dim MyPages     as LONG
  Dim MyBookmarks as Long
  Dim MyPaper     as LONG
  Dim MyBin       as LONG
  Dim MyOrient    as LONG
  dim MyDuPlex    as Long
END TYPE

'Text, Rect, Line, Ellipse, Font, NewPage, PageNumber, End
Type DocCommand
  dim MyAction  as LONG
  Dim MyXPos    as SINGLE
  Dim MyYPos    as SINGLE
  Dim MyXPos2   as SINGLE  'for rect
  Dim MyYPos2   as SINGLE  'for rect
  dim MyWidth   as single  'for line
  dim MyColor1  as INTEGER 'Font and Line border Color
  dim MyColor2  as INTEGER  'Rect and Ellipse Fill Color
  Dim MyStyle   as LONG   'Font and End Options
  Dim MySize    as LONG   'Font and New Page Position
  Dim MyAngle   as long   'Font
  Dim TextLen   as Long   'Text and Font
END TYPE

type DocMargins
  dim nTop as Single
  Dim nLeft as Single
  dim nBottom as Single
  Dim nRight as Single
END TYPE

type PolyPoints
  dim x as Single
  dim y as Single  
END TYPE

Type TableCell
   dim h as Long 'Height Adjustment in Pixels stored in Column 0
   Dim w as long 'Width  Adjustment in Pixels stored in Row 0
   dim MyText as wstring * 100
   Dim FontStyle as Long
   Dim MyCellColor as Integer
   Dim MyJustify as Long
END TYPE

'Global block
type EZPGlobals
  Dim PrinterName as WString * 255
  Dim DialogName as WString * 255
  dim FileName as Wstring * 255
  dim FileHandle as HANDLE
  dim FileSize as Long
  dim ControlWords(5) as Wstring * 20
  dim HScrollPos as Long
  dim VScrollPos as Long
  
  dim InchesOrCm as LONG
  Dim SorP as long
  dim MyOrient as Long
  Dim MyBin as Long
  Dim MyVersion as long
  Dim MyPaper as Long
  dim MyDuplex as Long
  dim Papers(EZP_MAX_PAPERS,1) as single
  dim EndStyle as Long
  
  dim PrintDlgFreedom as Long
  dim PrintNumCopies as Long
  dim PrintNumPageRanges as Long
  dim PrintPageRanges(10) as PRINTPAGERANGE  'DWORD nFromPage;
                                             'DWORD nToPage;
  
  dim x as Long      'EZP.x = EZP.CanvasRight - wRect.Right - 16
  dim y as Long      'EZP.y = EZP.CanvasBottom - wRect.Bottom - EZP_DIAGVIEWTOP - 16
  
  dim MyGraphics(EZP_MAX_GRAPHICS) as string
  dim NumGraphics as LONG
  
  dim Header as wstring * 255
  Dim Footer as WString * 255
  Dim HeaderFooterDate as WSTRING * 32
  dim HeaderFooterShowToggle as LONG
  Dim MarginsShowToggle as LONG
  Dim GridShowToggle as LONG
  dim UserMarginsShowToggle as Long
  Dim MyGridCols(EZP_MAXNUMCOLS) as SINGLE
  Dim MyGridRows(EZP_MAXNUMROWS) as SINGLE

  dim ScreenPPIx as single
  Dim ScreenPPIy as single
  Dim ScreenSizex as single
  dim ScreenSizey as single
  Dim ScreenResx as single
  dim ScreenResy as single
  Dim ScreenMargins as DocMargins
  dim Zoom as long
  dim Zoomfactor as Single
  
  Dim CanvasRight as Long
  dim CanvasBottom as long
  
  'Font current definition which will be used be Angle Font
  Dim ScreenFont as hFONT
  Dim ScreenFontName as WString * 64
  Dim ScreenFontStyle as Long
  dim ScreenFontSize as double
  Dim ScreenFontAngle as integer
  Dim ScreenFontColor as integer
  
  dim NumPages as LONG
  Dim BookmarkCount as Long
  Dim CurrentPage as Long
  dim PagePtrs(EZP_MAX_PAGES) as Long
  Dim Bookmarks(EZP_MAX_BOOKMARKS) as WString * 50
  
  dim token AS ULONG_PTR
  dim StartupInput AS GdiplusStartupInput
  
  Dim TextPtrLen as LONG
  Dim TextPtrPos as LONG
  dim TextPtrBreakChar as Wstring * 3
  dim TextPtrBreakOptions as long
  
  'These will be released with EZP_ResetGlobal
  Dim TextPtr as WString ptr 
  dim hDC as hDC
  dim hbit as hGDIOBJ
  dim memDC as hDC
  Dim RClickMenu as hMenu
END TYPE

Dim Shared EZP as EZPGlobals
'Initialize the zoom setting


declare function        WinMain     ( byval hInstance as HINSTANCE, _
                                      byval hPrevInstance as HINSTANCE, _
                                      byval szCmdLine as zstring ptr, _
                                      byval iCmdShow as integer ) as integer
                                  
                                  
	end WinMain( GetModuleHandle( null ), null, Command( ), SW_NORMAL )
   
declare function EZP_NewDialog( byval hParent as HWND, byval lpDialogProc as DLGPROC,hInstance as hInstance,cx as long,cy as long) as INT_PTR   
declare FUNCTION EZP_DlgProc (BYVAL hDlg AS INT_PTR, BYVAL wMsg AS Uint, BYVAL wParam AS wParam, BYVAL lParam AS lParam) AS Integer
declare sub EZP_ResetGlobals(MyOptions as Long)
declare FUNCTION EZP_DocFont(byval hFile as HANDLE,byval  MyStyle as long,byval MySize as long,byval  MyColor as INTEGER, byref MyFontName as wstring Ptr) as SINGLE
declare function EZP_ReadCurrentPage(byval hFile as HANDLE,SorP as Long=0) as Long
declare FUNCTION AfxGdipPrintHBITMAP (byval MyDC as hDC,BYVAL hbmp AS HBITMAP, BYVAL bStretch AS BOOLEAN = FALSE, BYVAL nStretchMode AS LONG = InterpolationModeHighQualityBicubic) AS LONG

'************************************************************************
' WinWPrint - James Klutho
'
' Based on WinXPrint - by Paul Squires and others      
'
' --- CHANGES ---
' Change to wide string and highly modified
'************************************************************************



'******** DEFINE PRINTER OBJECT ********
Type WinWPrint
   Private:
      _PrinterDC     As HDC
      _PrinterName   As WString * 100
      _Printers      As WString * MAX_PATH  'Delimited with |
      _DOCTitle      As WString * 255
      _DI            As DOCINFOW
      _hPrinter      as HANDLE
      _pDev          As LPDEVMODEW
      _hDev          as HANDLE
      _Orient        as LONG
      _Tray          as LONG
      _Paper         as LONG
      _Duplex        as LONG
      _pPageRanges   as LPPRINTPAGERANGE
      _Copies        as Long
      _NumPageRange as Long
     
   Public:
      Declare Function ShowPageSetup() As Long
      Declare Function ShowPrinterDialog(byref WResult as WSTRING PTR,byval hideflags as Long = 0) As LONG
      Declare Function ShowPrinterDialogEx(byref WResult as WSTRING PTR,byval hideflags as Long = 0,byval prefOptions as long = 0) As LONG
      declare Function IsPrinter(byref TestPrinter as Wstring) as LONG
      
      Declare SUB      SetPrinter(byval lOrientation as LONG = 0,byval lBin as LONG=0,byval lPaper as long=0,byval lDuplex as LONG=0)
      Declare Sub      ChangePrinter(byref pwPrinterName As WString Ptr)
      Declare Sub      DocStart(byref DOCTitle As WString = "Untitled")
      Declare Sub      PageStart()
      Declare Sub      PageEnd()
      Declare Sub      DocEnd()
      Declare Sub      DocAbort()
      Declare SUB      EnumPrinters(byref MyOutwStr as WString PTR)
     
      Declare Property PrinterDC() As HDC
      Declare Property PrinterName() As WString PTR
      Declare Property Printers() As WString PTR
      Declare Property PrinterName(byref PrinterName As WString )     
      Declare Property PageWidth() As Long
      Declare Property PageHeight() As Long
      Declare Property MarginWidth() As Long
      Declare Property MarginHeight() As Long
      Declare Property PaperWidth() As Long
      Declare Property PaperHeight() As Long
      Declare Property PPIx() As single
      Declare Property PPIy() As single
      Declare Property nCopies() As Long
      Declare Property NumPageRanges() As Long
      Declare Property PageRangePtr() As LPPRINTPAGERANGE

      Declare Constructor()
      DECLARE DESTRUCTOR()
End Type

DESTRUCTOR WinWPrint
   If _PrinterDC Then DeleteDC _PrinterDC  : _PrinterDC = 0
   if _hPrinter <> 0 then ClosePrinter(_hPrinter)  : _hPrinter = 0
   if _pDev <> 0 then GlobalUnLock(_hDev)  : _pDev = 0
   if _hDev <> 0 then  GlobalFree _hDev : _hDev = 0
   if _pPageRanges <> 0 then deallocate _pPageRanges
End DESTRUCTOR

Constructor WinWPrint()
   ' Attach the Windows default printer to the class
   Dim wPrinterName As WString * MAX_PATH
   Dim n As Long = MAX_PATH
   
   this.EnumPrinters(@_Printers)
   
   GetDefaultPrinterW(@wPrinterName,@n)
   
   'print wPrinterName
    _pPageRanges = CAllocate(10 * sizeof(tagPRINTPAGERANGE))
   
   if len(wPrinterName) < 1 THEN
          MessageBox GetForegroundWindow, "A default printer is not available." & CHR$(0), _
                              "PRINT CONTROL MESSAGE" & CHR$(0),  MB_OK
      else
        this.ChangePrinter(@wPrinterName)
   END IF 
End Constructor

Property WinWPrint.PrinterDC() As HDC
   Property = _PrinterDC
End Property

Property WinWPrint.PrinterName() As WString PTR
   Property = @_PrinterName
End Property

Property WinWPrint.Printers() As WString PTR
   Property = @_Printers
End Property

Property WinWPrint.PrinterName(byref wPrinterName As WString )     
   If Len(wPrinterName) > 0 Then
      this.ChangePrinter(@wPrinterName)
   End If
End Property
       
Property WinWPrint.PageWidth() As Long
   Property = GetDeviceCaps(_PrinterDC,HORZRES)
End Property

Property WinWPrint.PageHeight() As Long
   Property = GetDeviceCaps(_PrinterDC, VERTRES)
End Property

Property WinWPrint.MarginWidth() As Long
   Property = GetDeviceCaps(_PrinterDC,PHYSICALOFFSETX)
End Property

Property WinWPrint.MarginHeight() As Long
   Property = GetDeviceCaps(_PrinterDC, PHYSICALOFFSETY)
End Property

Property WinWPrint.nCopies() As Long
   Property = _Copies
End Property

Property WinWPrint.NumPageRanges() As Long
   Property = _NumPageRange
End Property
         
Property WinWPrint.PageRangePtr() As LPPRINTPAGERANGE
   Property =  _pPageRanges
End Property

Property WinWPrint.PaperWidth() As Long
   Property = GetDeviceCaps(_PrinterDC,PHYSICALWIDTH)
End Property

Property WinWPrint.PaperHeight() As Long
   Property = GetDeviceCaps(_PrinterDC, PHYSICALHEIGHT)
End Property

Property WinWPrint.PPIx() As single
   Property = GetDeviceCaps(_PrinterDC,LOGPIXELSX)
End Property

Property WinWPrint.PPIy() As single
   Property = GetDeviceCaps(_PrinterDC, LOGPIXELSY)
End Property

Function WinWPrint.IsPrinter(byref TestPrinter as WString) as LONG
  Dim MyPrinter as WString * 100
  MyPrinter = TestPrinter
  MyPrinter = "|" & MyPrinter & "|"
  Function = Instr(_Printers,MyPrinter)   
END FUNCTION

sub WinWPrint.EnumPrinters(byref MyOutWStr as Wstring Ptr)
 
   DIM counter AS LONG
   DIM MyBytesNeeded AS LONG
   DIM MyStructuresNeeded AS LONG
   DIM MyReturned AS LONG
   DIM MyFlags AS LONG
   DIM MyResult AS LONG
   dim MywStrPtr as WString Ptr
   
   Dim MyPrinters AS WSTRING * MAX_PATH
   DIM rPrinters() AS PRINTER_INFO_2W

'or PRINTER_ENUM_NAME  or PRINTER_ENUM_SHARED  
  ' MyFlags = PRINTER_ENUM_LOCAL  or PRINTER_ENUM_CONNECTIONS  'PRINTER_ENUM_NAME
   MyFlags = PRINTER_ENUM_LOCAL  or PRINTER_ENUM_CONNECTIONS  
   
   EnumPrintersW MyFlags, BYVAL NULL, 2, BYVAL NULL, 0, @MyBytesNeeded, @MyReturned

   MyStructuresNeeded = MyBytesNeeded \ SIZEOF(rPrinters(0)) + 1
   REDIM rPrinters(0 TO MyStructuresNeeded) AS PRINTER_INFO_2W
   
   MyResult = EnumPrintersW(MyFlags, "", 2, cast(LPBYTE,@rPrinters(1)), LEN(rPrinters(0)) * UBOUND(rPrinters), @MyBytesNeeded, @MyReturned)
   'print MyResult
    'rint GetLastError

   MyPrinters = "|" 
    IF MyResult THEN 
      FOR counter = 1 TO MyReturned
        MywStrPtr = rPrinters(counter).pPrinterName 
        MyPrinters  = MyPrinters & *MywStrPtr & "|"
      next counter   
    end if
    
    'print MyPrinters
    
    *MyOutWStr = MyPrinters
End sub

Sub WinWPrint.SetPrinter(byval lOrientation as LONG = 0,byval lBin as LONG=0,byval lPaper as long=0,byval lDuplex as LONG=0)
  _Orient = lOrientation
  _Tray = lBin
  _Paper = lPaper
  _Duplex = lDuplex 
  this.ChangePrinter(@_PrinterName)
END SUB

Sub WinWPrint.ChangePrinter(byref pwPrinterNameIn As WString Ptr)
   Dim devLen as LONG
   Dim MyTestPrinterName as Wstring * 100
   Dim MyDefaultPrinterName as Wstring * 100
   MyTestPrinterName = *pwPrinterNameIn
   Dim pwPrinterName as WSTRING ptr
   dim n as LONG = 100
   
   pwPrinterName = pwPrinterNameIn
   GetDefaultPrinterW(@MyDefaultPrinterName,@n)
   
   'I can't get the remote computers to enumerate so if the pwPrinterNameIn = default printer then bypass
   'Will let printer names starting with \\ pass for now.  I assume they are remote network printers
   if this.IsPrinter(MytestPrinterName) = 0 and (MyTestPrinterName <> MyDefaultPrinterName) and Left(MyTestPrinterName,2) <> "\\"THEN   
      MessageBox GetForegroundWindow, "The printer that you specified is not available." _
      "  Use the Default Printer" & CHR$(0), _
      "PRINT CONTROL MESSAGE" & CHR$(0),  MB_OK
      GetDefaultPrinterW(@MyTestPrinterName,@n)
      pwPrinterName = varptr(MyTestPrinterName)
   END IF
   
   if _hPrinter <> 0 then ClosePrinter(_hPrinter)  : _hPrinter = 0
   If _PrinterDC Then DeleteDC _PrinterDC  : _PrinterDC = 0
   if _pDev <> 0 then GlobalUnLock(_hDev)  : _pDev = 0
   if _hDev <> 0 then  GlobalFree _hDev : _hDev = 0
   
   IF OpenPrinterW(pwPrinterName,@_hPrinter,NULL) THEN
      'DevModeW may be bigger than stucture so test
      devLen = DocumentPropertiesW(GetForegroundWindow(),_hPrinter,pwPrinterName,NULL,NULL,0)
      _hDev = GlobalAlloc(GMEM_MOVEABLE Or GMEM_ZEROINIT, devLen)
      _pDev = GlobalLock(_hDev)
       DocumentPropertiesW(GetForegroundWindow(),_hPrinter,pwPrinterName,_pDev,NULL,DM_OUT_BUFFER)
         
          if _Orient <> 0 THEN
              _pDev->dmFields = _pDev->dmFields or DM_ORIENTATION
              _pDev->dmOrientation = DMORIENT_LANDSCAPE
            else   'EZP_PORTRAIT       =  0
              _pDev->dmFields = _pDev->dmFields or DM_ORIENTATION
              _pDev->dmOrientation = DMORIENT_PORTRAIT
          END IF
            
          if _Tray <> 0 THEN
              _pDev->dmFields = _pDev->dmFields or DM_DEFAULTSOURCE
              _pDev->dmDefaultSource = _Tray
          END IF
         
          if _Paper <> 0 THEN
              _pDev->dmFields = _pDev->dmFields or DM_PAPERSIZE
              _pDev->dmPaperSize = _Paper
              'print _Paper
              'Print "HERE"
          end if
        
          if _Duplex <> 0 THEN
             if _Duplex = EZP_DUPLEX_VERTICAL then
                 _pDev->dmFields = _pDev->dmFields or DM_DUPLEX
                 _pDev->dmDuplex = DMDUP_VERTICAL
              else
                 _pDev->dmFields = _pDev->dmFields or DM_DUPLEX
                 _pDev->dmDuplex = DMDUP_HORIZONTAL 
             end if     
          END IF

      DocumentPropertiesW(GetForegroundWindow(),_hPrinter,pwPrinterName,_pDev,_pDev,DM_OUT_BUFFER or DM_IN_BUFFER)
   
      _PrinterDC   = CreateDCW( Null, pwPrinterName, Null, _pDev )
      
      _PrinterName = *pwPrinterName
      
     else
       MessageBox GetForegroundWindow, "Printer name is not Valid" & CHR$(0), _
                              "PRINT CONTROL MESSAGE" & CHR$(0),  MB_OK  
   END IF
End Sub

Function WinWPrint.ShowPrinterDialog(byref WResult as Wstring PTR,byval hideflags as Long = 0) As Long
   Dim wPrinterName As WString * MAX_PATH
   dim wStrPtr as wString ptr
   Dim n         As Long = MAX_PATH
   Dim PDlg      As PrintDlgW
   dim pDevNames as DEVNAMES PTR
   
   ZeroMemory(@PDlg, Len(PrintDlgW)) 'sizeof(PDlg))
   PDlg.lStructSize = Len(PrintDlgW)
   PDlg.hWndOwner   = GetForegroundWindow()
   pDlg.hDevMode = NULL
   pDlg.hDevNames = NULL
   
   
   if hideflags =0 THEN
     pDlg.Flags = PD_HIDEPRINTTOFILE or PD_PRINTSETUP or PD_ALLPAGES or PD_NOSELECTION or PD_NOPAGENUMS or PD_RETURNDC
   END IF
      pDlg.Flags = PD_RETURNDC
   ' Call the print dialog up and let the user make changes
   If PrintDlgW(@PDlg) <> 0 Then
       pDevNames = GlobalLock(PDlg.hDevNames)
       wStrPtr = cast(wString ptr,cast(byte ptr,pDevNames) + (pDevNames->wDeviceOffset) * 2)  ' times 2 due to being a wString
       wPrinterName = *wStrPtr
       GlobalUnLock(PDlg.hDevNames)
      else
        function = 1
       *WResult = "" 
       goto bailprint    
   EndIf
   
    
   If Len(wPrinterName) > 0 Then
       this.ChangePrinter(@wPrinterName)
       *WResult = wPrinterName
       Function = 0
     else
       function = 1
       *WResult = ""   
   End If
   
bailprint:   
   if (pDlg.hDevMode <> NULL) then GlobalFree(pDlg.hDevMode) 
   if (pDlg.hDevNames <> NULL) then GlobalFree(pDlg.hDevNames) 
   if (pDlg.hDC <> NULL) then DeleteDC(pDlg.hDC)
   
End Function

Function WinWPrint.ShowPrinterDialogEx(byref WResult as Wstring PTR,byval hideflags as Long = 0,byval prefOptions as long = 0) As Long
   Dim wPrinterName As WString * MAX_PATH
   dim wStrPtr as wString ptr
   Dim n         As Long = MAX_PATH
   dim pDevNames as DEVNAMES PTR
   
   dim hres as HRESULT 
   
   dim as PRINTDLGEXW   mypdlg
   dim as LPPRINTDLGEXW mypdlgptr = @mypdlg
   
 
   ZeroMemory(@mypdlg, Len(PRINTDLGEXW)) 'sizeof(PDlg))
   ZeroMemory(_pPageRanges, 10 * sizeof(tagPRINTPAGERANGE)) 'Make sure the page ranges are zeroed
                                                           
   
'   if hideflags =0 THEN
'     mypdlg.Flags = PD_HIDEPRINTTOFILE or PD_PRINTSETUP or PD_ALLPAGES or PD_NOSELECTION or PD_NOPAGENUMS or PD_RETURNDC
'    else
'     mypdlg.Flags = PD_RETURNDC or PD_COLLATE
'   END IF
   
   ' mypdlg.Flags = PD_RETURNDC or PD_COLLATE 
   
   with mypdlg
    .lStructSize = Len(PRINTDLGEXW)
    .hwndOwner = GetForegroundWindow()
    if prefOptions = 0 then .hDevMode = NULL else .hDevMode = _pDev
    .hDevNames = NULL
    .hDC = NULL
    .Flags = PD_RETURNDC or PD_COLLATE or PD_DISABLEPRINTTOFILE
    .Flags2 = 0
    .ExclusionFlags = 0
    .nPageRanges = 0
    .nMaxPageRanges = 10
    .lpPageRanges = _pPageRanges
    .nMinPage = 1
    .nMaxPage = 1000
    .nCopies = 1
    .hInstance = 0
    .lpPrintTemplateName = NULL
    .lpCallback = NULL
    .nPropertyPages = 0
    .lphPropertyPages = NULL
    .nStartPage = &HFFFFFFFF'START_PAGE_GENERAL
    .dwResultAction = 0
   end with
'
 
 
   hres=PrintDlgExW(cast(PRINTDLGEXW ptr,mypdlgptr))
   
   'print "Here" ; hres, mypdlg.dwResultAction 

   _Copies = mypdlg.nCopies
   _NumPageRange = mypdlg.nPageRanges
    
   ' Call the print dialog up and let the user make changes
   If mypdlg.dwResultAction <> 0 Then  '0 if cancelled 1 is print
       pDevNames = GlobalLock(mypdlg.hDevNames)
       wStrPtr = cast(wString ptr,cast(byte ptr,pDevNames) + (pDevNames->wDeviceOffset) * 2)  ' times 2 due to being a wString
       wPrinterName = *wStrPtr
       GlobalUnLock(mypdlg.hDevNames)
      else
        function = 1
       *WResult = "" 
       goto bailprintex    
   EndIf
'       
   If Len(wPrinterName) > 0 Then
       this.ChangePrinter(@wPrinterName)
       *WResult = wPrinterName
       Function = 0
     else
       function = 1
       *WResult = ""   
   End If
'   
bailprintex:   
   if (mypdlg.hDevMode <> NULL) and (prefOptions <> 0) then GlobalFree(mypdlg.hDevMode) 
   if (mypdlg.hDevNames <> NULL) then GlobalFree(mypdlg.hDevNames) 
   if (mypdlg.hDC <> NULL) then DeleteDC(mypdlg.hDC)
   
End Function

Function WinWPrint.ShowPageSetup() As Long
   Dim PSD As PageSetupDlg
'Skeleton Page Setup
   With PSD
      .lStructSize = Len(PSD)
      .hWndOwner   = GetForegroundWindow()     
      .hInstance   = Null   
      .flags       = PSD_DISABLEPRINTER
   End With
   
   'Show the pagesetup dialog
   If PageSetupDlg(@PSD) Then
     Return 0
   Else
     Return -1
   End If
End Function

Sub WinWPrint.DocStart(byref DOCTitle As wString = "Untitled")
   _DOCTitle       = DOCTitle
   _DI.cbSize      = Len(DOCINFO)
   _DI.lpszDocName = @_DOCTitle
   StartDocW _PrinterDC, @_DI
End Sub

Sub WinWPrint.DocEnd()
   EndDoc _PrinterDC
End Sub

Sub WinWPrint.PageStart()
   StartPage _PrinterDC
End Sub

Sub WinWPrint.PageEnd()
   EndPage _PrinterDC
End Sub

Sub WinWPrint.DocAbort()
   AbortDoc _PrinterDC
End Sub

Dim SHARED XPrint as WinWPrint
'************************************************************************
' WinWPrint - End
'************************************************************************

' ========================================================================================
' Sets the current process as dots per inch (dpi) aware.
' Note: SetProcessDPIAware is subject to a possible race condition if a DLL caches dpi
' settings during initialization. For this reason, it is recommended that dpi-aware be set
' through the application (.exe) manifest rather than by calling SetProcessDPIAware.
' Return value: TRUE on success; FALSE on failure. By Jose Roca
' ========================================================================================
FUNCTION EZP_SetProcessDPIAware () AS BOOLEAN
   DIM AS ANY PTR pLib = DyLibLoad("user32.dll")
   IF pLib = 0 THEN EXIT FUNCTION
   DIM pProc AS FUNCTION () AS LONG
   pProc = DyLibSymbol(pLib, "SetProcessDPIAware")
   IF pProc = 0 THEN EXIT FUNCTION
   FUNCTION = pProc()
   DyLibFree(pLib)
END FUNCTION

FUNCTION EZP_IsProcessDPIAware () AS BOOLEAN
   DIM AS ANY PTR pLib = DyLibLoad("user32.dll")
   IF pLib = 0 THEN EXIT FUNCTION
   DIM pProc AS FUNCTION () AS LONG
   pProc = DyLibSymbol(pLib, "IsProcessDPIAware")
   IF pProc = 0 THEN EXIT FUNCTION
   FUNCTION = pProc()
   DyLibFree(pLib)
END FUNCTION
' ========================================================================================


'const EZP_CLOSE       = 0 
'const EZP_PRINT       = 1
'const EZP_FITWIN      = 2
'const EZP_FITWIDTH    = 3


Function EZP_ROUND(byval x as Single) as SINGLE
   dim MyResult as SINGLE
   dim MyFrac as SINGLE
   MyResult = Fix(x)

   if FRAC(x) > 0.499999999 then MyResult = MyResult + 1

   Function = MyResult
END FUNCTION


     



sub EZP_ChangeWords(byref MyPrint as WString, byref MyClose as WString, byref MyFitWindow as WString, byref MyFitWidth as WString)
  EZP.ControlWords(EZP_PRINT) = MyPrint
  EZP.ControlWords(EZP_CLOSE) = MyClose
  EZP.ControlWords(EZP_FITWIN) = MyFitWindow
  EZP.ControlWords(EZP_FITWIDTH) = MyFitWidth 
END SUB

sub EZP_SetHeaderFooterDate(byref MyDate as WString = "")
  if len(MyDate) = 0 THEN
      EZP.HeaderFooterDate = wstr(date)
     else
      EZP.HeaderFooterDate = MyDate  
  END IF
    
END SUB



'************************************************************************
' Text Width Routines
' EZP_GetTextWidth returns inches of a text string
'************************************************************************

Function EZP_G(byval row as long, byval col as LONG, pt as long = 0) as SINGLE
'return centimeters if EZP.InchesOrCm <> 0
   
'const EZP_t        =  0
'const EZP_l        =  1
'const EZP_b        =  2
'const EZP_r        =  3

'print "EZP_G  ";EZP.MyGridCols(4)
   
   
  dim p as SINGLE,cm as SINGLE
  dim ry as single, rx as SINGLE
  dim xx as LONG, yy as LONG
  dim mycol as long, myrow as LONG
  dim t as single, l as single, b as single,r as SINGLE
  

  if col > 0 and col <= EZP.MyGridCols(0)THEN
     rx = 0
     for xx = 1 to col
        rx = rx + EZP.MyGridCols(xx)  
     NEXT xx
     r = rx
     l = r - EZP.MyGridCols(xx-1)     
  END IF
  
  if row > 0 and row <= EZP.MyGridRows(0)THEN
     ry = 0
     for yy = 1 to row
        ry = ry + EZP.MyGridRows(yy)  
     NEXT yy
     b = ry
     t = b - EZP.MyGridRows(yy-1)
  END IF
  
  'print t,l,b,r
  
  select CASE pt
     case 0
        p=t
     case 1
        p=l
     case 2
        p=b
     case 3
        p=r      
  END SELECT
  'print "EZP_G";p
  'Need to always be in inches
  if EZP.InchesOrCm <> 0 THEN cm=2.53999 else cm = 1
  Function = p * cm
END FUNCTION





FUNCTION EZP_GetTextWidth(byref MyString AS Wstring, BYVAL iPPIX AS INTEGER, BYVAL MyDC AS hDC, _
                      BYVAL hMyFont AS hFont = EZP.ScreenFont) AS SINGLE

   DIM iPixels AS LONG
   DIM iReturn AS HGDIOBJ
   DIM rSize AS Size

   iReturn = SelectObject(MyDC, hMyFont)

   GetTextExtentPoint32W MyDC, Varptr(MyString), LEN(MyString), @rSize
   iPixels = rSize.cx
   SelectObject MyDC, iReturn

   '- Convert to inches
   FUNCTION = iPixels / iPPIX

END FUNCTION

Sub EZP_GetText(byref MyTextOut as Wstring PTR, BYVAL maxWidth as single)
   '
   '  This will return the longest string of text
   '  that will fit in the specified width
   '  maxWidth in inches
   '
   DIM iCount AS LONG
   DIM iLoop AS LONG
   DIM lastLine AS WSTRING * 2000
   DIM checkLine AS WString * 2000
   DIM iFound AS LONG
   Dim hMyFont AS hFont
   dim MyDC AS hDC
   dim iPPIX AS INTEGER
   dim MyTextIn as Wstring * 2000
   DIM tlen as LONG
   'dim MyWidth as LONG
   Dim DeleteDCFlag as LONG
   dim hDesktop as hWnd
   
   tlen = Min(EZP.TextPtrLen,2000)
   lastLine = ""
   checkLine = ""

   iCount = 0

   iFound = False
   
   if EZP.SorP = 0 THEN
       MyDC = EZP.memDC
    Else
       MyDC = xPrint.PrinterDC
   END IF

   if EZP.SorP = 0 THEN
     iPPIX = GetDeviceCaps(MyDC, LOGPIXELSX)
    else 
     IPPIX = XPrint.PPIx
   END IF
   
   MyTextIn = MID(*EZP.TextPtr,EZP.TextPtrPos,tlen)
   hMyFont =EZP.ScreenFont
   
   FOR iLoop = 1 TO LEN(MyTextIn)
      IF iLoop > 1999 THEN EXIT FOR

      IF (MID(MyTextIn, iLoop, 1) = CHR$(32)) AND (iLoop > 1) THEN

         checkLine = LEFT$(MyTextIn, iLoop - 1)
         IF EZP_GetTextWidth(checkLine,iPPIX,MyDC,hMyFont) <= MaxWidth THEN
            lastLine = checkLine
         ELSE
            iFound = True
            EXIT FOR
         END IF
      'ELSEIF (MID$(MyTextIn, iLoop, 1) = CHR$(13)) THEN
      ELSEIF (MID$(MyTextIn, iLoop, 1) = EZP.TextPtrBreakChar) THEN
         IF iLoop > 1 THEN
            checkLine = LEFT$(MyTextIn, iLoop - 1) + " "
            IF EZP_GetTextWidth(checkLine,iPPIX,MyDC,hMyFont) <= MaxWidth THEN
               lastLine = checkLine
            END IF
         ELSE
            lastLine = " "
         END IF
         iFound = true
         EXIT FOR

      END IF
   NEXT iLoop

   IF lastLine = "" THEN

      '- Chop off characters until we fit the line
      IF INSTR(TRIM$(MyTextIn), " ") > 1 THEN
         checkline = LEFT$(MyTextIn, INSTR(MyTextIn, " "))
      ELSE

         checkline = MyTextIn
      END IF

      DO WHILE LEN(checkline) > 1
         IF EZP_GetTextWidth(checkline, iPPIX,MyDC, hMyFont) <= MaxWidth THEN
            EXIT DO
         END IF
         checkline = LEFT$(checkline, LEN(checkline) - 1)
      LOOP
      lastLine = checkline
      IF LEFT$(lastLine, 1) = CHR$(10) THEN
         lastLine = MID$(lastLine, 2) + " "
      END IF

      *MyTextOut = lastline

   ELSE
      IF iFound = False THEN
         checkLine = MyTextIn
         IF EZP_GetTextWidth   (checkLine, iPPIX,MyDC, hMyFont ) <= MaxWidth THEN
            lastLine = checkLine
         END IF
      END IF

      IF LEFT$(lastLine, 1) = CHR$(10) THEN
         lastLine = MID$(lastLine, 2) + " "
      END IF

      *MyTextOut = lastLine

   END IF
   
   EZP.TextPtrPos = EZP.TextPtrPos + len(lastLine)
 
   
END Sub

'************************************************************************
' GDI Plus code is a modification of Jose Roca CWindows code - James Klutho
' Thanks for your work Jose
'************************************************************************
Function EZP_AddImage(Byref wszFileName as WSTRING) as LONG
   DIM token AS ULONG_PTR
   DIM StartupInput AS GdiplusStartupInput
   'DIM wszFileName AS WSTRING * 260
   DIM pBitmap AS GpBitmap PTR
   Dim hbmp as HBITMAP
   dim bm as BITMAP
   dim as DIBSECTION dibs
   'dim sout as STRING
   dim binfo as BITMAPINFOHEADER
   
   function = 0
   if EZP.NumGraphics >= EZP_MAX_GRAPHICS THEN exit function
  
   ' // Initialize GDI+
   StartupInput.GdiplusVersion = 1
   GdiplusStartup(@token, @StartupInput, NULL)

   GdipCreateBitmapFromFile(wszFileName, @pBitmap)
   if pBitmap <> 0 THEN
        EZP.NumGraphics = EZP.NumGraphics + 1   
        GdipCreateHBITMAPFromBitmap(pBitmap,@hbmp,0)
        GetObject( hbmp, sizeof(DIBSECTION), @dibs )
        'EZP.MyGrapics(0) is the scratch string to hold temp graphics
       ' dibs.dsBmih.biCompression = BI_RGB
        EZP.MyGraphics(0)= space(dibs.dsBmih.biSizeImage + dibs.dsBmih.biSize)  
        CopyMemory(strptr(EZP.MyGraphics(0)),varptr(dibs.dsBmih.biSize),dibs.dsBmih.biSize)
        CopyMemory(strptr(EZP.MyGraphics(0))+ dibs.dsBmih.biSize,dibs.dsBm.bmBits,dibs.dsBmih.biSizeImage)
        Function = EZP.NumGraphics
       
         
         ' // Cleanup
         DeleteObject hbmp
         GdipDisposeImage(cast(gpImage ptr,pBitmap))       
    END IF

   ' // Shutdown GDI+
   GdiplusShutdown token
 
END Function

Function EZP_GetImage(byval WhichImage as LONG) as HBitmap
    dim binfo as BITMAPINFOHEADER
    dim hbmp as hBITMAP
    
    Function = 0
    if WhichImage < 1 or WhichImage > EZP_MAX_GRAPHICS THEN Exit Function
       
    CopyMemory(@binfo,strptr(EZP.MyGraphics(WhichImage)),40)    
    hbmp = CreateBitmap( binfo.biWidth, binfo.biHeight,binfo.biPlanes,binfo.bibitcount, cast(byte ptr,strptr(EZP.MyGraphics(WhichImage)))+ binfo.biSize)
    function = hbmp
end function

SUB EZP_Star (byval MyDC as hDC,byval  Num_Points as Long,byval MySize as long,byval myx as long,byval myy as Long)

   'myy and myx are screen or printer point not inches are cm
   'the star is centered on myx myy
   
   dim x as long, y as long, z AS LONG
   dim cx AS SINGLE, cy AS SINGLE, dt AS SINGLE, th AS SINGLE
   Dim rc AS RECT
  
   REDIM w(0 TO 1) AS SINGLE
   REDIM h(0 TO 1) AS SINGLE
   REDIM rgPoints(1 TO NUM_POINTS) AS POINT
   

   
   rc.top = myy + MySize  : rc.bottom = myy - MySize
   rc.left = myx - MySize : rc.right = myx + MySize
   

   x = rc.Right - rc.Left
   y = rc.Bottom - rc.Top

   cx = myx 
   cy = myy 
   w(0) = x * 0.15! : w(1) = x * 0.5!
   h(0) = y * 0.15! : h(1) = y * 0.5!
   dt = 2 * pi / NUM_POINTS
   th = pi / 2!
   FOR z = 1 TO NUM_POINTS
      rgPoints(NUM_POINTS - z + 1).x = cx + w(z MOD 2) * COS(th)
      rgPoints(NUM_POINTS - z + 1).y = cy + h(z MOD 2) * SIN(th)
      th = th + dt
   NEXT
   
   Polygon(MyDC,@rgPoints(1), NUM_POINTS)
End sub


SUB EZP_DrawArrow (BYVAL hdc AS hDC,byval MyStyle as long,byval x as single,byval y as single,byval x2 as single,byval  y2 as single _
    ,byval MyColor2 as integer,byval  MySize as single = 2)

   DIM hStatus AS GpStatus
   DIM pGraphics AS GpGraphics PTR
   DIM pArrowPen AS GpPen PTR 
   DIM pMyArrowCap AS GpAdjustableArrowCap PTR
   DIM fillstate AS LONG
   DIM StartupInput AS GdiplusStartupInput
   DIM token AS ULONG_PTR
   dim capsize as single
   Dim r As UInteger , g As UInteger, b As UInteger, a As UInteger
   r = RGBA_R(MyColor2)
   g = RGBA_G(MyColor2)
   b = RGBA_B(MyColor2)
      
   Dim As UInteger col = RGBA(b, g, r, 255)
 
   capsize = MySize * 3
   
    ' // Initialize GDI+
   StartupInput.GdiplusVersion = 1
   GdiplusStartup(@token, @StartupInput, NULL)

   hStatus = GdipCreateFromHDC(hdc, @pGraphics)
   GdipSetSmoothingMode(pGraphics, SmoothingModeAntiAlias)
   'GdipSetPageUnit(pGraphics, UnitPixel)
  
   ' // Create an AdjustableArrowCap with the fill state set to TRUE.
   hStatus = GdipCreateAdjustableArrowCap(capsize, capsize, TRUE, @pMyArrowCap)
  
   ' // Set fill state to FALSE.
   'hStatus = GdipSetAdjustableArrowCapFillState(pMyArrowCap, TRUE)

   ' // Create a Pen, and assign pMyArrowCap as the end cap.
   hStatus = GdipCreatePen1(col, MySize, UnitWorld, @pArrowPen)
 
   
   Select CASE MyStyle
      case EZP_ARROWBOTH
          hStatus = GdipSetPenCustomEndCap(pArrowPen, cast(GpCustomLineCap ptr,pMyArrowCap))
          hStatus = GdipSetPenCustomStartCap(pArrowPen, cast(GpCustomLineCap ptr,pMyArrowCap))
      case EZP_ARROWSTART
          hStatus = GdipSetPenCustomStartCap(pArrowPen, cast(GpCustomLineCap ptr,pMyArrowCap))    
      case EZP_ARROWEND
          hStatus = GdipSetPenCustomEndCap(pArrowPen, cast(GpCustomLineCap ptr,pMyArrowCap))
      CASE ELSE
         'Nothing for now
   END SELECT
   ' // Draw a line using pArrowPen.
  ' GdipSetPageUnit(pGraphics, UnitPixel)
   hStatus = GdipDrawLineI(pGraphics, pArrowPen, x, y, x2, y2)

   ' // Cleanup
   IF pMyArrowCap THEN GdipDeleteCustomLineCap(casT(GpCustomLineCap ptr,pMyArrowCap))
   IF pArrowPen THEN GdipDeletePen(pArrowPen)
   IF pGraphics THEN GdipDeleteGraphics(pGraphics)
   IF pGraphics THEN GdipDeleteGraphics(pGraphics)

END SUB



FUNCTION EZP_DrawBitmap (BYVAL MyDC as hDC,BYVAL hbmp AS HBITMAP,byval MyAngle as single, BYVAL x AS SINGLE = 0, BYVAL y AS SINGLE = 0, _
   BYVAL nRight AS SINGLE = 0, BYVAL nBottom AS SINGLE = 0, byval AspectFlag as integer = 0) AS GpStatus

   DIM nStatus AS GpStatus, pGraphics AS GpGraphics PTR, pBitmap AS GpBitmap PTR
   DIM nWidth AS DWORD, nHeight AS DWORD
   DIM StartupInput AS GdiplusStartupInput
   DIM token AS ULONG_PTR
   Dim pixelColor As COLORREF
   dim TempDC1 as hDC
   dim TempDC2 as hDC
   dim hbit as HBITMAP
   dim version AS UINT32 = 1
   dim xTranslation as SINGLE,yTranslation as SINGLE
 
   
    ' // Initialize GDI+
   StartupInput.GdiplusVersion = version
   GdiplusStartup(@token, @StartupInput, NULL)
   
       nStatus = GdipCreateBitmapFromHBITMAP (hbmp, NULL, @pBitmap)
      
       GdipGetImageWidth(cast(GpImage PTR, pBitmap), @nWidth)
       GdipGetImageHeight(cast(GpImage PTR, pBitmap), @nHeight)
      
       'Full Picture
       if nRight = 0 and nBottom = 0 THEN
         nRight = nWidth   
         nBottom = nHeight   
       end if 
       
       'Force angle to zero because I can't get the rotatation around center to work
       MyAngle = 0
       
       if  AspectFlag <> 0 and MyAngle = 0 THEN
        if nRight = 0 and nBottom = 0 THEN
          'Nothing for now
         else
           nBottom = nBottom *(nHeight/nWidth)    
        end if  
       end if
       
      'Need the flip the bitmap  
      GdipImageRotateFlip(cast(any ptr,pBitmap),6)
       
      nStatus = GdipCreateFromHDC(MyDC, @pGraphics)

     ' By default the printers are set to 100 pixels per inch - change to unitpixel to match the screen
      GdipSetPageUnit(pGraphics, UnitPixel) '
     
      if MyAngle <> 0 THEN
        GdipTranslateWorldTransform(pGraphics, -(x + (nRight/2)), -(y + (nBottom/2)), MatrixOrderAppend)
'     
         GdipRotateWorldTransform(pGraphics, MyAngle, MatrixOrderAppend)
'         GdipScaleWorldTransform(pGraphics, nRight/nWidth, nBottom/nHeight,MatrixOrderAppend)
         GdipScaleWorldTransform(pGraphics, nRight/nWidth, nBottom/nHeight,MatrixOrderAppend) 
         GdipTranslateWorldTransform(pGraphics, x + (nRight/2), y + (nBottom/2), MatrixOrderAppend)
'        GdipTranslateWorldTransform(pGraphics, x + (nRight/2), y + (nBottom/2), MatrixOrderAppend)
'         GdipScaleWorldTransform(pGraphics, nRight/nWidth, nBottom/nHeight,MatrixOrderAppend) 
      END IF
    
      IF nStatus <> S_OK OR pGraphics = 0 THEN Print "Problem with Graphic DC" 
       if MyAngle <> 0 THEN
          'print nRight,nBottom 
           'nStatus = GdipDrawImage(pGraphics,CAST(GpImage PTR, pBitmap),-nRight-60,-nBottom-70)
           nStatus = GdipDrawImage(pGraphics,CAST(GpImage PTR, pBitmap),x,y)
        else
           nStatus = GdipDrawImageRectI(pGraphics,CAST(GpImage PTR, pBitmap),x,y,nRight,nBottom)
      end if
      IF nStatus <> S_OK OR pGraphics = 0 THEN Print "Problem with Printing"
       
      GdipResetWorldTransform(pGraphics)
       
   
     LExit:
       
     IF pBitmap THEN GdipDisposeImage(cast(GpImage PTR, pBitmap))
     IF pGraphics THEN GdipDeleteGraphics(pGraphics)
   
   ' // Shutdown GDI+
   GdiplusShutdown token
   
   RETURN nStatus

END FUNCTION

'*****************End GDIplus *******************************************************

Sub EZP_FillPaperArray()
    EZP.Papers(1,0)=8.5 :  EZP.Papers(1,1)=11
    EZP.Papers(2,0)=8.5 :  EZP.Papers(2,1)=11
    EZP.Papers(3,0)=11  :  EZP.Papers(3,1)=17
    EZP.Papers(4,0)=17  :  EZP.Papers(4,1)=11
    EZP.Papers(5,0)=8.5 :  EZP.Papers(5,1)=14
    EZP.Papers(6,0)=5.5 :  EZP.Papers(6,1)=8.5
    EZP.Papers(7,0)=7.5 :  EZP.Papers(7,1)=10
    'Metric
    EZP.Papers(8,0)=287  :  EZP.Papers(8,1)=420
    EZP.Papers(9,0)=210  :  EZP.Papers(9,1)=297
    EZP.Papers(10,0)=210 :  EZP.Papers(10,1)=297
    EZP.Papers(11,0)=148  :  EZP.Papers(11,1)=210
    EZP.Papers(12,0)=250 :  EZP.Papers(12,1)=354
    EZP.Papers(13,0)=182 :  EZP.Papers(13,1)=257
    EZP.Papers(14,0)=8.5 :  EZP.Papers(14,1)=13
    EZP.Papers(15,0)=215 :  EZP.Papers(15,1)=275
    EZP.Papers(16,0)=10 :  EZP.Papers(16,1)=14
    EZP.Papers(17,0)=11 :  EZP.Papers(17,1)=17
    EZP.Papers(18,0)=8.5 :  EZP.Papers(18,1)=11
    'Envelopes
    EZP.Papers(19,0)=3.875 :  EZP.Papers(19,1)=8.875
    EZP.Papers(20,0)=4.125 :  EZP.Papers(20,1)=9.5
    EZP.Papers(21,0)=4.5   :  EZP.Papers(21,1)=10.375
    EZP.Papers(22,0)=8.5   :  EZP.Papers(22,1)=11
    EZP.Papers(23,0)=4     :  EZP.Papers(23,1)=11.5
    EZP.Papers(24,0)=4.5   :  EZP.Papers(24,1)=10.375  'Not defined
    EZP.Papers(25,0)=4.5   :  EZP.Papers(25,1)=10.375  'Not defined
    EZP.Papers(26,0)=4.5   :  EZP.Papers(26,1)=10.375  'Not defined
    EZP.Papers(27,0)=110   :  EZP.Papers(27,1)=220
    EZP.Papers(28,0)=162   :  EZP.Papers(28,1)=229
    EZP.Papers(29,0)=324   :  EZP.Papers(29,1)=458
    EZP.Papers(30,0)=229   :  EZP.Papers(30,1)=324 
    EZP.Papers(31,0)=114   :  EZP.Papers(31,1)=162
    EZP.Papers(32,0)=114   :  EZP.Papers(32,1)=229
    EZP.Papers(33,0)=250   :  EZP.Papers(33,1)=353
    EZP.Papers(34,0)=176   :  EZP.Papers(34,1)=250
    EZP.Papers(35,0)=176   :  EZP.Papers(35,1)=125         
END SUB

FUNCTION EZP_OpenFileDialogW (BYVAL hndl     AS HWND,   _ 'parent window
                         byref wCaption      AS WSTRING, _ 'caption
                         byref wFileName     AS WSTRING PTR, _ 'filename
                         byref wInitialDir   AS WSTRING, _ 'starting directory
                         byref wFilter       AS WSTRING, _ 'all valid filename extensions
                         byref wDefExtension AS WSTRING, _ 'default extension
                         BYREF dFlags        AS DWORD   _ 'flags
                         ) AS LONG

    DIM AS OPENFILENAMEW ofn
    DIM wFileTitle as wstring * 255
    DIM mfilename AS WSTRING * MAX_PATH + 1
    dim wInitialDir2 as Wstring * MAX_PATH + 1 
    mfilename = *wFileName
   
    IF LEN(wInitialDir) = 0 THEN 
       GetCurrentDirectoryW(MAX_PATH + 1,varptr(wInitialDir2))
      else
       wInitialDir2 = wInitialDir   
    end if   
   
    WITH ofn
      .lStructSize       = SIZEOF(OPENFILENAMEW)
      .hwndOwner         = hndl
      .hInstance         = GetModuleHandle(NULL)
      .lpstrFilter       = VARPTR(wFilter)
      .lpstrCustomFilter = NULL
      .nMaxCustFilter    = 0
      .nFilterIndex      = 1
      .lpstrFile         = @mfilename
      .nMaxFile          = SIZEOF(mfilename)
      .lpstrFileTitle    = VARPTR(wFileTitle)
      .nMaxFileTitle     = NULL 'SIZEOF(szFileTitle)
      .lpstrInitialDir   = varptr(wInitialDir2)   'STRPTR(sInitialDir)
      .lpstrTitle        = VARPTR(wCaption)
      .Flags             = dFlags
      .nFileOffset       = 0
      .nFileExtension    = IIF(LEN(wDefExtension) = 0, LEN(mfilename), INSTR(mfilename, "." & wDefExtension) - 1) 
      .lpstrDefExt       = NULL
      .lCustData         = 0
      .lpfnHook          = NULL
      .lpTemplateName    = NULL
   END WITH
   
   FUNCTION = GetOpenFileNameW(@ofn)
   IF LEN(mfilename) THEN *wFileName = mfilename
END FUNCTION
   
FUNCTION EZP_ParseCountW(byref s as Wstring,byref del as Wstring = ",") as long
  dim MyCount as long
  dim x as long
  
  MyCount = 0 
  for x = 0 to len(s)-1
     if chr(s[x]) = del then MyCount = MyCount +1
  next x
    
  RETURN MyCount + 1

END FUNCTION

SUB EZP_ParseW(byref s as Wstring,byval  MyItem as long,byref MyWStr as WString Ptr,byref del as wstring = ",")
  dim MyTargetStart as long, MyTargetEnd as long, MyCount as long
  dim x as long
  dim MyStart as long, MyEnd as long ' Location of characters
'
MyCount = 0 
MyTargetStart = MyItem -1
MyTargetEnd = MyItem
If MyItem =1 then MyStart = 1 : MyEnd = len(s)  

  for x = 0 to len(s)-1
     if chr(s[x]) = del then 
       MyCount = MyCount + 1
       If MyCount = MyTargetStart then MyStart = x + 2
       If MyCount = MyTargetEnd then MyEnd = x : Goto Done
     end if   
  next x
 
 Done: 
  
*MyWStr = mid(s,MyStart,MyEnd-MyStart+1)
END SUB
   
' SetSBar hwnd, wMaxWidth, iHscrollMax, v->iHscrollPos,  SB_HORZ
SUB EZP_SetSBar( byval myhwnd AS HWND, BYVAL vhPage AS LONG, BYVAL vhMax AS LONG,BYVAL vhPos AS LONG, BYVAL vhBar AS LONG)
  DIM si AS SCROLLINFO

  si.cbSize = SIZEOF(si)
  si.fMask  = SIF_ALL OR SIF_DISABLENOSCROLL
  si.nMin   = 0
  si.nMax   = MAX(0, vhMax)
  si.nPage  = vhPage
  si.nPos   = vhPos
  SetScrollInfo myhwnd, vhBar, @si, -1

END SUB

FUNCTION EZP_CreateAngleFont ( BYVAL dcOutput AS hDC, _
                           byref fontName AS WSTRING, BYVAL fontStyle AS INTEGER, _
                           BYVAL fontSize AS DOUBLE, BYVAL degrees AS INTEGER = 0 ) AS hFont
   
   '
   DIM wFontName AS WSTRING * 100
   DIM iPPIY AS INTEGER
   DIM iPPIX AS INTEGER
   DIM rNewFont AS LOGFONTW
 
   iPPIX = GetDeviceCaps(dcOutput, LOGPIXELSX)
   iPPIY = GetDeviceCaps(dcOutput, LOGPIXELSY)
 
   wFontName = TRIM(fontName)
 
   '- These FontSize values passed as POINTS. I must
   '  convert them into logical pixels.
   '
   rNewFont.lfHeight = -1 * (fontSize * iPPIY ) / 72
   rNewFont.lfWidth = 0
   rNewFont.lfEscapement = degrees * 10
   rNewFont.lfOrientation = degrees * 10
 
   IF ( fontStyle AND EZP_BOLD ) = EZP_BOLD THEN
      rNewFont.lfWeight = FW_BOLD
   ELSE
      rNewFont.lfWeight = FW_NORMAL
   END IF
 
   IF (fontStyle AND EZP_ITALIC) = EZP_ITALIC THEN
     rNewFont.lfItalic = 1
   ELSE
     rNewFont.lfItalic = 0
   END IF
 
   IF (fontStyle AND EZP_STRIKE) = EZP_STRIKE THEN
     rNewFont.lfStrikeOut = 1
   ELSE
     rNewFont.lfStrikeOut = 0
   END IF
 
   IF (fontStyle AND EZP_UNDER) = EZP_UNDER THEN
     rNewFont.lfUnderline = 1
   ELSE
     rNewFont.lfUnderline = 0
   END IF
 
   rNewFont.lfCharSet = DEFAULT_CHARSET
   rNewFont.lfOutPrecision = OUT_DEFAULT_PRECIS
   rNewFont.lfClipPrecision = CLIP_DEFAULT_PRECIS
   rNewFont.lfQuality = PROOF_QUALITY
   rNewFont.lfPitchAndFamily = DEFAULT_PITCH + FF_DONTCARE
   rNewFont.lfFaceName = TRIM(wFontName) + CHR(0)
 
   '- First Retrieve Original Font
   wFontName = ""
   FUNCTION = CreateFontIndirectW(@rNewFont)
 
END FUNCTION


                                                                  
function EZP_OffScreenCanvas(byval MyOrient as long,byval CanvasRight as Single,byval  CanvasBottom as Single) as long  '
       dim dcDeskTop as hDC
       dim hDeskTop as HWND
       dim wRect as RECT
       dim hBrush as hGDIOBJ
       dim temp as Single
       DIM metricflag as Long
       
        'Just to make it certain
        wRect.Top = 0  :  wRect.Left = 0
        
        if MyOrient <> EZP_PORTRAIT THEN  'Landscape
           temp = CanvasRight
           CanvasRight = CanvasBottom
           CanvasBottom = temp
        END IF
        
        if CanvasRight > 20 THEN  'test for metric papers
           metricflag = 1
        END IF
        
       
        IF EZP.hbit  THEN DeleteObject SelectObject(EZP.memDC,EZP.hbit)
        IF EZP.memDC THEN DeleteDC EZP.memDC
        
        hDesktop = GetDesktopWindow()
        dcDesktop = GetDC(hDesktop)
           EZP.ScreenPPIx = GetDeviceCaps(dcDesktop, LOGPIXELSX)
           EZP.ScreenPPIy = GetDeviceCaps(dcDesktop, LOGPIXELSY)
           EZP.ScreenSizex = GetDeviceCaps(dcDesktop, HORZSIZE)
           EZP.ScreenSizey = GetDeviceCaps(dcDesktop, VERTSIZE)
           EZP.ScreenResx = GetDeviceCaps(dcDesktop, HORZRES)
           EZP.ScreenResy = GetDeviceCaps(dcDesktop, VERTRES)
           
           'Measurements are stored internally in inches    
           if metricflag = 0 THEN  'Inches
               wRect.Right = CanvasRight * EZP.ScreenPPIx : wRect.Bottom = CanvasBottom * EZP.ScreenPPIy
             Else 'Metric  25.4 mm/inch
               wRect.Right = CanvasRight * EZP.ScreenPPIx / 25.4 : wRect.Bottom = CanvasBottom * EZP.ScreenPPIy / 25.4
               'print "Metric"
           END IF
           
           EZP.CanvasRight = wRect.Right
           EZP.CanvasBottom = wRect.Bottom
           EZP.memDC = CreateCompatibleDC(dcDesktop)
          
           EZP.hbit  = CreateCompatibleBitmap(dcDesktop, wRect.Right, wRect.Bottom)
           EZP.hbit =  SelectObject(EZP.memDC, EZP.hbit)

           hbrush   = GetStockObject(WHITE_BRUSH)
           PatBlt EZP.memDC, 0, 0, wRect.Right, wRect.Bottom,  PATCOPY
           
           'Will probably delete next 6 lines
           IF hbrush THEN SelectObject EZP.memDC, hbrush
      
        ReleaseDC hDesktop, dcDesktop
        
        function = 0
 end function       

   
function EZP_DocBegin(byref StrFile as Wstring,byval InchOrCm as long, _
byval WhichPaper as long,byval  WhichOrient as long, byval WhichBin as long = 0,byval DoDuplex as long = 0) as HANDLE
 Dim MyDocResult as HANDLE
 dim dh as DOCHEADER
 dim dh2 as DOCHEADER
 dim sFile as Wstring * 255
 dim hFile as HANDLE
 Dim NumBytes as UINTEGER
 dim MyFileSize as INTEGER
 dim overlap as LPOVERLAPPED ptr
 Dim wTestStr as wstring * 255
 dim xx as long, yy as LONG
 
    'Initalize some fields 
    EZP_FillPaperArray
    if len(EZP.ControlWords(EZP_PRINT)) = 0 then EZP.ControlWords(EZP_PRINT) = "Print"
    if len(EZP.ControlWords(EZP_CLOSE)) = 0 then EZP.ControlWords(EZP_CLOSE) = "Close"
    if len(EZP.ControlWords(EZP_FITWIN)) = 0 then EZP.ControlWords(EZP_FITWIN) = "Fit to Window"
    if len(EZP.ControlWords(EZP_FITWIDTH)) = 0 then EZP.ControlWords(EZP_FITWIDTH) = "Fit to Width" 
    if len(EZP.HeaderFooterDate) = 0 then EZP_SetHeaderFooterDate
    
    if EZP.MyGridCols(0) = 0 THEN
       Erase EZP.MyGridCols, EZP.MyGridRows
       EZP.MyGridCols(0) = EZP_MAXNUMCOLS
       EZP.MyGridRows(0) = EZP_MAXNUMROWS
       for xx = 1 to EZP.MyGridCols(0)
          EZP.MyGridCols(xx) = EZP_GRIDCOLDEFAULT 
       NEXT xx
       for yy = 1 to EZP.MyGridRows(0)
          EZP.MyGridRows(yy) = EZP_GRIDROWDEFAULT 
       NEXT yy
    end if
    
   EZP.ScreenMargins.nTop = EZP_GRIDMARGINSDEFAULT
   EZP.ScreenMargins.nLeft = EZP_GRIDMARGINSDEFAULT
   EZP.ScreenMargins.nBottom = EZP_GRIDMARGINSDEFAULT
   EZP.ScreenMargins.nRight = EZP_GRIDMARGINSDEFAULT 
 
   'MyDocResult = 0 is a success -1 is failed
    MyDocResult = 0
    
    sFile = StrFile
    if len(sFile) = 0 THEN
        sFile = "TempFile.ezp"
    END IF
 
   
   'Kill the existing file if present
   DeleteFileW(varptr(sFile))
   
   'xx = MAKELONG(EZP_VERSION,EZP.InchesOrCm)
   
   EZP.InchesOrCm = InchOrCm
   dh.MyAction = EZP_NWDC
   dh.MyVersion = MAKELONG(EZP_VERSION,EZP.InchesOrCm) 
   dh.MyDuplex = DoDuplex
    if WhichPaper < 1 or WhichBin > 35 THEN
       WhichPaper = EZP_DEFAULTPAPERSIZE 'letter
   END IF
   dh.MyPaper = WhichPaper
   dh.MyOrient = WhichOrient
   if WhichBin < 1 or WhichBin >15 THEN
       WhichBin =7 'autobin
   END IF
   dh.MyBin = WhichBin
   dh.MyPages = 1  'Start out with the First Page
 
   EZP.NumPages = 1 
   hFile = CreateFileW(VARptr(sFile),GENERIC_WRITE OR GENERIC_READ,0,NULL,CREATE_NEW,FILE_ATTRIBUTE_NORMAL,NULL)
   if hFile = INVALID_HANDLE_VALUE THEN
        MyDocResult = 0
        Exit Function
      else 
        MyDocResult = hFile  
        EZP.FileName = sFile
        EZP.FileHandle = hFile 
        WriteFile(hFile,@dh,Sizeof(dh),cast(LPDWORD,@NumBytes),null)  
   END IF
    
   'Make a default font and screen DC here
 '  EZP_DocFont(hFile,EZP_NORMAL,EZP_FONT_SIZE,EZP_BLACK,0,EZP_FONT_NAME)
     dim hDesktop as hWnd
     dim hFont as hFont
     dim dcDesktop as hDC
     
     hDesktop = GetDesktopWindow()
     dcDesktop = GetDC(hDesktop)
     EZP.memDC = dcDesktop  'This will be deleted when the Off Screen Canvas is generated
     'DeleteDC dcDesktop
     hFont= EZP_CreateAngleFont(dcDesktop,EZP_FONT_NAME,EZP_NORMAL,EZP_FONT_SIZE,0)
     
     if hFont <> 0 THEN
       if EZP.ScreenFont <> 0 then DeleteObject EZP.ScreenFont
     end if     
     
     EZP.ScreenFont = hFont
     EZP.ScreenFontName = EZP_FONT_NAME
     EZP.ScreenFontStyle = EZP_NORMAL
     EZP.ScreenFontSize = EZP_FONT_SIZE
     EZP.ScreenFontAngle = 0
     EZP.ScreenFontColor = EZP_BLACK
                   
     EZP.CurrentPage = 1
     
   function = MyDocResult  
END FUNCTION

sub EZP_DocNewPage(byval hFile as Handle,byref MyBookmark as WString = "")
  dim doc as DOCCOMMAND
  dim dh as DOCHEADER
  Dim NumBytes as UINTEGER
  dim SavedFilePointer as long
  
  'Load the file pointer to the next command in the MySize Field
  doc.MyAction = EZP_NWPG
  doc.TextLen =  len(MyBookmark)
  
  'Mysize points to the first record after the NewPage block
  doc.MySize = SetFilePointer(hFile,0,NULL,FILE_CURRENT) + Sizeof(doc) + doc.TextLen * 2
  WriteFile(hFile,@doc,Sizeof(doc),cast(LPDWORD,@NumBytes),NULL)
  if len(MyBookmark) > 0 THEN
     WriteFile(hFile,Varptr(MyBookMark),(doc.TextLen + 0) * 2,cast(LPDWORD,@NumBytes),NULL)
  END IF
  
  'Update Page Info in the DOCHEADER
  SavedFilePointer = GetFilePointer(hFile) 'Save the current Pointer
  SetFilePointer(hFile,0,NULL,FILE_BEGIN)
  ReadFile(hFile,@dh,Sizeof(dh),cast(LPDWORD,@NumBytes),NULL)
  dh.MyPages = dh.MyPages + 1
   if len(MyBookmark) > 0 THEN
     dh.MyBookmarks = dh.MyBookmarks + 1
  END IF
  SetFilePointer(hFile,0,NULL,FILE_BEGIN)
  WriteFile(hFile,@dh,Sizeof(dh),cast(LPDWORD,@NumBytes),NULL)
  SetFilePointer(hFile,SavedFilePointer,NULL,FILE_BEGIN) 'Put the pointer back
  'Save the last font  
  EZP_DocFont(hFile,EZP.ScreenFontStyle,EZP.ScreenFontSize,EZP.ScreenFontColor,EZP.ScreenFontName)
  EZP.CurrentPage = EZP.CurrentPage + 1   
END SUB

sub EZP_DocEnd(byval hFile as HANDLE,byval MyStyle as long = 0)
   dim doc as DOCCOMMAND
   dim dh as DOCHEADER
   Dim NumBytes as UINTEGER
   
   doc.MyAction = EZP_ENDD
   doc.MyStyle = MyStyle
   WriteFile(hFile,@doc,Sizeof(doc),cast(LPDWORD,@NumBytes),NULL)
   
   CloseHandle(hFile) :  EZP.FileHandle = 0
   
END sub

Function EZP_DocFont(byval hFile as HANDLE, byval MyStyle as long,byval  MySize as long,byval  MyColor as INTEGER,byref MyFontName as wstring Ptr) as SINGLE
   dim doc as DOCCOMMAND
   Dim NumBytes as UINTEGER
   dim hDesktop as hWnd
   dim dcDesktop as hDC
   dim hfont as hfont
   DIM rSize AS Size
   Dim MyString as WSTRING * 10
   dim ScaleFactor as SINGLE
   
   if EZP.InchesOrCm = 0 THEN  ScaleFactor = 1 else ScaleFactor = 2.54
      
   MyString = "123AKXy"
 
    hFont= EZP_CreateAngleFont(EZP.memDC,*MyFontName,MyStyle,MySize,0)
    SelectObject EZP.memDC,hFont
    GetTextExtentPoint32W EZP.memDC, Varptr(MyString), LEN(MyString), @rSize 
    
    if hFont <> 0 THEN
      if EZP.ScreenFont <> 0 then DeleteObject EZP.ScreenFont
    end if  
                
    EZP.ScreenFont = hFont
    EZP.ScreenFontName = *MyFontName
    EZP.ScreenFontStyle = MyStyle
    EZP.ScreenFontSize =  MySize
    EZP.ScreenFontAngle = 0
    EZP.ScreenFontColor = MyColor
                   
   
   doc.MyAction = EZP_FONT
   doc.MySize = MySize
   doc.MyColor1 = MyColor
   doc.MyAngle = 0
   doc.MyStyle = MyStyle
   doc.TextLen = len(*MyFontName)
   WriteFile(hFile,@doc,Sizeof(doc),cast(LPDWORD,@NumBytes),NULL)
   WriteFile(hFile,MyFontName,(doc.TextLen + 0) * 2,cast(LPDWORD,@NumBytes),NULL)
   
   function = (rSize.cy / GetDeviceCaps(EZP.memDC, LOGPIXELSY)) * ScaleFactor 
END function


sub EZP_DocText(byval hFile as HANDLE,byval  x as single,byval  y as single, byref MyText as Wstring Ptr,byval MyAngle as long = 0, _
byval x2 as single = 0,byval y2 as single = 0,byval  MyJust as long = 0,byval MyBorderColor as Integer = 0,byval MyFillColor as integer = 0)

   dim doc as DOCCOMMAND
   Dim NumBytes as UINTEGER
   dim cm as SINGLE
   
   if EZP.InchesOrCm <> 0 THEN cm = 1/2.54 else cm = 1
    
   doc.MyAction = EZP_TEXT
   doc.MyXPos = x * cm
   doc.MyYPos = y  * cm
   doc.TextLen = len(*MyText)
   doc.MyAngle = MyAngle
   doc.MyXPos2 = x2 * cm
   doc.MyYPos2 = y2 * cm
   doc.MyStyle = MyJust    'left center right
   doc.MyColor1 = MyBorderColor '
   doc.MyColor2 = MyFillColor  'if fill color = 0 (black) then toggle to white
   
   WriteFile(hFile,@doc,Sizeof(doc),cast(LPDWORD,@NumBytes),NULL)
   WriteFile(hFile,MyText,(doc.TextLen + 0) * 2,cast(LPDWORD,@NumBytes),NULL)
END SUB

Function EZP_DocTextEx(byval hFile as HANDLE,byval  x2 as single, byval x as single,byval  y as single,byval MyLen as Long ptr = 0,byref BreakChar as WString = "") as LONG
   ' MyOption = 0 Reset Text Ptr to zero, = 1 Read Grid at current position
   dim doc as DOCCOMMAND
   Dim NumBytes as UINTEGER
   dim cm as single
   Dim MyText as wstring * 255
   static mycount as long = 0
   
   mycount = mycount + 1
   MyText=""
   
   if EZP.InchesOrCm <> 0 THEN cm = 1/2.54 else cm = 1
    
   doc.MyAction = EZP_TEXT
   doc.MyXPos = x * cm
   doc.MyYPos = y * cm
   doc.TextLen = 0
   doc.MyAngle = 0
   doc.MyXPos2 = x2 * cm  'This is the width of the desired string
   doc.MyYPos2 = 0
   doc.MyStyle = 0   'This char will break the string
   doc.MyColor1 = 0 
   doc.MyColor2 = 0 
   
   
  
   
    if len(BreakChar) <> 0 then
      EZP.TextPtrBreakChar = left(BreakChar,1)
      'EZP.TextPtrBreakOptions = MyOptions
    end if   
   
   if EZP.TextPtr <> 0 THEN
      EZP_GetText(VARPTR(MyText), doc.MyXPos2) 'Updates EZP.TextPtrPos aso
   end if
   
   MyText = LTrim(MyText)  'get rid on leading spaces
   
   doc.MyXPos2 = 0
   doc.TextLen = len(MyText)
   
   if MyLen <>  0 THEN *MyLen = doc.TextLen 
      
   
      WriteFile(hFile,@doc,Sizeof(doc),cast(LPDWORD,@NumBytes),NULL)
      WriteFile(hFile,varptr(MyText),(doc.TextLen + 0) * 2,cast(LPDWORD,@NumBytes),NULL) 
   
  
   Function = EZP.TextPtrLen - EZP.TextPtrPos 'current position of the Block Text Pointer after current posting 
END Function



Function EZP_DocTextExRead(byval hFile as HANDLE,byref MyText as Wstring PtR) as LONG

   dim doc as DOCCOMMAND
   Dim NumBytes as UINTEGER
    
   doc.MyAction = EZP_TXTR
   doc.MyXPos = 0
   doc.MyYPos = 0
   doc.TextLen = len(*MyText)
   doc.MyAngle = 0
   doc.MyXPos2 = 0
   doc.MyYPos2 = 0
   doc.MyStyle = 0   
   doc.MyColor1 = 0 
   doc.MyColor2 = 0
   
   if doc.TextLen > 0 THEN
        if EZP.TextPtr <> 0 then Deallocate EZP.TextPtr : EZP.TextPtr = 0
        EZP.TextPtr = cast(wstring ptr,CAllocate((doc.TextLen + 4 ) * Len(WString)))  'Pad by 8 bytes for safety
        *EZP.TextPtr = *MyText
        EZP.TextPtrLen = doc.TextLen
        EZP.TextPtrPos = 1
   end if
  
   WriteFile(hFile,@doc,Sizeof(doc),cast(LPDWORD,@NumBytes),NULL)
   WriteFile(hFile,MyText,(doc.TextLen + 0) * 2,cast(LPDWORD,@NumBytes),NULL)
   
   function = EZP.TextPtrLen
END Function

sub EZP_DocHeader(byval hFile as HANDLE,byref MyText as Wstring Ptr,byval x as SINGLE,byval y as single,byval x2 as single,byval y2 as single,byval  MyStyle as long = 0,byval  MySize as single = 0)

   dim doc as DOCCOMMAND
   Dim NumBytes as UINTEGER
   dim cm as single

   if EZP.InchesOrCm <> 0 THEN cm = 1/2.54 else cm = 1
    
   doc.MyAction = EZP_HEAD
   doc.MyXPos = x * cm
   doc.MyYPos = y* cm
   doc.TextLen = len(*MyText)
   doc.MyAngle = 0
   doc.MyXPos2 = x2 * cm
   doc.MyYPos2 = y2 * cm
   doc.MyStyle = MyStyle    'left center right
   doc.MyColor1 = 0 '
   doc.MyColor2 = 0  'if fill color = 0 (black) then toggle to white
   doc.MySize = MySize  ' of Page Number or Date
   
   WriteFile(hFile,@doc,Sizeof(doc),cast(LPDWORD,@NumBytes),NULL)
   WriteFile(hFile,MyText,(doc.TextLen + 0) * 2,cast(LPDWORD,@NumBytes),NULL)
END SUB

sub EZP_DocFooter(byval hFile as HANDLE,byref MyText as Wstring Ptr,byval x as SINGLE,byval y as single,byval x2 as single,byval y2 as single,byval  MyStyle as long = 0,byval  MySize as single = 0)

   dim doc as DOCCOMMAND
   Dim NumBytes as UINTEGER
   dim cm as single

   if EZP.InchesOrCm <> 0 THEN cm = 1/2.54 else cm = 1
    
   doc.MyAction = EZP_FOOT
   doc.MyXPos = x * cm
   doc.MyYPos = y * cm
   doc.TextLen = len(*MyText)
   doc.MyAngle = 0
   doc.MyXPos2 = x2 * cm
   doc.MyYPos2 = y2 * cm
   doc.MyStyle = MyStyle    'left center right
   doc.MyColor1 = 0 '
   doc.MyColor2 = 0  'if fill color = 0 (black) then toggle to white
   doc.MySize = MySize
   
   WriteFile(hFile,@doc,Sizeof(doc),cast(LPDWORD,@NumBytes),NULL)
   WriteFile(hFile,MyText,(doc.TextLen + 0) * 2,cast(LPDWORD,@NumBytes),NULL)
END SUB

sub EZP_DocDateStamp(byval hFile as HANDLE,byref MyText as Wstring)

   dim doc as DOCCOMMAND
   Dim NumBytes as UINTEGER
    
   doc.MyAction = EZP_DATE
   doc.MyXPos = 0
   doc.MyYPos = 0
   doc.TextLen = len(MyText)
   doc.MyAngle = 0
   doc.MyXPos2 = 0
   doc.MyYPos2 = 0
   doc.MyStyle = 0    'left center right
   doc.MyColor1 = 0 '
   doc.MyColor2 = 0  'if fill color = 0 (black) then toggle to white
   
   WriteFile(hFile,@doc,Sizeof(doc),cast(LPDWORD,@NumBytes),NULL)
   WriteFile(hFile,varptr(MyText),(doc.TextLen + 0) * 2,cast(LPDWORD,@NumBytes),NULL)
END SUB

sub EZP_DocChangeNames(byval hFile as HANDLE,byref MyPrintName as WSTRING,byref MyCloseName as WSTRING,byref MyWindowFitName as WSTRING,byref MyWidthFitName as WSTRING)

   dim doc as DOCCOMMAND
   Dim NumBytes as UINTEGER
   Dim MyText as WSTRING * 100
   
   MyText = MyPrintName & "|" & MyCloseName  & "|" & MyWindowFitName  & "|" & MyWidthFitName
    
   doc.MyAction = EZP_NAME
   doc.MyXPos = 0
   doc.MyYPos = 0
   doc.TextLen = len(MyText)
   doc.MyAngle = 0
   doc.MyXPos2 = 0
   doc.MyYPos2 = 0
   doc.MyStyle = 0    'left center right
   doc.MyColor1 = 0 '
   doc.MyColor2 = 0  'if fill color = 0 (black) then toggle to white
   
   WriteFile(hFile,@doc,Sizeof(doc),cast(LPDWORD,@NumBytes),NULL)
   WriteFile(hFile,varptr(MyText),(doc.TextLen + 0) * 2,cast(LPDWORD,@NumBytes),NULL)
END SUB

Function EZP_DocAddImage(byval hFile as HANDLE, byref MyBitMapFile as Wstring) as LONG

   dim doc as DOCCOMMAND
   Dim NumBytes as UINTEGER
   dim MyResult as LONG
   
   MyResult = EZP_AddImage(MyBitMapFile)
   if MyResult = 0 THEN
     Function = 0 
     exit Function 
   END IF
    
   doc.MyAction = EZP_IMGA
   doc.MyXPos = 0
   doc.MyYPos = 0
   doc.TextLen = len(EZP.MyGraphics(0))  'Ansi string so not times 2
   doc.MyAngle = 0
   doc.MyXPos2 = 0
   doc.MyYPos2 = 0
   doc.MyStyle = MyResult
   doc.MyColor1 = 0
   doc.MyColor2 = 0
   
   WriteFile(hFile,@doc,Sizeof(doc),cast(LPDWORD,@NumBytes),NULL)
   WriteFile(hFile,strptr(EZP.MyGraphics(0)),doc.TextLen,cast(LPDWORD,@NumBytes),NULL)
   function = MyResult
END Function

SUB EZP_DocDrawImage(byval hFile as HANDLE, byval WhichBitmap as LONG,byval MyAngle as LONG, byval x as single, y as single, byval x2 as single = 0, byval y2 as single = 0, byval AspectFlag as long = 0)
   
   dim doc as DOCCOMMAND
   Dim NumBytes as UINTEGER
   dim MyResult as LONG
   dim cm as single

   if EZP.InchesOrCm <> 0 THEN cm = 1/2.53999 else cm = 1
   
   doc.MyAction = EZP_IMGR
   doc.MyXPos = x * cm
   doc.MyYPos = y * cm
   doc.TextLen = 0
   doc.MyAngle = MyAngle
   doc.MyXPos2 = x2 * cm
   doc.MyYPos2 = y2 * cm
   doc.MyStyle = WhichBitmap
   doc.MyColor1 = 0
   doc.MyColor2 = 0
   doc.MySize  = AspectFlag
 
   WriteFile(hFile,@doc,Sizeof(doc),cast(LPDWORD,@NumBytes),NULL)
 
END SUB

'int nXStartArc, // first radial ending point  
'int nYStartArc, // first radial ending point  
'int nXEndArc, // second radial ending point  
'int nYEndArc  // second radial ending point  


sub EZP_DocArc(byval hFile as HANDLE,byval  x as single,byval  y as single,byval  x2 as single, byval y2 as single, _
byval X1Arc as Single,byval Y1Arc as single,byval x2Arc as single,Y2Arc as single _
,byval MyWidth as single,byval MyStyle as long,byval  MyColor as long)
   dim doc as DOCCOMMAND
   Dim NumBytes as UINTEGER
   dim cm as single
   dim MyText as WString * 100

   if EZP.InchesOrCm <> 0 THEN cm = 1/2.54 else cm = 1
   
   MyText = WSTR(X1Arc) & "," & WSTR(Y1Arc) & "," & WSTR(X2Arc) & "," & WSTR(Y2Arc)   'Put the Arc in a string
   
   doc.MyAction = EZP_ARC1
   doc.MyXPos = x * cm
   doc.MyYPos = y * cm
   doc.MyXPos2 = x2 * cm
   doc.MyYPos2 = y2 * cm
   doc.MyWidth = MyWidth
   doc.MyStyle = MyStyle
   doc.MyColor1 = MyColor
   doc.TextLen = len(MyText)
   
   WriteFile(hFile,@doc,Sizeof(doc),cast(LPDWORD,@NumBytes),NULL)
   WriteFile(hFile,varptr(MyText),(doc.TextLen + 0) * 2,cast(LPDWORD,@NumBytes),NULL)
END SUB

sub EZP_DocPie(byval hFile as HANDLE,byval x as single,byval  y as single,byval  x2 as single,byval  y2 as single, _
byval X1Arc as Single,byval Y1Arc as single,byval x2Arc as single,byval Y2Arc as single _
,byval MyWidth as single,byval  MyStyle as long,byval  MyColor as long,byval MyFill as Long)
   dim doc as DOCCOMMAND
   Dim NumBytes as UINTEGER
   dim cm as single
   dim MyText as WString * 100

   if EZP.InchesOrCm <> 0 THEN cm = 1/2.54 else cm = 1
   MyText = WSTR(X1Arc) & "," & WSTR(Y1Arc) & "," & WSTR(X2Arc) & "," & WSTR(Y2Arc)   'Put the Arc in a string
   
   doc.MyAction = EZP_PIE1
   doc.MyXPos = x * cm
   doc.MyYPos = y * cm
   doc.MyXPos2 = x2 * cm
   doc.MyYPos2 = y2 * cm
   doc.MyWidth = MyWidth
   doc.MyStyle = MyStyle
   doc.MyColor1 = MyColor
   doc.MyColor2 = MyFill
   doc.TextLen = len(MyText)
   
   WriteFile(hFile,@doc,Sizeof(doc),cast(LPDWORD,@NumBytes),NULL)
   WriteFile(hFile,varptr(MyText),(doc.TextLen + 0) * 2,cast(LPDWORD,@NumBytes),NULL)
END SUB

sub EZP_DocLine(byval hFile as HANDLE, byval x as single,byval  y as single,byval  x2 as single,byval y2 as single,byval MyWidth as single,byval MyStyle as long,byval  MyColor as long)
   dim doc as DOCCOMMAND
   Dim NumBytes as UINTEGER
   dim cm as single

   if EZP.InchesOrCm <> 0 THEN cm = 1/2.54 else cm = 1
   
   doc.MyAction = EZP_LINE
   doc.MyXPos = x * cm
   doc.MyYPos = y * cm
   doc.MyXPos2 = x2 * cm
   doc.MyYPos2 = y2 * cm
   doc.MyWidth = MyWidth
   doc.MyStyle = MyStyle
   doc.MyColor1 = MyColor
   doc.MyAngle = 0
   WriteFile(hFile,@doc,Sizeof(doc),cast(LPDWORD,@NumBytes),NULL)
END SUB

sub EZP_DocRect(byval hFile as HANDLE,byval  x as single,byval  y as single,byval  x2 as single,byval  y2 as single, _
byval MyFillStyle as single,byval  MyFillColor as Integer,byval MyWidth as long,byval  MyBorderColor as integer _
,byval RoundRectPix as integer = 0)

   dim doc as DOCCOMMAND
   Dim NumBytes as UINTEGER
   dim cm as single
   
   if EZP.InchesOrCm <> 0 THEN cm = 1/2.54 else cm = 1
   
   doc.MyAction = EZP_RECT
   doc.MyXPos = x * cm
   doc.MyYPos = y * cm
   doc.MyXPos2 = x2 * cm
   doc.MyYPos2 = y2 * cm
   doc.MyWidth = MyWidth
   doc.MyStyle = MyFillStyle
   doc.MyColor1 = MyFillColor
   doc.MyColor2 = MyBorderColor
   doc.MyAngle = RoundRectPix
   WriteFile(hFile,@doc,Sizeof(doc),cast(LPDWORD,@NumBytes),NULL)
END SUB

sub EZP_DocEllipse(byval hFile as HANDLE,byval  x as single,byval  y as single, byval x2 as single, byval y2 as single, _
byval MyFillStyle as single,byval  MyFillColor as Integer,byval MyWidth as long, byval MyBorderColor as integer) 
   dim doc as DOCCOMMAND
   Dim NumBytes as UINTEGER
   dim cm as single

   if EZP.InchesOrCm <> 0 THEN cm = 1/2.54 else cm = 1
   
   doc.MyAction = EZP_ELLP
   doc.MyXPos = x * cm
   doc.MyYPos = y * cm
   doc.MyXPos2 = x2 * cm
   doc.MyYPos2 = y2 * cm
   doc.MyWidth = MyWidth
   doc.MyStyle = MyFillStyle
   doc.MyColor1 = MyFillColor
   doc.MyColor2 = MyBorderColor
   WriteFile(hFile,@doc,Sizeof(doc),cast(LPDWORD,@NumBytes),NULL)
END SUB

sub EZP_DocGridSet(byval hFile as HANDLE,byval  Action as Long,byval x as Single,byval y as single, byval w as single = 0, byval h as SINGLE = 0 ) 
   dim doc as DOCCOMMAND
   Dim NumBytes as UINTEGER
   dim rx as SINGLE
   dim xx as long, yy as LONG
   
   'Actions  NewGrid = 0 Move to corner of grid cell = 1 which is stored in EZP.Globals
   'const EZP_GRIDNEW        =  0
   'const EZP_GRIDMOD        =  1
   'const EZP_GRIDMARGINS    =  2
   
   if Action = EZP_GRIDNEW THEn
    'Erase EZP.MyGridCols, EZP.MyGridRows
    'print "Here"
    EZP.MyGridCols(0) = EZP_MAXNUMCOLS
    EZP.MyGridRows(0) = EZP_MAXNUMROWS
    if EZP.InchesOrCm <> 0 THEN rx=2.53999 else rx = 1
    for xx = 1 to EZP.MyGridCols(0)
       EZP.MyGridCols(xx) = x / rx 
    NEXT xx
    for yy = 1 to EZP.MyGridRows(0)
       EZP.MyGridRows(yy) = y / rx
    NEXT yy
   END IF 
   'print "NEW GRID  ";EZP.MyGridCols(1)
   
   if Action = EZP_GRIDMOD THEn
    if EZP.InchesOrCm <> 0 THEN rx=2.53999 else rx = 1
    if x > 0 and x <= EZP_MAXNUMCOLS and w > 0 then EZP.MyGridCols(int(x)) = w / rx 
    if y > 0 and y <= EZP_MAXNUMROWS and h > 0 then EZP.MyGridRows(int(y)) = h / rx
   END IF
   
   if Action = EZP_GRIDMARGINS THEn
    if EZP.InchesOrCm <> 0 THEN rx=2.53999 else rx = 1
    if x > 0 and x <= EZP_MAXNUMCOLS and x > 0 then  EZP.ScreenMargins.nLeft = x / rx 
    if y > 0 and y <= EZP_MAXNUMROWS and y > 0 then EZP.ScreenMargins.nTop = y / rx
    if w > 0 and w <= EZP_MAXNUMCOLS and w > 0 then  EZP.ScreenMargins.nRight = w / rx 
    if h > 0 and h <= EZP_MAXNUMROWS and h > 0 then EZP.ScreenMargins.nBottom = h / rx
   END IF
   
   
   EZP.ScreenMargins.nTop = EZP_GRIDMARGINSDEFAULT
   EZP.ScreenMargins.nLeft = EZP_GRIDMARGINSDEFAULT
   EZP.ScreenMargins.nBottom = EZP_GRIDMARGINSDEFAULT
   EZP.ScreenMargins.nRight = EZP_GRIDMARGINSDEFAULT
   
 ' print "Col Set " ; EZP.MyGridCols(1),EZP.MyGridCols(2),EZP.MyGridCols(3),EZP.MyGridCols(4),EZP.MyGridCols(5),EZP.MyGridCols(6)
 ' print "Row Set " ;EZP.MyGridRows(1),EZP.MyGridRows(2),EZP.MyGridRows(3),EZP.MyGridRows(4),EZP.MyGridRows(5),EZP.MyGridRows(6)
  
   
   
   doc.MyAction = EZP_GRID
   doc.MyXPos = x
   doc.MyYPos = y
   doc.MyXPos2 = w
   doc.MyYPos2 = h
   doc.MySize = 0
   doc.MyStyle = Action
   doc.MyColor1 = 0 
   doc.MyColor2 = 0
   WriteFile(hFile,@doc,Sizeof(doc),cast(LPDWORD,@NumBytes),NULL)
END SUB

sub EZP_DocPolygon(byval hFile as HANDLE,byval NumPoints as long, MyPoints() as PolyPoints,byval lineColor as integer,byval  fillColor as integer) 
   dim doc as DOCCOMMAND
   Dim NumBytes as UINTEGER
   Dim MyText as WSTRING * 5000
   dim cm as single,temp as Single
   dim j as long

   if EZP.InchesOrCm <> 0 THEN cm = 1/2.54 else cm = 1
   
   MyText = ""  'Placeholder comma delimited array of points
   for j=0 to NumPoints-1
      temp=MyPoints(j).x * cm
      MyText = MyText & wstr(temp) & ","
      temp=MyPoints(j).y * cm
      MyText = MyText & wstr(temp) & ","
   NEXT
    
   doc.MyAction = EZP_POLY
   doc.MyXPos = 0  'x * cm
   doc.MyYPos = 0  'y * cm
   doc.TextLen = len(MyText)
   doc.MyAngle = 0
   doc.MyXPos2 = 0
   doc.MyYPos2 = 0
   doc.MySize = NumPoints
   doc.MyStyle = 0   
   doc.MyColor1 = lineColor 
   doc.MyColor2 = fillColor  
   
   WriteFile(hFile,@doc,Sizeof(doc),cast(LPDWORD,@NumBytes),NULL)
   WriteFile(hFile,varptr(MyText),(doc.TextLen + 0) * 2,cast(LPDWORD,@NumBytes),NULL)
END SUB

sub EZP_DocShape(byval hFile as HANDLE,byval  WhichShape as long,byval x as single,byval y as single,byval x2 as single,byval  y2 as single,byval fillColor as integer= 0 , MySize as Long= 2) 
   dim doc as DOCCOMMAND
   Dim NumBytes as UINTEGER
   Dim MyText as WSTRING * 5000
   dim cm as single

   if EZP.InchesOrCm <> 0 THEN cm = 1/2.54 else cm = 1
   
   MyText = "xx"  'Placeholder comma delimited array of points
    
   doc.MyAction = EZP_SHAP
   doc.MyXPos = x * cm
   doc.MyYPos = y * cm
   doc.TextLen = len(MyText)
   doc.MyAngle = 0
   doc.MyXPos2 = x2 * cm
   doc.MyYPos2 = y2 * cm
   doc.MySize = MySize
   doc.MyStyle = WhichShape   
   doc.MyColor1 = 0
   doc.MyColor2 = fillColor
   doc.MyAngle = 0  
   
   WriteFile(hFile,@doc,Sizeof(doc),cast(LPDWORD,@NumBytes),NULL)
   WriteFile(hFile,varptr(MyText),(doc.TextLen + 0) * 2,cast(LPDWORD,@NumBytes),NULL)
END SUB

sub EZP_DocSymbol(byval hFile as HANDLE,byval  WhichSymbol as long,byval  MySize as long,byval x as single,byval y as single,byval lineColor as integer,byval  fillColor as integer = -123) 
   dim doc as DOCCOMMAND
   Dim NumBytes as UINTEGER
   dim cm as single

   if EZP.InchesOrCm <> 0 THEN cm = 1/2.54 else cm = 1
    
   doc.MyAction = EZP_SMBL
   doc.MyXPos = x * cm
   doc.MyYPos = y * cm 
   doc.MySize = MySize
   doc.MyStyle = WhichSymbol   
   doc.MyColor1 = lineColor 
   doc.MyColor2 = fillColor
   
   WriteFile(hFile,@doc,Sizeof(doc),cast(LPDWORD,@NumBytes),NULL)
END SUB

sub EZP_DocTable(byval hFile as HANDLE,MyTable() as TableCell,byval x as single,byval y as single,byval CellCharWidth as long = 8,byval SuppressGridLines as long = 0) 
   dim doc as DOCCOMMAND
   Dim NumBytes as UINTEGER
   Dim MyText as STRING
   dim cm as single
   dim  Myarraylen as Integer, Myarraysize as Integer
   dim text as wstring ptr

   if EZP.InchesOrCm <> 0 THEN cm = 1/2.54 else cm = 1
   
   Myarraylen = (UBound(MyTable, 2) - LBound(MyTable, 2) + 1) _
         * (UBound(MyTable, 1) - LBound(MyTable, 1) + 1)

   Myarraysize = Myarraylen * SizeOf(TableCell)
   MyText= space(Myarraysize)  
   CopyMemory(strptr(MyText),varptr(MyTable(0,0)),Myarraysize)
  
   
   doc.MyAction = EZP_TABL
   doc.MyXPos = x * cm
   doc.MyYPos = y * cm
   doc.TextLen = len(MyText)
   doc.MyXPos2 = 0 'x2 * cm
   doc.MyYPos2 = 0 'y2 * cm
   doc.MySize = UBound(MyTable, 2) - LBound(MyTable, 2) + 1
   doc.MyAngle = UBound(MyTable, 1) - LBound(MyTable, 1) + 1
   doc.MyStyle = SuppressGridLines   
   doc.MyColor1 = CellCharWidth
   doc.MyColor2 = 0
 
   WriteFile(hFile,@doc,Sizeof(doc),cast(LPDWORD,@NumBytes),NULL)
   WriteFile(hFile,strptr(MyText),(doc.TextLen + 0),cast(LPDWORD,@NumBytes),NULL)
END SUB



function EZP_ReadCurrentPage(byval hFile as HANDLE,byval SorP as Long=0) as Long
  Dim MyDocResult as Long
  dim doc as DOCCOMMAND
  dim TextwStr as wString * EZP_MAXTEXTLEN
  dim CurrentPointer as Long
  dim FileSize as long
  Dim NumBytes as UINTEGER
  dim MyDC as hDC,TempDC as hDC
  dim ScaleFactorX as Single ,ScaleFactorY as Single
  dim x as long, y as Long,x2 as long,y2 as Long,m as Single,xx as long, yy as LONG ,just as Long
  dim hFont as hFont,SavedFont as hFont
  dim lp as point
  Dim lb as LogBrush
  dim MyPen as hGDIOBJ , MyPen2 as hGDIOBJ, MyPen3 as hGDIOBJ
  Dim MyBrush as hGDIOBJ, MyBrush2 as hGDIOBJ
  dim DrawRect as RECT
  Dim hBmp as HBITMAP
  dim myBitmap as BITMAP
  dim WhichImage as LONG
  Dim CalcTextLen as LONG
  Dim MyTextIn as WString * 2000
  Dim MyTextOut as Wstring * 2000
  dim iPPIX AS INTEGER
  dim tempfontname as wstring * 100
  dim tempfontStyle as Integer
  dim tempfontsize as Double
  dim tempfontsizepage as Double
  Dim temphfstyle as long
  dim tempcolor as COLORREF
  dim SavePen as hGDIOBJ
  dim rx as single, ry as single, rx2 as single, ry2 as SINGLE
  dim rrh as long, rrw as long ,rs as Long, rc as Long
  dim as long temp,temp1,temp2,temp3,temp4
  dim TransparentFlag as LONG
  dim ReadCount as Long
  dim stable as String
  REdim tt(5,5) as TableCell
  DIM rSize AS Size
  'For Arrows
  dim LineVector as Point, Origin as Point
  dim pts(10) as Point
  Dim MyRegion as hRgn
  Dim nRadius as INTEGER
  Dim AS double IncidentAngle
  
  
  'Read the page twice to incorporate grid changes - the grid is in the background
  for ReadCount = 1 to 2
     
  EZP.SorP = SorP
  CurrentPointer = EZP.PagePtrs(EZP.CurrentPage)
  FileSize = EZP.FileSize
  SetFilePointer(hFile,CurrentPointer,NULL,FILE_BEGIN)
  if EZP.SorP = 0 THEN
       MyDC = EZP.memDC
       Mybrush = GetStockObject(WHITE_BRUSH)
       SelectObject EZP.memDC ,  Mybrush
       PatBlt EZP.memDC, 0, 0, EZP.CanvasRight, EZP.CanvasBottom,  PATCOPY
       DeleteObject Mybrush
    Else
       MyDC = xPrint.PrinterDC
  END IF
  
  if EZP.SorP = 0 THEN
     if EZP.InchesOrCm = 0 THEN
          ScaleFactorX = EZP.ScreenPPIx
          ScaleFactorY = EZP.ScreenPPIy
       Else
          ScaleFactorX = EZP.ScreenPPIx/1   '2.54       'XPrint.PPIx/EZP.ScreenPPIx
          ScaleFactorY = EZP.ScreenPPIy/1   '2.54
     END IF
     iPPIX = EZP.ScreenPPIx
    else 
      if EZP.InchesOrCm = 0 THEN
          ScaleFactorX = XPrint.PPIx
          ScaleFactorY = XPrint.PPIy
       Else
          ScaleFactorX = XPrint.PPIx/1     '2.54
          ScaleFactorY = XPrint.PPIy/1     '2.54
     END IF
     IPPIX = XPrint.PPIx
   END IF
    
  if hFile <> INVALID_HANDLE_VALUE THEN
     'Create Footer, Header and Margin Objects
     lb.lbstyle = BS_SOLID
     lb.lbcolor = EZP_SILVER 
     
     if MyPen <> 0 then  DeleteObject MyPen
     if MyPen2 <> 0 then   DeleteObject MyPen2
     if MyPen3 <> 0 then   DeleteObject MyPen3
     if MyBrush <> 0 then   DeleteObject MyBrush
     
     MyBrush=CreateBrushIndirect(@lb)
     MyPen = CreatePen(EZP_PDOT,1,EZP_GAINSBORO)
     MyPen2 = CreatePen(EZP_PSOLID,1,EZP_GAINSBORO)
     MyPen3 = CreatePen(EZP_PDOT,2,EZP_GAINSBORO)
     
     'Do the Grid
     
 ' print "Col R " ; EZP.MyGridCols(1),EZP.MyGridCols(2),EZP.MyGridCols(3),EZP.MyGridCols(4),EZP.MyGridCols(5),EZP.MyGridCols(6)
 ' print "Row R " ;EZP.MyGridRows(1),EZP.MyGridRows(2),EZP.MyGridRows(3),EZP.MyGridRows(4),EZP.MyGridRows(5),EZP.MyGridRows(6)
  
  
     if EZP.SorP = 0 and EZP.GridShowToggle <> 0 THEN
        if EZP.MyGridCols(0) <> 0 THEN
           x2 = EZP.MyGridCols(0)
           y2 = EZP.MyGridRows(0)
           SavePen = SelectObject(MyDC,MyPen2)
         
           m = 0
           for x= 1 to x2
              m = m + EZP.MyGridCols(x) * ScaleFactorX
              if m <= Xprint.PaperWidth * (EZP.ScreenPPIx/XPrint.PPIx) THEN
                  MoveToEx MyDC,m,0,@lp
                  LineTo MyDC,m,(Xprint.PaperHeight * (EZP.ScreenPPIy/XPrint.PPIy))
              END IF    
           NEXT x
           
          ' print EZP.MyGridCols(1)
           
           m = 0
           for y= 1 to y2
              m = m + EZP.MyGridRows(y) * ScaleFactorY
              if m <= Xprint.PaperHeight * (EZP.ScreenPPIy/XPrint.PPIy) THEN    
                 MoveToEx MyDC,0,m,@lp 
                 LineTo MyDC,Xprint.PaperWidth * (EZP.ScreenPPIx/XPrint.PPIx),m
              end if          
           next y
           
           SelectObject MyDC,SavePen
        END IF         
     end if   
     
     'Do the Printer Margins
     if EZP.SorP = 0 and EZP.MarginsShowToggle <> 0 THEN
         SavePen = SelectObject(MyDC,MyPen)
         m = (Xprint.MarginWidth) * (EZP.ScreenPPIx/XPrint.PPIx)
         
         MoveToEx MyDC,m,0,@lp
         LineTo MyDC,m,(Xprint.PaperHeight * (EZP.ScreenPPIy/XPrint.PPIy))
         
         MoveToEx MyDC,(Xprint.PaperWidth * (EZP.ScreenPPIx/XPrint.PPIx) - m),0,@lp 
         LineTo MyDC,(Xprint.PaperWidth * (EZP.ScreenPPIx/XPrint.PPIx) - m),(Xprint.PaperHeight * (EZP.ScreenPPIy/XPrint.PPIy))

         MoveToEx MyDC,0,m,@lp 
         LineTo MyDC,Xprint.PaperWidth * (EZP.ScreenPPIx/XPrint.PPIx),m

         MoveToEx MyDC,0,(Xprint.PaperHeight * (EZP.ScreenPPIy/XPrint.PPIy))-m,@lp 
         LineTo MyDC,Xprint.PaperWidth * (EZP.ScreenPPIx/XPrint.PPIx),(Xprint.PaperHeight * (EZP.ScreenPPIy/XPrint.PPIy))-m
         
         SelectObject MyDC,SavePen
        
     END IF
     
     'Do the User Margins
     if EZP.SorP = 0 and EZP.UserMarginsShowToggle <> 0 THEN
         SavePen = SelectObject(MyDC,MyPen3)
         
         m = EZP.ScreenMargins.nTop * ScaleFactorX   
         MoveToEx MyDC,0,m,@lp 
         LineTo MyDC,Xprint.PaperWidth * (EZP.ScreenPPIx/XPrint.PPIx),m
         
         m = Xprint.PaperHeight * (EZP.ScreenPPIy/XPrint.PPIy)- EZP.ScreenMargins.nBottom * ScaleFactorX   
         MoveToEx MyDC,0,m,@lp 
         LineTo MyDC,Xprint.PaperWidth * (EZP.ScreenPPIx/XPrint.PPIx),m
        
         m = EZP.ScreenMargins.nLeft * ScaleFactorY 
         MoveToEx MyDC,m,0,@lp
         LineTo MyDC,m,(Xprint.PaperHeight * (EZP.ScreenPPIy/XPrint.PPIy))
         
         m = Xprint.PaperWidth * (EZP.ScreenPPIx/XPrint.PPIx)- EZP.ScreenMargins.nRight * ScaleFactorY          
         MoveToEx MyDC,m,0,@lp
         LineTo MyDC,m,(Xprint.PaperHeight * (EZP.ScreenPPIy/XPrint.PPIy))
    
         SelectObject MyDC,SavePen
        
     END IF
     
     'Do the Headers and Footers if they exist            
     if len(EZP.Header) > 0 THEN
         tempcolor = GetTextColor(MyDC)
         SetTextColor MyDC,EZP_BLACK
         EZP_ParseW(EZP.Header,1,varptr(MyTextIn),"|")
         EZP_ParseW(EZP.Header,2,varptr(tempfontname),"|")
         EZP_ParseW(EZP.Header,3,varptr(MyTextOut),"|") : tempfontStyle = val(MyTextOut)
         EZP_ParseW(EZP.Header,4,varptr(MyTextOut),"|") : tempfontSize = val(MyTextOut)
         
         EZP_ParseW(EZP.Header,5,varptr(MyTextOut),"|") : rx = val(MyTextOut)
         EZP_ParseW(EZP.Header,6,varptr(MyTextOut),"|") : ry = val(MyTextOut)
         EZP_ParseW(EZP.Header,7,varptr(MyTextOut),"|") : rx2 = val(MyTextOut)
         EZP_ParseW(EZP.Header,8,varptr(MyTextOut),"|") : ry2 = val(MyTextOut)
         
         EZP_ParseW(EZP.Header,9,varptr(MyTextOut),"|") : temphfstyle = val(MyTextOut)
         EZP_ParseW(EZP.Header,10,varptr(MyTextOut),"|") : tempfontsizepage = val(MyTextOut)
         
         hFont= EZP_CreateAngleFont(MyDC,tempfontname,tempfontStyle,tempfontSize)
         SavedFont = SelectObject(MyDC,hFont)
'        
         DrawRect.Top = ry * ScaleFactorY : DrawRect.Left = rx * ScaleFactorX 
         DrawRect.Bottom = ry2 * ScaleFactorY : DrawRect.Right = rx2 * ScaleFactorX
         
         DrawTextW MyDC,varptr(MyTextIn), len(MyTextIn),@DrawRect,DT_CENTER or DT_TOP or DT_SINGLELINE
         
         if EZP.HeaderFooterShowToggle <> 0 and EZP.SorP = 0 then FrameRect(MyDC,@DrawRect,MyBrush)
         
         if tempfontsizepage <> 0 THEN
            DeleteObject hFont : hFont = 0
            hFont= EZP_CreateAngleFont(MyDC,tempfontname,tempfontStyle,tempfontsizepage)
            SelectObject(MyDC,hFont)
            
            IF (temphfstyle AND EZP_PAGENUMLEFT) = EZP_PAGENUMLEFT THEN
                MyTextIn = wstr(EZP.CurrentPage)
                DrawRect.Left = DrawRect.Left 
                DrawTextW MyDC,varptr(MyTextIn), len(MyTextIn),@DrawRect,DT_LEFT or DT_BOTTOM or DT_SINGLELINE
            END IF
            
            IF (temphfstyle AND EZP_DATELEFT) = EZP_DATELEFT THEN
                MyTextIn = wstr(EZP.HeaderFooterDate)
                DrawRect.Left = DrawRect.Left 
                DrawTextW MyDC,varptr(MyTextIn), len(MyTextIn),@DrawRect,DT_LEFT or DT_BOTTOM or DT_SINGLELINE
            END IF
            
            IF (temphfstyle AND EZP_PAGENUMRIGHT) = EZP_PAGENUMRIGHT THEN
                MyTextIn = wstr(EZP.CurrentPage)
                DrawRect.Right = DrawRect.Right
                DrawTextW MyDC,varptr(MyTextIn), len(MyTextIn),@DrawRect,DT_RIGHT or DT_BOTTOM or DT_SINGLELINE
            END IF
            
            IF (temphfstyle AND EZP_DATERIGHT) = EZP_DATERIGHT THEN
                MyTextIn = wstr(EZP.HeaderFooterDate)
                DrawRect.Right = DrawRect.Right
                DrawTextW MyDC,varptr(MyTextIn), len(MyTextIn),@DrawRect,DT_RIGHT or DT_BOTTOM or DT_SINGLELINE
            END IF
            
         END IF

         if SavedFont <> 0 THEN
          SelectObject(MyDC,SavedFont) : SavedFont = 0
          DeleteObject hFont : hFont = 0
         END IF
         SetTextColor MyDC,tempcolor
   
     END IF
     
     if len(EZP.Footer) > 0 THEN
         tempcolor = GetTextColor(MyDC)
         SetTextColor MyDC,EZP_BLACK
         EZP_ParseW(EZP.Footer,1,varptr(MyTextIn),"|")
         EZP_ParseW(EZP.Footer,2,varptr(tempfontname),"|")
         EZP_ParseW(EZP.Footer,3,varptr(MyTextOut),"|") : tempfontStyle = val(MyTextOut)
         EZP_ParseW(EZP.Footer,4,varptr(MyTextOut),"|") : tempfontSize = val(MyTextOut)
         
         EZP_ParseW(EZP.Footer,5,varptr(MyTextOut),"|") : rx = val(MyTextOut)
         EZP_ParseW(EZP.Footer,6,varptr(MyTextOut),"|") : ry = val(MyTextOut)
         EZP_ParseW(EZP.Footer,7,varptr(MyTextOut),"|") : rx2 = val(MyTextOut)
         EZP_ParseW(EZP.Footer,8,varptr(MyTextOut),"|") : ry2 = val(MyTextOut)
         
         EZP_ParseW(EZP.Footer,9,varptr(MyTextOut),"|") : temphfstyle = val(MyTextOut)
         EZP_ParseW(EZP.Footer,10,varptr(MyTextOut),"|") : tempfontsizepage = val(MyTextOut)
         
         hFont= EZP_CreateAngleFont(MyDC,tempfontname,tempfontStyle,tempfontSize)
         SavedFont = SelectObject(MyDC,hFont)
'        
         DrawRect.Top = ry * ScaleFactorY : DrawRect.Left = rx * ScaleFactorX 
         DrawRect.Bottom = ry2 * ScaleFactorY : DrawRect.Right = rx2 * ScaleFactorX
         
         DrawTextW MyDC,varptr(MyTextIn), len(MyTextIn),@DrawRect,DT_CENTER or DT_BOTTOM or DT_SINGLELINE 
         
         if EZP.HeaderFooterShowToggle <> 0 and EZP.SorP = 0 then FrameRect(MyDC,@DrawRect,MyBrush)
         
         if tempfontsizepage <> 0 THEN
            DeleteObject hFont : hFont = 0
            hFont= EZP_CreateAngleFont(MyDC,tempfontname,tempfontStyle,tempfontsizepage)
            SelectObject(MyDC,hFont)
            
            IF (temphfstyle AND EZP_PAGENUMLEFT) = EZP_PAGENUMLEFT THEN
                MyTextIn = wstr(EZP.CurrentPage)
                DrawRect.Left = DrawRect.Left
                DrawTextW MyDC,varptr(MyTextIn), len(MyTextIn),@DrawRect,DT_LEFT or DT_TOP  or DT_SINGLELINE
            END IF
            
            IF (temphfstyle AND EZP_DATELEFT) = EZP_DATELEFT THEN
                MyTextIn = wstr(EZP.HeaderFooterDate)
                DrawRect.Left = DrawRect.Left 
                DrawTextW MyDC,varptr(MyTextIn), len(MyTextIn),@DrawRect,DT_LEFT or DT_TOP or DT_SINGLELINE
            END IF
            
            IF (temphfstyle AND EZP_PAGENUMRIGHT) = EZP_PAGENUMRIGHT THEN
                MyTextIn = wstr(EZP.CurrentPage)
                DrawRect.Right = DrawRect.Right 
                DrawTextW MyDC,varptr(MyTextIn), len(MyTextIn),@DrawRect,DT_RIGHT or DT_TOP or DT_SINGLELINE
            END IF
            
            IF (temphfstyle AND EZP_DATERIGHT) = EZP_DATERIGHT THEN
                MyTextIn = wstr(EZP.HeaderFooterDate)
                DrawRect.Right = DrawRect.Right 
                DrawTextW MyDC,varptr(MyTextIn), len(MyTextIn),@DrawRect,DT_RIGHT or DT_TOP or DT_SINGLELINE
            END IF
            
         END IF

         if SavedFont <> 0 THEN
          SelectObject(MyDC,SavedFont) : SavedFont = 0
          DeleteObject hFont : hFont = 0
         END IF
         SetTextColor MyDC,tempcolor
     END IF
     
 
      
  ' End of Headers, Footers and Margins    
    ' dim loopcount as long
     do while CurrentPointer < FileSize
              'loopcount = loopcount + 1
              ReadFile(hFile,@doc,Sizeof(doc),cast(LPDWORD,@NumBytes),NULL)
              'print loopcount,EZP.GridCellBottom,EZP.GridCellTop
                
              if doc.MyAction=EZP_TEXT THEN
                if doc.MyAngle > 0 THEN
                  hFont= EZP_CreateAngleFont(MyDC,EZP.ScreenFontName,EZP.ScreenFontStyle,EZP.ScreenFontSize,doc.MyAngle)
                  SavedFont = SelectObject(MyDC,hFont) 
                END IF
                if doc.TextLen > 0 THEN
                  ZeroMemory (@TextwStr, Sizeof(TextwStr))
                  ReadFile(hFile,varptr(TextwStr),doc.TextLen * 2,cast(LPDWORD,@NumBytes),NULL)
                  
                  lb.lbstyle = BS_SOLID
                  lb.lbcolor = doc.MyColor1 
                  MyBrush=CreateBrushIndirect(@lb) 'Border Color
                  
                  lb.lbstyle = BS_SOLID
                  if doc.MyColor2 = 0 then lb.lbcolor = EZP_WHITE else lb.lbcolor = doc.MyColor2
                  MyBrush2=CreateBrushIndirect(@lb) 'Fill Color
                  
  
                  x=doc.MyXPos * ScaleFactorX : y=doc.MyYPos * ScaleFactorY
                  x2=doc.MyXPos2 * ScaleFactorX : y2=doc.MyYPos2 * ScaleFactorY
                  
                  if x2=0 or y2=0 THEN
                       TextOutW(MyDC,x,y,varptr(TextwStr),doc.TextLen)
                     else
                        if doc.MyAngle = 0 THEN
                           DrawRect.Top = y : DrawRect.Left = x : DrawRect.Bottom = y2 : DrawRect.Right = x2
                           
                           DrawTextW MyDC,varptr(TextwStr), len(TextwStr),@DrawRect,abs(doc.MyStyle) or DT_WORDBREAK or DT_CALCRECT 'Calc the rect size
                           FillRect MyDC,@DrawRect,MyBrush2 'Fill the next rect size
                           DrawTextW MyDC,varptr(TextwStr), len(TextwStr),@DrawRect,abs(doc.MyStyle) or DT_WORDBREAK ' Draw the text to the proper box
                           if doc.MyStyle < 0 THEN
                              FrameRect(MyDC,@DrawRect,MyBrush) 
                           END IF
                         
                        END IF  
                  END IF
                 
                END IF
                if SavedFont <> 0 THEN
                   SelectObject(MyDC,SavedFont)
                   DeleteObject hFont : hFont = 0
                END IF
                
                if MyBrush <> 0 THEN DeleteObject MyBrush
                if MyBrush2 <> 0 THEN DeleteObject MyBrush2   
                ZeroMemory (@TextwStr, Sizeof(TextwStr))
              END IF
              
              if doc.MyAction=EZP_TXTR THEN 'Fill the Test Buffer
                if doc.TextLen > 0 THEN
                    if EZP.TextPtr <> 0 then Deallocate EZP.TextPtr : EZP.TextPtr = 0 : EZP.TextPtrLen = 0 : EZP.TextPtrPos = 1
                    EZP.TextPtr = cast(wstring ptr,CAllocate((doc.TextLen + 4 ) * Len(WString)))  'Pad by 8 bytes for safety
                   
                    EZP.TextPtrLen = doc.TextLen
                    ReadFile(hFile,EZP.TextPtr,EZP.TextPtrLen * 2,cast(LPDWORD,@NumBytes),NULL)
 
                end if
              END IF
              
              if doc.MyAction=EZP_NWPG THEN
                 Goto EndofPage
              end if

              if doc.MyAction=EZP_FONT THEN
                if doc.TextLen > 0 THEN
                 ZeroMemory (@TextwStr, Sizeof(TextwStr))
                 ReadFile(hFile,varptr(TextwStr),doc.TextLen * 2,cast(LPDWORD,@NumBytes),NULL)
                
                 hFont= EZP_CreateAngleFont(MyDC,TextwStr,doc.MyStyle,doc.MySize,0)
                 
                 if hFont <> 0 THEN
                   if EZP.ScreenFont <> 0 then DeleteObject EZP.ScreenFont
                 end if    
                 
                 EZP.ScreenFont = hFont
                 EZP.ScreenFontName = TextwStr
                 EZP.ScreenFontStyle = doc.MyStyle
                 EZP.ScreenFontSize = doc.MySize
                 EZP.ScreenFontAngle = 0
                 EZP.ScreenFontColor = doc.MyColor1
                 
                 if hFont <> 0 THEN
                   SelectObject MyDC,hFont
                   SetTextColor MyDC,doc.MyColor1
                   SetBkMode MyDC,TRANSPARENT  'OPAQUE
                 END IF
                end if
              END IF
              
              if doc.MyAction=EZP_ARC1 THEN
                 x=doc.MyXPos * ScaleFactorX : y=doc.MyYPos * ScaleFactorY
                 x2=doc.MyXPos2 * ScaleFactorX : y2=doc.MyYPos2 * ScaleFactorY 
                 if doc.TextLen > 0 THEN
                    ZeroMemory (@TextwStr, Sizeof(TextwStr))
                    ReadFile(hFile,varptr(TextwStr),doc.TextLen * 2,cast(LPDWORD,@NumBytes),NULL)
                    'Make the arc  rx as single, ry as single, rx2 as single, ry2 as SINGLE
                    EZP_ParseW(TextwStr,1,varptr(MyTextIn),",") 
                    rx = val(MyTextIn)* ScaleFactorX 
                    EZP_ParseW(TextwStr,2,varptr(MyTextIn),",") 
                    ry = val(MyTextIn)* ScaleFactorY
                    EZP_ParseW(TextwStr,3,varptr(MyTextIn),",") 
                    rx2 = val(MyTextIn)* ScaleFactorX
                    EZP_ParseW(TextwStr,4,varptr(MyTextIn),",") 
                    ry2 = val(MyTextIn)* ScaleFactory
                 end if
                 MyPen = CreatePen(doc.MyStyle,doc.MyWidth,doc.MyColor1)
                 SelectObject MyDC,MyPen
                 Arc(MyDC,x,y,x2,y2,rx,ry,rx2,ry2)
                 DeleteObject MyPen : MyPen = 0
              END IF
              
              if doc.MyAction=EZP_SMBL THEN
                 x=doc.MyXPos * ScaleFactorX : y=doc.MyYPos * ScaleFactorY
                 MyPen = CreatePen(EZP_PSOLID,1,doc.MyColor1)
                 SelectObject MyDC,MyPen
                 if doc.MyColor2 <> -123 THEN
                   lb.lbstyle=EZP_BSOLID
                   lb.lbcolor=doc.MyColor2 
                   MyBrush=CreateBrushIndirect(@lb)
                   SelectObject MyDC,MyBrush 
                  else
                   SelectObject MyDC,GetStockObject(HOLLOW_BRUSH)    
                 END IF
                 rs = doc.MySize : if rs < 1 then rs = 1 : if rs > 100 then rs = 100
                 if EZP.SorP = 1 then rs = rs * (XPrint.PPIx/EZP.ScreenPPIx)
                 redim pts(10) as Point
                 Select CASE doc.MyStyle
                    case EZP_SYMBOL_RECT
                       rc = 4
                       pts(0).x = x-rs : pts(0).y = y-rs
                       pts(1).x = x+rs : pts(1).y = y-rs
                       pts(2).x = x+rs : pts(2).y = y+rs
                       pts(3).x = x-rs : pts(3).y = y+rs
                       Polygon(MyDC,@pts(0), rc)
                     case EZP_SYMBOL_DIAMOND
                       rc = 4
                       pts(0).x = x-0 : pts(0).y = y-rs
                       pts(1).x = x+rs : pts(1).y = y-0
                       pts(2).x = x+0 : pts(2).y = y+rs
                       pts(3).x = x-rs : pts(3).y = y+0
                       Polygon(MyDC,@pts(0), rc) 
                     case EZP_SYMBOL_TRIRIGHT
                       rc = 3
                       pts(0).x = x-rs : pts(0).y = y-rs
                       pts(1).x = x+rs : pts(1).y = y-0
                       pts(2).x = x-rs : pts(2).y = y+rs
                       Polygon(MyDC,@pts(0), rc) 
                     case EZP_SYMBOL_TRILEFT
                       rc = 3
                       pts(0).x = x+rs : pts(0).y = y-rs
                       pts(1).x = x+rs : pts(1).y = y+rs
                       pts(2).x = x-rs : pts(2).y = y+0
                       Polygon(MyDC,@pts(0), rc) 
                     case EZP_SYMBOL_TRIDOWN
                       rc = 3
                       pts(0).x = x-rs : pts(0).y = y-rs
                       pts(1).x = x+rs : pts(1).y = y-rs
                       pts(2).x = x+0 : pts(2).y = y+rs
                       Polygon(MyDC,@pts(0), rc) 
                     case EZP_SYMBOL_TRIUP
                       rc = 3
                       pts(0).x = x-0 : pts(0).y = y-rs
                       pts(1).x = x+rs : pts(1).y = y+rs
                       pts(2).x = x-rs : pts(2).y = y+rs
                       Polygon(MyDC,@pts(0), rc)
                     case EZP_SYMBOL_CIRCLE
                       Ellipse MyDC,x-rs ,y-rs,x+rs,y+rs 
                     case EZP_SYMBOL_STAR  
                       EZP_Star(MyDC,10,rs,x,y)      
                    CASE ELSE
                 END SELECT
                                
                 DeleteObject MyPen : MyPen = 0
                 if MyBrush <> 0 THEN DeleteObject MyBrush  : MyBrush = 0
              END IF
              
              if doc.MyAction=EZP_PIE1 THEN
                 x=doc.MyXPos * ScaleFactorX : y=doc.MyYPos * ScaleFactorY
                 x2=doc.MyXPos2 * ScaleFactorX : y2=doc.MyYPos2 * ScaleFactorY 
                 if doc.TextLen > 0 THEN
                    ZeroMemory (@TextwStr, Sizeof(TextwStr))
                    ReadFile(hFile,varptr(TextwStr),doc.TextLen * 2,cast(LPDWORD,@NumBytes),NULL)
                    'Make the arc  rx as single, ry as single, rx2 as single, ry2 as SINGLE
                    EZP_ParseW(TextwStr,1,varptr(MyTextIn),",") 
                    rx = val(MyTextIn)* ScaleFactorX 
                    EZP_ParseW(TextwStr,2,varptr(MyTextIn),",") 
                    ry = val(MyTextIn)* ScaleFactorY
                    EZP_ParseW(TextwStr,3,varptr(MyTextIn),",") 
                    rx2 = val(MyTextIn)* ScaleFactorX
                    EZP_ParseW(TextwStr,4,varptr(MyTextIn),",") 
                    ry2 = val(MyTextIn)* ScaleFactory
                 end if
                 MyPen = CreatePen(doc.MyStyle,doc.MyWidth,doc.MyColor1)
                 SelectObject MyDC,MyPen
                 if doc.MyColor2 <> -123 THEN
                   lb.lbstyle=EZP_BSOLID
                   lb.lbcolor=doc.MyColor2 
                   MyBrush=CreateBrushIndirect(@lb)
                   SelectObject MyDC,MyBrush
                  else
                   SelectObject MyDC,GetStockObject(HOLLOW_BRUSH) 
                 END IF
                 Pie(MyDC,x,y,x2,y2,rx,ry,rx2,ry2)
                 DeleteObject MyPen : MyPen = 0
                 if MyBrush <> 0 THEN DeleteObject MyBrush : MyBrush = 0
              END IF

              
              if doc.MyAction=EZP_LINE THEN
                 x=doc.MyXPos * ScaleFactorX : y=doc.MyYPos * ScaleFactorY
                 x2=doc.MyXPos2 * ScaleFactorX : y2=doc.MyYPos2 * ScaleFactorY 
                 MyPen = CreatePen(doc.MyStyle,doc.MyWidth,doc.MyColor1)
                 SelectObject MyDC,MyPen
                 MoveToEx MyDC,x,y,@lp
                 LineTo MyDC,x2,y2
                 DeleteObject MyPen  : MyPen = 0               
              END IF
                         
              if doc.MyAction=EZP_GRID THEN   'Grid for page positioning
                 'The grid is stored in inches
                  ' print "Xpos  "; doc.MyXPos
                 Select CASE doc.MyStyle
                    case EZP_GRIDNEW
                         'Erase EZP.MyGridCols, EZP.MyGridRows
                         EZP.MyGridCols(0) = EZP_MAXNUMCOLS
                         EZP.MyGridRows(0) = EZP_MAXNUMROWS
                         if EZP.InchesOrCm <> 0 THEN rx=2.53999 else rx = 1
                         'if EZP.InchesOrCm <> 0 THEN rx=1 else rx = 1
                         for xx = 1 to EZP.MyGridCols(0)
                            EZP.MyGridCols(xx) = doc.MyXPos / rx
                             
                         NEXT xx
                         for yy = 1 to EZP.MyGridRows(0)
                            EZP.MyGridRows(yy) = doc.MyYPos / rx
                         NEXT yy
                         
                         ' print "Page  "; EZP.MyGridCols(1)
                          'print "Inches  "; EZP.InchesOrCm 
                              
                    Case EZP_GRIDMOD
                         if EZP.InchesOrCm <> 0 THEN rx=2.53999 else rx = 1
                         'if EZP.InchesOrCm <> 0 THEN rx=1 else rx = 1
                         if doc.MyXPos > 0 and doc.MyXPos <= EZP.MyGridCols(0) and doc.MyXPos2 > 0 THEN
                            EZP.MyGridCols(doc.MyXPos) = doc.MyXPos2 / rx
                         end if
                         if doc.MyYPos > 0 and doc.MyYPos <= EZP.MyGridRows(0)and doc.MyYPos2 > 0 THEN
                            EZP.MyGridRows(doc.MyYPos) = doc.MyYPos2 / rx
                         end if  
                    case EZP_GRIDMARGINS
                         if EZP.InchesOrCm <> 0 THEN rx=2.53999 else rx = 1
                         'if EZP.InchesOrCm <> 0 THEN rx=1 else rx = 1
                         if doc.MyXPos > 0 and doc.MyXPos <= EZP_MAXNUMCOLS and doc.MyXPos > 0 then  EZP.ScreenMargins.nLeft = doc.MyXPos / rx 
                         if doc.MyYPos > 0 and doc.MyYPos <= EZP_MAXNUMROWS and doc.MyYPos > 0 then EZP.ScreenMargins.nTop = doc.MyYPos / rx
                         if doc.MyXPos2 > 0 and doc.MyXPos2 <= EZP_MAXNUMCOLS and doc.MyXPos2 > 0 then  EZP.ScreenMargins.nRight = doc.MyXPos2 / rx 
                         if doc.MyYPos2 > 0 and doc.MyYPos2 <= EZP_MAXNUMROWS and doc.MyYPos2 > 0 then EZP.ScreenMargins.nBottom = doc.MyYPos2 / rx
                    CASE ELSE
                       'Nothing for now
                 END SELECT
                 
              END IF
              
              if doc.MyAction=EZP_POLY THEN   'Polygon

                 if doc.TextLen > 0 THEN
                    ZeroMemory (@TextwStr, Sizeof(TextwStr))
                    ReadFile(hFile,varptr(TextwStr),doc.TextLen * 2,cast(LPDWORD,@NumBytes),NULL)
                    'Make the polygon
                    x= doc.MySize
                    x2 = 1 'text parse counter
                    redim pts(x) as Point
                    for y = 0 to x-1
                       EZP_ParseW(TextwStr,x2,varptr(MyTextIn),",")
                       x2 = x2 + 1
                       pts(y).x = val(MyTextIn)* ScaleFactorX
                       EZP_ParseW(TextwStr,x2,varptr(MyTextIn),",")
                       x2 = x2 + 1
                       pts(y).y = val(MyTextIn)* ScaleFactorY  
                    NEXT
                    
                    MyPen = CreatePen(EZP_PSOLID,1,doc.MyColor1)
                    SelectObject MyDC,MyPen
                    if doc.MyColor2 <> -123 THEN
                      lb.lbstyle=EZP_BSOLID
                      lb.lbcolor=doc.MyColor2 
                      MyBrush=CreateBrushIndirect(@lb)
                      SelectObject MyDC,MyBrush
                     else
                      SelectObject MyDC,GetStockObject(HOLLOW_BRUSH) 
                    END IF
                    Polygon(MyDC,@pts(0), x)
                    DeleteObject MyPen : MyPen = 0
                    if MyBrush <> 0 THEN DeleteObject MyBrush  : MyBrush = 0
                    
                 end if
              END IF
              
'               const EZP_ARROWNONE   = 0
'               const EZP_ARROWEND    = 1
'               const EZP_ARROWSTART  = 2
'               const EZP_ARROWBOTH   = 3
'               const EZP_ARROWAUTO   = 4
              
              if doc.MyAction=EZP_SHAP THEN   
                 x=doc.MyXPos * ScaleFactorX : y=doc.MyYPos * ScaleFactorY
                 x2=doc.MyXPos2 * ScaleFactorX : y2=doc.MyYPos2 * ScaleFactorY 
                 if doc.TextLen > 0 THEN
                    ZeroMemory (@TextwStr, Sizeof(TextwStr))
                    ReadFile(hFile,varptr(TextwStr),doc.TextLen * 2,cast(LPDWORD,@NumBytes),NULL)
                    'Make the Arrow
                    
                     MyPen = CreatePen(EZP_PSOLID,1,doc.MyColor1)
                     SelectObject MyDC,MyPen
                     'if doc.MyColor2 <> -123 THEN
                      lb.lbstyle=EZP_BSOLID
                      lb.lbcolor=doc.MyColor2 
                      MyBrush=CreateBrushIndirect(@lb)
                      SelectObject MyDC,MyBrush
                      'else
'                      lb.lbstyle=EZP_BSOLID
'                      lb.lbcolor=EZP_WHITE 
'                      MyBrush=CreateBrushIndirect(@lb)
'                      SelectObject MyDC,MyBrush
                      
                      'SelectObject MyDC,GetStockObject(HOLLOW_BRUSH)   
                     'END IF
                     
                      dim a as single, b as single,c as single, d as single,e as single,f as single, dist as Single
                      dim myunit as single,myunit2 as single, rx as single,ry as Single,g as SINGLE
                      redim pts(6) as Point
                     
                     select CASE doc.MyStyle
                       case EZP_ARROWAUTO
                          
                           a = x   :  b = y
                           c = x2  :  d = y2
                           e = a + (c-a) * 2/3 :  f = b + (d-b) * 2/3

                           rx = a-c   : ry = b-d
                           dist = SQR(rx*rx + ry*ry)
                           rx /= dist : ry /= dist
                           myunit = 1/12 * dist
                          
                           pts(0).x = a+myunit*ry   : pts(0).y = b-myunit*rx
                           pts(1).x = e+myunit*ry   : pts(1).y = f-myunit*rx
                           pts(2).x = e+2*myunit*ry : pts(2).y = f-2*myunit*rx
                           pts(3).x = c           : pts(3).y = d
                           pts(4).x = e-2*myunit*ry : pts(4).y = f+2*myunit*rx
                           pts(5).x = e-myunit*ry   : pts(5).y = f+myunit*rx
                           pts(6).x = a-myunit*ry   : pts(6).y = b+myunit*rx
                           Polygon(MyDC,@pts(0), 7)
                                                      
                        case EZP_ARROWBOTH,EZP_ARROWEND ,EZP_ARROWSTART
                                x=doc.MyXPos * ScaleFactorX : y=doc.MyYPos * ScaleFactorY
                                x2=doc.MyXPos2 * ScaleFactorX : y2=doc.MyYPos2 * ScaleFactorY
                                     
                                 if SorP <> 0 THEN
                                     x = x / XPrint.PPIx : y= y / XPrint.PPIy
                                     x2 = x2 / XPrint.PPIx : y2= y2 / XPrint.PPIy
                                     
                                     x = x  * EZP.ScreenPPIx : y = y * EZP.ScreenPPIy
                                     x2 = x2 * EZP.ScreenPPIx : y2= y2 * EZP.ScreenPPIy
                                 END IF
                                          
                             EZP_DrawArrow(MyDC,doc.MyStyle,x,y,x2,y2,doc.MyColor2,doc.MySize)
                            
                       CASE ELSE
                          
                    END SELECT
                    
                    
                    
                    
                    DeleteObject MyPen : MyPen = 0
                    if MyBrush <> 0 THEN DeleteObject MyBrush : MyBrush = 0
                 end if
              END IF
              
              if doc.MyAction=EZP_TABL THEN   'Table 
                 x=doc.MyXPos * ScaleFactorX : y=doc.MyYPos * ScaleFactorY
                 x2=doc.MyXPos2 * ScaleFactorX : y2=doc.MyYPos2 * ScaleFactorY 
                 if doc.TextLen > 0 THEN
                    stable = space(doc.TextLen)
                    xx= doc.MyAngle -1 : yy= doc.MySize - 1
                    Redim tt(xx,yy) as TableCell
                    ReadFile(hFile,strptr(stable),doc.TextLen,cast(LPDWORD,@NumBytes),NULL)
                    CopyMemory(varptr(tt(0,0)),strptr(stable),doc.TextLen)
                    MyPen = CreatePen(PS_SOLID,1,EZP_BLACK)
                    SelectObject MyDC,MyPen
                    
                    MyTextIn = "2222222222222222222222222222222222222222222222222222222"
                    MyTextIn = Mid(MyTextIn,1,doc.MyColor1)
                    GetTextExtentPoint32W MyDC, Varptr(MyTextIn), LEN(MyTextIn), @rSize
                    
                    Dim LeftRightSlack as long = 4 
                    dim CXChar as long = (rSize.cx/doc.MyColor1) '
                 
                     DrawRect.Top = y :DrawRect.Bottom = DrawRect.Top + rSize.cy 
                     for temp = 0 to yy
                        DrawRect.Left = x : DrawRect.Right = DrawRect.Left + rSize.cx
                        DrawRect.Bottom = DrawRect.Bottom + (tt(0,temp).h * rSize.cy) + CXChar/4  'Average Char Width divided by 4 to add just a bit
                        for temp1 = 0 to xx
                            DrawRect.Right = DrawRect.Right + (tt(temp1,0).w * CXChar)
                            SavedFont = 0
                            if tt(temp1,temp).FontStyle <> 0 THEN
                              hFont= EZP_CreateAngleFont(MyDC,EZP.ScreenFontName,tt(temp1,temp).FontStyle,EZP.ScreenFontSize)
                              SavedFont = SelectObject(MyDC,hFont) 
                            END IF
                            
                            if tt(temp1,temp).MyJustify = EZP_CENTER THEN
                                 just = DT_CENTER
                               elseif tt(temp1,temp).MyJustify = EZP_RIGHT then
                                 just = DT_RIGHT
                               elseif tt(temp1,temp).MyJustify = EZP_LEFT then
                                 just = DT_LEFT  
                               else   
                                 just = DT_CENTER
                            end if
                            
                            just = just or DT_WORDBREAK
                            
                            if tt(temp1,temp).MyCellColor <> 0 THEN
                               lb.lbstyle=EZP_BSOLID
                               lb.lbcolor=tt(temp1,temp).MyCellColor 
                               MyBrush=CreateBrushIndirect(@lb)
                               SelectObject MyDC,MyBrush 
                               FillRect MyDC,@DrawRect,MyBrush
                               DeleteObject MyBrush : MyBrush = 0
                            end if
                            if just = DT_RIGHT or DT_WORDBREAK then DrawRect.Right = DrawRect.Right - LeftRightSlack 
                            if just = DT_LEFT or DT_WORDBREAK then DrawRect.Left = DrawRect.Left + LeftRightSlack
                            DrawTextW MyDC,varptr(tt(temp1,temp).MyText), len(tt(temp1,temp).MyText),@DrawRect,just
                            if just = DT_LEFT or DT_WORDBREAK then DrawRect.Left = DrawRect.Left - LeftRightSlack
                            if just = DT_RIGHT or DT_WORDBREAK then DrawRect.Right = DrawRect.Right + LeftRightSlack
                            if doc.MyStyle = 0 then FrameRect(MyDC,@DrawRect,GetStockObject(BLACK_BRUSH))
                            DrawRect.left = DrawRect.right -1 : DrawRect.Right = DrawRect.right + rSize.cx
                            if SavedFont <> 0 THEN SelectObject(MyDC,SavedFont)
                        NEXT temp1
                        DrawRect.Top = DrawRect.Bottom -1 :DrawRect.Bottom = DrawRect.Top + rSize.cy
                     NEXT temp
                     
                    DeleteObject MyPen : MyPen = 0
                 end if
              END IF
              
              if doc.MyAction=EZP_ELLP THEN 
                 'left top right bottom
                 x=doc.MyXPos * ScaleFactorX : y=doc.MyYPos * ScaleFactorY
                 x2=doc.MyXPos2 * ScaleFactorX : y2=doc.MyYPos2 * ScaleFactorY 
                 lb.lbstyle=doc.MyStyle
                 lb.lbcolor=doc.MyColor1 
                 MyBrush=CreateBrushIndirect(@lb)
                 MyPen = CreatePen(PS_SOLID,doc.MyWidth,doc.MyColor2)
                 SelectObject MyDC,MyPen
                 SelectObject MyDC,MyBrush
                 Ellipse MyDC,x ,y,x2,y2
                 DeleteObject MyPen : MyPen = 0
                 DeleteObject MyBrush : MyBrush = 0
              END IF
              
              if doc.MyAction=EZP_RECT THEN 
                 'left top right bottom
                 x=doc.MyXPos * ScaleFactorX : y=doc.MyYPos * ScaleFactorY
                 x2=doc.MyXPos2 * ScaleFactorX : y2=doc.MyYPos2 * ScaleFactorY
                 rrw = doc.MyAngle
                 if SorP = 1 THEN
                    rrw = rrw * (XPrint.PPIx/EZP.ScreenPPIx)
                 END IF
                 lb.lbstyle=doc.MyStyle
                 lb.lbcolor=doc.MyColor1 
                 MyBrush=CreateBrushIndirect(@lb)
                 MyPen = CreatePen(PS_SOLID,doc.MyWidth,doc.MyColor2)
                 SelectObject MyDC,MyPen
                 SelectObject MyDC,MyBrush
                 if rrw = 0 THEN
                    Rectangle MyDC,x ,y,x2,y2
                   else
                    RoundRect MyDC,x ,y,x2,y2,rrw,rrw   
                 END IF
                 
                 DeleteObject MyPen : MyPen = 0
                 DeleteObject MyBrush : MyBrush = 0
              END IF

              
              if doc.MyAction=EZP_IMGA THEN 
                if doc.TextLen > 0 THEN
                    SetFilePointer(hFile,CurrentPointer + doc.TextLen,NULL,FILE_BEGIN)
                end if
              END IF
              
              if doc.MyAction=EZP_NAME THEN 
                if doc.TextLen > 0 THEN
                    SetFilePointer(hFile,CurrentPointer + doc.TextLen  * 2,NULL,FILE_BEGIN)
                end if
              END IF
              
              if doc.MyAction=EZP_DATE THEN 
                if doc.TextLen > 0 THEN
                    SetFilePointer(hFile,CurrentPointer + doc.TextLen * 2,NULL,FILE_BEGIN)
                end if
              END IF
              
              
              if doc.MyAction=EZP_HEAD THEN 
                if doc.TextLen > 0 THEN
                    SetFilePointer(hFile,CurrentPointer + doc.TextLen * 2,NULL,FILE_BEGIN)
                end if
              END IF
              
              if doc.MyAction=EZP_FOOT THEN 
                if doc.TextLen > 0 THEN
                    SetFilePointer(hFile,CurrentPointer + doc.TextLen * 2,NULL,FILE_BEGIN)
                end if
              END IF
              
              if doc.MyAction=EZP_IMGR and ReadCount = 2 THEN 
                if doc.MyStyle > 0 THEN
                  WhichImage=doc.MyStyle
                  
                      x=doc.MyXPos * ScaleFactorX : y=doc.MyYPos * ScaleFactorY
                      x2=doc.MyXPos2 * ScaleFactorX : y2=doc.MyYPos2 * ScaleFactorY
                      
                  if SorP <> 0 THEN
'                      x = x / XPrint.PPIx : y= y / XPrint.PPIy
'                      x2 = x2 / XPrint.PPIx : y2= y2 / XPrint.PPIy
'                      
'                      x = x  * EZP.ScreenPPIx : y = y * EZP.ScreenPPIy
'                      x2 = x2 * EZP.ScreenPPIx : y2= y2 * EZP.ScreenPPIy
                       
                      'print x,y,x2,y2
                  END IF
                  
                  'x=doc.MyXPos * ScaleFactorX : y=doc.MyYPos * ScaleFactorY
                 ' x2=doc.MyXPos2 * ScaleFactorX : y2=doc.MyYPos2 * ScaleFactorY
                  TransparentFlag =doc.MySize 
                  hBmp=EZP_GetImage(WhichImage)
                  
                  if x2 = 0 and y2= 0 THEN
                       EZP_DrawBitmap(MyDC,hBmp,doc.MyAngle,x,y,0,0,TransparentFlag)
                    else
                       EZP_DrawBitmap(MyDC,hBmp,doc.MyAngle,x,y,x2,y2,TransparentFlag) 
                  END IF                  
                  
                  DeleteObject hBmp
                END IF
              END IF
              
              if doc.MyAction=EZP_ENDD THEN
                 Goto EndofPage
              END IF
              
              CurrentPointer = GetFilePointer(hFile)
        Loop
  end if   
  
  EndofPage:
     
          'Delete Footer, Header and Margin Objects
      if MyPen <> 0 then  DeleteObject MyPen
      if MyPen2 <> 0 then   DeleteObject MyPen2
      if MyPen3 <> 0 then   DeleteObject MyPen3
      if MyBrush <> 0 then   DeleteObject MyBrush
      
   NEXT ReadCount 
   
   'print "TextPtr Value Leave";EZP.TextPtr    
     
  function = MyDocResult 
END FUNCTION

function EZP_PrintDoc(byval hFile as Handle, byval MyOptions as Long,byval MyCopies as long = 1) as long
    dim MyPage as Long, MySavedPage as LONG
    Dim NumCopies as Long
    dim NumPrintRanges as Long
    dim MyCount as Long
    dim MyWStr as WSTRING * 150
    
    'MyOptions = 0 no Dialog  1 with Dialog
    
    if MyOptions = 0 THEN 'No Dialog - just print them all
          for NumCopies = 1 to MyCopies
               MySavedPage = EZP.CurrentPage
               Xprint.DocStart
               
               for MyPage = 1 to EZP.NumPages
                 Xprint.PageStart
                 EZP.CurrentPage = MyPage 
                 EZP_ReadCurrentPage(hFile,1)
                 Xprint.PageEnd 
               NEXT MyPage
               
               Xprint.DocEnd
               
               EZP.CurrentPage = MySavedPage     
               EZP_ReadCurrentPage(hFile,0)   
          NEXT NumCopies
       else  ' With dialog
          'Need to work in the print ranges

         for NumCopies = 1 to EZP.PrintNumCopies
               MySavedPage = EZP.CurrentPage
               Xprint.DocStart
               
               
'                dim PrintPageRanges(10) as PRINTPAGERANGE  'DWORD nFromPage;
'                                             'DWORD nToPage;
               
              if EZP.PrintNumPageRanges = 0 THEN
                  for MyPage = 1 to EZP.NumPages
                    Xprint.PageStart
                    EZP.CurrentPage = MyPage 
                    EZP_ReadCurrentPage(hFile,1)
                    Xprint.PageEnd 
                  NEXT MyPage
                else 'we have ranges
                 for MyCount = 0 to EZP.PrintNumPageRanges - 1
                    
                  if EZP.PrintPageRanges(MyCount).nFromPage < 1 then EZP.PrintPageRanges(MyCount).nFromPage = 1
                  if EZP.PrintPageRanges(MyCount).nFromPage > EZP.NumPages then EZP.PrintPageRanges(MyCount).nFromPage = EZP.NumPages   
                  if EZP.PrintPageRanges(MyCount).nToPage < 1 then EZP.PrintPageRanges(MyCount).nToPage = 1
                  if EZP.PrintPageRanges(MyCount).nToPage > EZP.NumPages then EZP.PrintPageRanges(MyCount).nToPage = EZP.NumPages
                     
                  for MyPage = EZP.PrintPageRanges(MyCount).nFromPage to EZP.PrintPageRanges(MyCount).nToPage
                    Xprint.PageStart
                    EZP.CurrentPage = MyPage 
                    EZP_ReadCurrentPage(hFile,1)
                    Xprint.PageEnd 
                  NEXT MyPage
                   
                 NEXT MyCount
               END IF
               
               Xprint.DocEnd
               
               EZP.CurrentPage = MySavedPage     
               EZP_ReadCurrentPage(hFile,0)
               
          NEXT NumCopies
     
      END IF  
      
    function = 0
END FUNCTION


Function EZP_DoDoc(byval HWND as HWND,byval MyOptions as long, byref TheDoc as Wstring,byval  ViewerOptions as long = 0) as long
 Dim MyDocResult as Long
 dim dh as DOCHEADER
 dim doc as DOCCOMMAND
 dim sFile as string
 dim PageCount as long
 dim BookmarkCount as Long
 dim hFile as handle
 Dim NumBytes as UINTEGER
 dim TextwStr as wString * 255
 dim FileSize as long
 dim CurrentPointer as Long
 Dim lp as point
 DIM MyDoc as wstring * MAX_PATH
 Dim Path   AS WSTRING * MAX_PATH
 Dim f      AS WSTRING * 50
 Dim MyStyle  AS DWORD
 Dim wPrinterName As WString * MAX_PATH
 Dim n As Long = MAX_PATH
 Dim viewflag as Long
 dim tempfontname as wstring * 100
 dim tempfontStyle as Integer
 dim tempfontsize as Double
 dim xx as long, yy as Long
 Dim firstpageflag as Long
 dim rx as single
 dim hFont as hfont
 Dim dcDesktop as hDC
 Dim hDesktop as hWnd
 dim DoDocViewOverride as long
 
 
 
   firstpageflag = 0
   EZP_ResetGlobals 0
   EZP.Zoom = 0
   EZP.Zoomfactor = 1 
   EZP.ScreenMargins.nTop = EZP_GRIDMARGINSDEFAULT
   EZP.ScreenMargins.nLeft = EZP_GRIDMARGINSDEFAULT
   EZP.ScreenMargins.nBottom = EZP_GRIDMARGINSDEFAULT
   EZP.ScreenMargins.nRight = EZP_GRIDMARGINSDEFAULT
   EZP.PrintNumCopies = 1  
   EZP.PrintNumPageRanges = 0
   
   if MyOptions <> 0 THEN
        EZP.PrintDlgFreedom  = 1
   END IF
   
      if EZP.MyGridCols(0) = 0 THEN
          Erase EZP.MyGridCols, EZP.MyGridRows
          EZP.MyGridCols(0) = EZP_MAXNUMCOLS
          EZP.MyGridRows(0) = EZP_MAXNUMROWS
          for xx = 1 to EZP.MyGridCols(0)
             EZP.MyGridCols(xx) = EZP_GRIDCOLDEFAULT 
          NEXT xx
          for yy = 1 to EZP.MyGridRows(0)
             EZP.MyGridRows(yy) = EZP_GRIDROWDEFAULT 
          NEXT yy
      end if 
   
   'Print "Start DoDoc"
''~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
''  Check for a the file to be processed
''~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 MyDoc = TheDoc
  if Len(MyDoc) = 0 THEN
       Path     = ""  ' CURDIR$
       f        = "*.ezp"
       MyStyle    = OFN_FILEMUSTEXIST OR OFN_HIDEREADONLY OR OFN_LONGNAMES

       MyDoc = "*.EZP" 

       IF EZP_OpenFileDialogW(0, "Open File", varptr(MyDoc), "", _
        "Text Files|*.ezp|All Files|*.*", "ezo", MyStyle) THEN
          'Print MyDoc
        else
           MessageBox GetForegroundWindow, "File Invalid - Abort EZP" & CHR$(0), _
                              "PRINT CONTROL MESSAGE" & CHR$(0),  MB_OK 
           MyDocResult = EZP_FAILED
         exit function
       END IF 
  END IF
''~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
''  Check for a Default Printer. One must be installed
''~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
   GetDefaultPrinterW(@wPrinterName,@n)
   if len(wPrinterName) < 1 THEN
          MessageBox GetForegroundWindow, "A default printer must be installed for EZP.  Abort" & CHR$(0), _
                              "PRINT CONTROL MESSAGE" & CHR$(0),  MB_OK
     MyDocResult = EZP_FAILED
     Goto bailDoDoc
   END IF
   
   'Initialize for fields is DoDoc called in isolation
   EZP_FillPaperArray 
   if len(EZP.ControlWords(EZP_PRINT)) = 0 then EZP.ControlWords(EZP_PRINT) = "Print"
   if len(EZP.ControlWords(EZP_CLOSE)) = 0 then EZP.ControlWords(EZP_CLOSE) = "Close"
   if len(EZP.ControlWords(EZP_FITWIN)) = 0 then EZP.ControlWords(EZP_FITWIN) = "Fit to Window"
   if len(EZP.ControlWords(EZP_FITWIDTH)) = 0 then EZP.ControlWords(EZP_FITWIDTH) = "Fit to Width"
   if len(EZP.HeaderFooterDate) = 0 then EZP_SetHeaderFooterDate
   

   if ViewerOptions > 0 then
      IF (ViewerOptions AND EZP_VIEWER_GRID) = EZP_VIEWER_GRID THEN
          EZP.GridShowToggle = 1
      END IF
      
      IF (ViewerOptions AND EZP_VIEWER_MARGINS) = EZP_VIEWER_MARGINS THEN
          EZP.MarginsShowToggle = 1
      END IF
      
      IF (ViewerOptions AND EZP_VIEWER_HEADERFOOTER) = EZP_VIEWER_HEADERFOOTER THEN
          EZP.HeaderFooterShowToggle = 1
      END IF
      
      IF (ViewerOptions AND EZP_VIEWER_ZOOMFIT) = EZP_VIEWER_ZOOMFIT THEN
          EZP.Zoom = EZP_ZOOMFIT
      END IF
      
      IF (ViewerOptions AND EZP_IGNOREFILEEND) = EZP_IGNOREFILEEND THEN
          DoDocViewOverride = 1
      END IF
   end if   
 
    
''~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
''  Process File
''~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
    MyDocResult = 0   'MyDocResult = 0 is a success
    hFile = CreateFileW(VARptr(MyDoc),GENERIC_READ,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL)
    if hFile <> INVALID_HANDLE_VALUE THEN
        FileSize = GetFileSize(hFile, NULL)
        EZP.FileSize = FileSize
        EZP.FileName = MyDoc
        EZP.FileHandle = hFile
       
       
        ReadFile(hFile,@dh,Sizeof(dh),cast(LPDWORD,@NumBytes),NULL)
 
        CurrentPointer = GetFilePointer(hFile)
        EZP.CurrentPage = 1
        PageCount = 1
        BookMarkCount = 0
        EZP.PagePtrs(0)=min(dh.MyPages,EZP_MAX_PAGES)
        EZP.NumPages = EZP.PagePtrs(0)
        EZP.PagePtrs(PageCount) = CurrentPointer
        EZP.MyVersion = loword(dh.MyVersion)
        EZP.InchesorCM = HIword(dh.MyVersion)
        EZP.MyBin = dh.MyBin
        EZP.MyPaper = dh.MyPaper
        EZP.MyOrient = dh.MyOrient
        EZP.MyDuplex = dh.MyDuplex
        
        if EZP_VERSION <> EZP.MyVersion then
            'Display a failure message
          MessageBox GetForegroundWindow, "Version of document does not match display engine - Abort" & CHR$(0), _
                              "PRINT CONTROL MESSAGE" & CHR$(0),  MB_OK
            CloseHandle(hFile)
            exit function           
        end if   
        
        XPrint.SetPrinter(EZP.MyOrient,EZP.MyBin, EZP.MyPaper,EZP.MyDuplex)

         
        do while CurrentPointer < FileSize
              ReadFile(hFile,@doc,Sizeof(doc),cast(LPDWORD,@NumBytes),NULL)
              'inclued due to extra text len read
              if doc.MyAction=EZP_TEXT THEN
                if doc.TextLen > 0 THEN
                  ZeroMemory (@TextwStr, Sizeof(TextwStr))
                  ReadFile(hFile,varptr(TextwStr),doc.TextLen * 2,cast(LPDWORD,@NumBytes),NULL) 
                END if
              END IF
              
              if doc.MyAction=EZP_IMGA THEN
                if doc.TextLen > 0 THEN
                  EZP.MyGraphics(doc.MyStyle)=space(doc.TextLen) 
                  ReadFile(hFile,Strptr(EZP.MyGraphics(doc.MyStyle)),doc.TextLen,cast(LPDWORD,@NumBytes),NULL) 
                END if
              END IF
              
              'Page Number or date is in doc.mystyle and font size of such is in doc.mysize
              if doc.MyAction=EZP_FOOT THEN
                if doc.TextLen > 0 THEN  
                  ReadFile(hFile,VARptr(EZP.Footer),doc.TextLen * 2,cast(LPDWORD,@NumBytes),NULL)
                  EZP.Footer = EZP.Footer & "|" & tempfontname & "|" & wstr(tempfontStyle) & "|" & wstr(tempfontSize) _
                   & "|" & wstr(doc.MyXPos) & "|" & wstr(doc.MyYPos) & "|" & wstr(doc.MyXPos2) & "|" & wstr(doc.MyYPos2) & "|" & wstr(doc.Mystyle) & "|" & wstr(doc.MySize)
                END if
              END IF
              
              if doc.MyAction=EZP_NAME THEN
                if doc.TextLen > 0 THEN
                  ZeroMemory (@TextwStr, Sizeof(TextwStr))  
                  ReadFile(hFile,VARptr(TextwStr),doc.TextLen * 2,cast(LPDWORD,@NumBytes),NULL)
                  
                  EZP_ParseW(TextwStr,1,varptr(f),"|") : EZP.ControlWords(EZP_PRINT) = f
                  EZP_ParseW(TextwStr,2,varptr(f),"|") : EZP.ControlWords(EZP_CLOSE) = f
                  EZP_ParseW(TextwStr,3,varptr(f),"|") : EZP.ControlWords(EZP_FITWIN) = f
                  EZP_ParseW(TextwStr,4,varptr(f),"|") : EZP.ControlWords(EZP_FITWIDTH) = f
                  
                END if
              END IF
              
              if doc.MyAction=EZP_DATE THEN
                if doc.TextLen > 0 THEN  
                  ReadFile(hFile,VARptr(EZP.HeaderFooterDate),doc.TextLen * 2,cast(LPDWORD,@NumBytes),NULL)
                END if
              END IF
              
              if doc.MyAction=EZP_TXTR THEN 
                if doc.TextLen > 0 THEN
                    SetFilePointer(hFile,CurrentPointer + doc.TextLen * 2,NULL,FILE_BEGIN)
                end if
              END IF
              
              if doc.MyAction=EZP_ARC1 THEN 
                if doc.TextLen > 0 THEN
                    SetFilePointer(hFile,CurrentPointer + doc.TextLen * 2,NULL,FILE_BEGIN)
                end if
              END IF
              
              if doc.MyAction=EZP_PIE1 THEN 
                if doc.TextLen > 0 THEN
                    SetFilePointer(hFile,CurrentPointer + doc.TextLen * 2,NULL,FILE_BEGIN)
                end if
              END IF
              
              if doc.MyAction=EZP_POLY THEN 
                if doc.TextLen > 0 THEN
                    SetFilePointer(hFile,CurrentPointer + doc.TextLen * 2,NULL,FILE_BEGIN)
                end if
              END IF
              
              if doc.MyAction=EZP_TABL THEN 
                if doc.TextLen > 0 THEN
                    SetFilePointer(hFile,CurrentPointer + doc.TextLen,NULL,FILE_BEGIN)
                end if
              END IF
              
              if doc.MyAction=EZP_SHAP THEN 
                if doc.TextLen > 0 THEN
                    SetFilePointer(hFile,CurrentPointer + doc.TextLen * 2,NULL,FILE_BEGIN)
                end if
              END IF
              
              if doc.MyAction=EZP_HEAD THEN
                if doc.TextLen > 0 THEN
                  ReadFile(hFile,VARptr(EZP.Header),doc.TextLen * 2,cast(LPDWORD,@NumBytes),NULL)
                  EZP.Header = EZP.Header & "|" & tempfontname & "|" & wstr(tempfontStyle) & "|" & wstr(tempfontSize) _
                   & "|" & wstr(doc.MyXPos) & "|" & wstr(doc.MyYPos) & "|" & wstr(doc.MyXPos2) & "|" & wstr(doc.MyYPos2) & "|" & wstr(doc.Mystyle) & "|" & wstr(doc.MySize)
                END if
              END IF
              
              'inclued due to extra text len read and load temp font info in case of header or footer
              if doc.MyAction=EZP_FONT THEN
                 if doc.TextLen > 0 THEN 
                   tempfontStyle = doc.MyStyle
                   tempfontsize = doc.MySize
                   ZeroMemory (@TextwStr, Sizeof(TextwStr))
                   ReadFile(hFile,varptr(TextwStr),doc.TextLen * 2,cast(LPDWORD,@NumBytes),NULL)
                   tempfontname = TextwStr

                 end if
              END IF
              'This is the one we are interested in to get the page counters 
              if doc.MyAction=EZP_NWPG THEN
                 'print "New Page  " &  Str(CurrentPointer)
                 PageCount = PageCount + 1
                 if PageCount < EZP_MAX_PAGES + 1 then EZP. PagePtrs(PageCount) = doc.MySize
                 if doc.TextLen > 0 THEN
                     ZeroMemory (@TextwStr, Sizeof(TextwStr))
                     ReadFile(hFile,varptr(TextwStr),doc.TextLen * 2,cast(LPDWORD,@NumBytes),NULL)
                     if Len(TextwStr) > 32 THEN
                       TextwStr = Left(TextwStr,32) 
                     END IF 'Len
                     TextwStr = TextwStr & "|" & str(min(PageCount,EZP_MAX_PAGES))
                     'print TextwStr
                     'if PageCount < MAX_PAGES + 1 then EZP. PagePtrs(PageCount) = doc.MySize
                     'Only make a bookmark if it is legit
                        if BookMarkCount < EZP_MAX_BOOKMARKS and PageCount < EZP_MAX_PAGES + 1 then 
                           EZP.Bookmarks(BookMarkCount) = TextwStr  
                           BookMarkCount = BookMarkCount + 1
                        end if 'PageCount
                  END IF 'TextLen
                  firstpageflag = 1
              END IF 'EZP_NWPG
              
              if doc.MyAction = EZP_ENDD THEN
                  EZP.EndStyle = doc.MyStyle
              END IF
                            
              CurrentPointer = GetFilePointer(hFile)
             EZP.BookmarkCount = BookMarkCount
        Loop
        
       else
          MyDocResult = EZP_FAILED
         'Display a failure message
          MessageBox GetForegroundWindow, "Could not Process the Print Control File" & CHR$(0), _
                              "PRINT CONTROL MESSAGE" & CHR$(0),  MB_OK
          'Resetting the global won't be necessary since the Offscreen was not created
          exit function                    
      END IF 'hFile
      
      'Print "End DoDoc  Read"
      

      if EZP.EndStyle > EZP_END_PRINT + EZP_END_PRINT_NODIALOG + EZP_END_DELETE THEN
          MessageBox GetForegroundWindow, "The Ending Style is Invalid - Will View File" & CHR$(0), _
                                    "PRINT CONTROL MESSAGE" & CHR$(0),  MB_OK
         EZP.EndStyle = 0                           
      END IF
 
     if EZP.EndStyle = EZP_END_VIEW or EZP.EndStyle = EZP_END_DELETE THEN
           viewflag = 0 ' "View the File and make OffScreencanvas  SorP = 0"
       else 'Go straight to printing
          
          if (EZP.EndStyle = EZP_END_PRINT) or (EZP.EndStyle = EZP_END_PRINT + EZP_END_DELETE) THEN
              viewflag = 1 '  Print "Print with Dialog"  'SorP = 1
             else
              viewflag = 2 ' Print "Print with No Dialog"
          END IF 
     END IF 
     
      'const EZP_END_VIEW             =   0
      'const EZP_END_PRINT            =   1    'With Dialogs
      'const EZP_END_PRINT_NODIALOG   =   2
      'const EZP_END_DELETE           =   4    'Delete File
      
     if DoDocViewOverride = 1 then viewflag = 0

     'File dicatated actions
     if viewflag = 0 THEN
           EZP_OffScreenCanvas(EZP.MyOrient,EZP.Papers(EZP.MyPaper,0),EZP.Papers(EZP.MyPaper,1))  'This hard-wired size is temporary
           EZP_ReadCurrentPage(hFile)
           EZP.DialogName  = "EZP Print Preview Dialog"
           EZP_NewDialog(HWND,cast(DLGPROC,@EZP_DlgProc),getModuleHandle(NULL),380,300)
        elseif viewflag = 1 then
           XPrint.ShowPrinterDialogEx(@TextwStr)
           if Len(TextwStr)<> 0 THEN
              EZP.PrintNumCopies = XPrint.nCopies
              EZP.PrintNumPageRanges = XPrint.NumPageRanges
              CopyMemory(varptr(EZP.PrintPageRanges(0)),XPrint.PageRangePtr,10 * sizeof(tagPRINTPAGERANGE))   
              EZP_PrintDoc(hFile,1)  'Print with Print Dialog
            END IF
        elseif viewflag = 2 then    
          EZP_PrintDoc(hFile,0)  'should be 2  Print with no dialog
     END IF 
    
     CloseHandle(hFile)
     
     if (EZP.EndStyle > 3) THEN
        'Kill the existing file if present
        DeleteFileW(varptr(MyDoc)) 
     END IF
     
     bailDoDoc:
     EZP_ResetGlobals 0
     function = MyDocResult 
END Function

SUB EZP_MakeRClickMenu(BYVAL hwnd AS HWND, BYVAL items as wstring ptr )  'items are comma delimited
   DIM x as long
   DIM temp as long
   Dim hMenuPopUp as hMenu
   Dim szBuffer as zstring * 50
   Dim tempstr2 as wstring * 50
   dim tempstr2ptr as Wstring ptr
  
   
   tempstr2ptr = varptr(tempstr2)
  
       'Menu
         IF LEN(TRIM(*items)) > 0 THEN
            x=EZP_PARSECOUNTW(*items)
            IF x > 20 THEN GOTO Done  'max of 20 items
 '          
            hMenuPopUp=CreatePopUpMenu
            FOR temp = 1 TO x
               EZP_PARSEW(*items,temp,tempstr2)
               szBuffer=tempstr2
              IF tempstr2="-" THEN
                 AppendMenu hMenuPopUp,MF_SEPARATOR,0,""
                ELSE
                 AppendMenu hMenuPopUp,MF_STRING,temp,szBuffer
              END IF
            NEXT temp
            IF EZP.RClickMenu <> 0 THEN DestroyMenu EZP.RClickMenu
            EZP.RClickMenu=hMenuPopUp
         END IF 'len
       
  Done:
End Sub

sub EZP_ResetGlobals(byval MyOptions as Long)
  dim x as LONG
  'Need to First Release Objects
  if EZP.ScreenFont <> 0 then DeleteObject EZP.ScreenFont
  if EZP.TextPtr <> 0 then Deallocate EZP.TextPtr
  if EZP.hbit <> 0 then DeleteObject SelectObject(EZP.memDC, EZP.hbit)
  if EZP.memDC <> 0 then DeleteDC cast(hDC,EZP.memDC)
  if EZP.hDC <> 0 then DeleteDC cast(hDC,EZP.hDC)
  ZeroMemory (@EZP, Sizeof(EZPGlobals)) 
END SUB

FUNCTION EZP_ControlAdd (byref strClassName AS WSTRING, BYVAL hDlg AS INT_PTR, BYVAL nId AS integer, _
    byref strTitle AS WSTRING, BYVAL x AS INTEGER, BYVAL y AS INTEGER, BYVAL cx AS INTEGER, _
   BYVAL cy AS INTEGER, BYVAL dwStyle AS UINTEGer, BYVAL dwExStyle AS UINTEGER) AS UINTEGER

   DIM hwnd AS HWND
   dim rc AS RECT
   dim MyWStrPtr as WString ptr
   dim MyWStrPtr2 as WString ptr
   
   MyWStrPtr = varptr(strClassName)
   MyWStrPtr2 = varptr(strTitle)
   
   rc.Left = x : rc.Right  = x + cx
   rc.Top  = y : rc.Bottom = y + cy

   MapDialogRect cast(HWND,hDlg), @rc
   
   hwnd = CreateWindowExW (dwExStyle, MyWStrPtr, MyWStrPtr2, dwStyle, rc.Left, rc.Top, rc.Right - rc.Left, _
          rc.Bottom - rc.Top,cast(HWND, hDlg), cast(hMenu, nID), GetModuleHandle(NULL), NULL)
   
   IF hWnd THEN SendMessage hWnd, WM_SETFONT, SendMessage(cast(HWND,hDlg), WM_GETFONT, 0, 0), 0
  
  function = 0
END FUNCTION

FUNCTION EZP_DlgProc (BYVAL hDlg AS INT_PTR, BYVAL wMsg AS Uint, BYVAL wParam AS wParam, BYVAL lParam AS lParam) AS Integer
   
  DIM ps AS PAINTSTRUCT
  dim hdc as hDC , myhdc as hDC
  dim wRect as RECT
  Dim MyWStr as WSTRING * 255
  dim  si   AS SCROLLINFO
  dim lp AS POINT
  dim iVscrollInc AS LONG, hScrlInc AS LONG
  dim HWND as HWND
  dim ParentHWND as HWND
  Dim _EZPPtr as EZPGlobals PTR
  Dim _EZP as EZPGlobals
  dim hBrush as hGDIOBJ
  dim TempName as wstring * 500
  dim counter as Long
  dim zoomselection as Long
  dim tempscalefactorx as single, tempscalefactory as Single
  dim x as long
  static myfont as hfont
  dim PointSize as Single ,lPointSize as Single
  
  
   SELECT CASE wMsg
      CASE WM_INITDIALOG
        MyhDC = GetDC(cast(HWND,hDlg))
        lPointSize = 10
        PointSize = (lPointSize * 96) \ GetDeviceCaps(MyhDC, LOGPIXELSY) 
        myfont = EZP_CreateAngleFont ( MyhDC,"Arial", EZP_BOLD,PointSize)
        ReleaseDC(cast(HWND,hDlg),MyhDC)
        SendMessage(cast(HWND,hDlg),WM_SETFONT,cast(WPARAM,Myfont),cast(LPARAM,0))                   

       
         EZP_ControlAdd "Button", hDlg, MyCloseBut, "Close", 5, 5, 30, 12, WS_CHILD OR WS_VISIBLE OR WS_TABSTOP OR BS_PUSHBUTTON, 0
         EZP_ControlAdd "Button", hDlg, MyFirstBut, "<<", 40, 5, 15, 12, WS_CHILD OR WS_VISIBLE OR WS_TABSTOP OR BS_PUSHBUTTON, 0
         EZP_ControlAdd "Button", hDlg, MyBackBut, "<", 60, 5, 15, 12, WS_CHILD OR WS_VISIBLE OR WS_TABSTOP OR BS_PUSHBUTTON, 0
         EZP_ControlAdd "static", hDlg, MyPageStatic, "", 83, 7, 30, 8, WS_CHILD OR WS_VISIBLE OR WS_TABSTOP OR SS_CENTER OR SS_NOTIFY, 0
         EZP_ControlAdd "Button", hDlg, MyNextBut, ">", 120, 5, 15, 12, WS_CHILD OR WS_VISIBLE OR WS_TABSTOP OR BS_PUSHBUTTON, 0
         EZP_ControlAdd "Button", hDlg, MyLastBut, ">>", 140, 5, 15, 12, WS_CHILD OR WS_VISIBLE OR WS_TABSTOP OR BS_PUSHBUTTON, 0
         EZP_ControlAdd "combobox", hDlg, MyZoomCombo, "", 160, 5, 60, 60, WS_CHILD OR WS_VISIBLE OR WS_TABSTOP OR CBS_DROPDOWNLIST OR SS_NOTIFY or WS_BORDER or WS_VSCROLL, 0
         EZP_ControlAdd "combobox", hDlg, MyBookmarkCombo, "", 225, 5, 100, 60, WS_CHILD OR WS_VISIBLE OR WS_TABSTOP OR CBS_DROPDOWNLIST OR SS_NOTIFY or WS_BORDER or WS_VSCROLL, 0
         EZP_ControlAdd "Button", hDlg, MyPrintBut, "Print", 335, 5, 30, 12, WS_CHILD OR WS_VISIBLE OR WS_TABSTOP OR BS_PUSHBUTTON, 0
         
         if len(EZP.DialogName) > 0 THEN 
            TempName = EZP.DialogName  & " : " &  *XPrint.PrinterName
            SetWindowTextW cast(HWND,hDlg), varptr(TempName) 
           else 
            TempName = "EZ Print Dialog"   & " : " &  *XPrint.PrinterName 
            SetWindowTextW cast(HWND,hDlg), varptr(TempName)
         END IF
         
         MyWStr = "Toggle Print Margins,Toggle Headers,Toggle Grid,Toggle User Margins"
         EZP_MakeRClickMenu(cast(HWND,hDlg),varptr(MyWStr))
         
         SendMessage(GetDlgItem(Cast(hWnd,hDlg),MyCloseBut),WM_SETFONT,cast(WPARAM,Myfont),cast(LPARAM,0))
         SendMessage(GetDlgItem(Cast(hWnd,hDlg),MyFirstBut),WM_SETFONT,cast(WPARAM,Myfont),cast(LPARAM,0))
         SendMessage(GetDlgItem(Cast(hWnd,hDlg),MyBackBut),WM_SETFONT,cast(WPARAM,Myfont),cast(LPARAM,0))
         SendMessage(GetDlgItem(Cast(hWnd,hDlg),MyPageStatic),WM_SETFONT,cast(WPARAM,Myfont),cast(LPARAM,0))
         SendMessage(GetDlgItem(Cast(hWnd,hDlg),MyNextBut),WM_SETFONT,cast(WPARAM,Myfont),cast(LPARAM,0))
         SendMessage(GetDlgItem(Cast(hWnd,hDlg),MyLastBut),WM_SETFONT,cast(WPARAM,Myfont),cast(LPARAM,0))
         SendMessage(GetDlgItem(Cast(hWnd,hDlg),MyZoomCombo),WM_SETFONT,cast(WPARAM,Myfont),cast(LPARAM,0))
         SendMessage(GetDlgItem(Cast(hWnd,hDlg),MyBookmarkCombo),WM_SETFONT,cast(WPARAM,Myfont),cast(LPARAM,0))
         SendMessage(GetDlgItem(Cast(hWnd,hDlg),MyPrintBut),WM_SETFONT,cast(WPARAM,Myfont),cast(LPARAM,0))
         
         
         'Disable Combobox if no bookmarks else fill the combobox
         if EZP.BookMarkCount = 0 then
            EnableWindow (GetDlgItem(Cast(hWnd,hDlg),MyBookmarkCombo),0)
           else
              for counter = 0 to EZP.BookmarkCount - 1
                 EZP_ParseW(EZP.Bookmarks(Counter),1,@TempName,"|")
                 SendMessageW GetDlgItem(Cast(hWnd,hDlg),MyBookmarkCombo), CB_INSERTSTRING, -1, cast(lParam,@TempName) 
              NEXT
         end if
         
       
          TempName = "100%" :  SendMessageW GetDlgItem(Cast(hWnd,hDlg),MyZoomCombo), CB_INSERTSTRING, -1, cast(lParam,@TempName) 
          TempName = EZP.ControlWords(EZP_FITWIN) :  SendMessageW GetDlgItem(Cast(hWnd,hDlg),MyZoomCombo), CB_INSERTSTRING, -1, cast(lParam,@TempName)
          TempName = EZP.ControlWords(EZP_FITWIDTH) :  SendMessageW GetDlgItem(Cast(hWnd,hDlg),MyZoomCombo), CB_INSERTSTRING, -1, cast(lParam,@TempName) 
          TempName = "125%" :  SendMessageW GetDlgItem(Cast(hWnd,hDlg),MyZoomCombo), CB_INSERTSTRING, -1, cast(lParam,@TempName)  
          TempName = "75%"  :  SendMessageW GetDlgItem(Cast(hWnd,hDlg),MyZoomCombo), CB_INSERTSTRING, -1, cast(lParam,@TempName) 
   
          SendMessage(GetDlgItem(Cast(hWnd,hDlg),MyZoomCombo),CB_SETCURSEL, EZP.Zoom, 0) '100
       
          SetWindowTextW GetDlgItem(Cast(hWnd,hDlg),MyPrintBut), varptr(EZP.ControlWords(EZP_PRINT))
          SetWindowTextW GetDlgItem(Cast(hWnd,hDlg),MyCloseBut), varptr(EZP.ControlWords(EZP_CLOSE))
         
          EZP_SetSBar cast(HWND,hDlg),EZP.CanvasBottom/10, EZP.CanvasBottom, EZP.VScrollPos,  SB_VERT
          EZP_SetSBar cast(HWND,hDlg),EZP.CanvasRight/10, EZP.CanvasRight, EZP.HScrollPos,  SB_HORZ
          ShowWindow(cast(HWND,hDlg), SW_MAXIMIZE)
             
      CASE WM_COMMAND  
        'Notifications 
         Select CASE LOWORD(wParam)

            case MarginsMenu  'Toggles
               if EZP.MarginsShowToggle = 0 then EZP.MarginsShowToggle = 1 else EZP.MarginsShowToggle = 0
               EZP_ReadCurrentPage(EZP.FileHandle,EZP.SorP)
               ' All the scaling and scrollbar modifications are done in the WM_SIZE
               GetClientRect(Cast(hWnd,hDlg),@wRect)
               PostMessage(Cast(hWnd,hDlg),WM_SIZE,SIZE_RESTORED,MAKELONG(wRect.Right-wRect.Left,wRect.Bottom-wRect.Top))
               InvalidateRect cast(HWND,hDlg), BYVAL NULL, 0 : UpdateWindow cast(HWND,hDlg)   
              
            case HeadersMenu  'Toggles 
               if EZP.HeaderFooterShowToggle = 0 then EZP.HeaderFooterShowToggle = 1 else EZP.HeaderFooterShowToggle = 0
               EZP_ReadCurrentPage(EZP.FileHandle,EZP.SorP)
               ' All the scaling and scrollbar modifications are done in the WM_SIZE
               GetClientRect(Cast(hWnd,hDlg),@wRect)
               PostMessage(Cast(hWnd,hDlg),WM_SIZE,SIZE_RESTORED,MAKELONG(wRect.Right-wRect.Left,wRect.Bottom-wRect.Top))
               InvalidateRect cast(HWND,hDlg), BYVAL NULL, 0 : UpdateWindow cast(HWND,hDlg)   
               
            case GridMenu  'Toggles 
               if EZP.GridShowToggle = 0 then EZP.GridShowToggle = 1 else EZP.GridShowToggle = 0
               EZP_ReadCurrentPage(EZP.FileHandle,EZP.SorP)
               ' All the scaling and scrollbar modifications are done in the WM_SIZE
               GetClientRect(Cast(hWnd,hDlg),@wRect)
               PostMessage(Cast(hWnd,hDlg),WM_SIZE,SIZE_RESTORED,MAKELONG(wRect.Right-wRect.Left,wRect.Bottom-wRect.Top))
               InvalidateRect cast(HWND,hDlg), BYVAL NULL, 0 : UpdateWindow cast(HWND,hDlg)   
             
            case UserMarginsMenu 'Toggles
               if EZP.UserMarginsShowToggle = 0 then EZP.UserMarginsShowToggle = 1 else EZP.UserMarginsShowToggle = 0
               EZP_ReadCurrentPage(EZP.FileHandle,EZP.SorP)
               ' All the scaling and scrollbar modifications are done in the WM_SIZE
               GetClientRect(Cast(hWnd,hDlg),@wRect)
               PostMessage(Cast(hWnd,hDlg),WM_SIZE,SIZE_RESTORED,MAKELONG(wRect.Right-wRect.Left,wRect.Bottom-wRect.Top))
               InvalidateRect cast(HWND,hDlg), BYVAL NULL, 0 : UpdateWindow cast(HWND,hDlg) 
               
            case MyZoomCombo
              select CASE Hiword(wParam)
                 case CBN_SELCHANGE
                  
                    GetClientRect(Cast(hWnd,hDlg),@wRect)
                    zoomselection= SendMessage(GetDlgItem(Cast(hWnd,hDlg),MyZoomCombo),CB_GETCURSEL, 0, 0)
                    if zoomselection <> CB_ERR THEN 'CB_ERR = -1
                     
                      select CASE zoomselection
                         case EZP_ZOOM100
                            EZP.zoom = EZP_ZOOM100
                            EZP.zoomfactor = 1 

                         case EZP_ZOOMFIT
                             EZP.Zoom = EZP_ZOOMFIT
                             
                         case EZP_ZOOMWIDTH
                              EZP.Zoom = EZP_ZOOMWIDTH
                              
                         case EZP_ZOOM125
                              EZP.zoom = EZP_ZOOM125
                              EZP.zoomfactor = 1.25 
                            
                         case EZP_ZOOM75            
                              EZP.zoom = EZP_ZOOM75
                              EZP.zoomfactor = 0.75  
                           
                      END SELECT
                      
                    ' All the scaling and scrollbar modifications are done in the WM_SIZE
                    GetClientRect(Cast(hWnd,hDlg),@wRect)
                    PostMessage(Cast(hWnd,hDlg),WM_SIZE,SIZE_RESTORED,MAKELONG(wRect.Right-wRect.Left,wRect.Bottom-wRect.Top))
                    InvalidateRect cast(HWND,hDlg), BYVAL NULL, 0 : UpdateWindow cast(HWND,hDlg)
                       
                    END IF 'if zoomselection not equal err
                    
                 CASE ELSE
                    'Not  CBN_SELCHANGE
              END SELECT
            
            case MyBookmarkCombo
               select CASE Hiword(wParam)
                case CBN_SELCHANGE 
                  
                    x = SendMessage(GetDlgItem(Cast(hWnd,hDlg),MyBookMarkCombo),CB_GETCURSEL, 0, 0)
                    SendMessageW(GetDlgItem(Cast(hWnd,hDlg),MyBookMarkCombo),CB_GETLBTEXT , x, cast(lParam,varptr(MyWStr)))
                    EZP_ParseW(EZP.Bookmarks(x),2,@TempName,"|")  'TempName is the string of the current page number
                    if val(TempName) > 0 and Val(TempName) <= EZP.NumPages THEN
                        hbrush = GetStockObject(WHITE_BRUSH)
                        SelectObject EZP.memDC ,  hbrush
                        PatBlt EZP.memDC, 0, 0, EZP.CanvasRight, EZP.CanvasBottom,  PATCOPY
                        DeleteObject hbrush
                       
                         EZP.CurrentPage = val(TempName)
                         EZP_ReadCurrentPage(EZP.FileHandle,EZP.SorP)
                         InvalidateRect cast(HWND,hDlg), BYVAL NULL, 0 : UpdateWindow cast(HWND,hDlg)
                    END IF
                    
                 CASE ELSE
               end select
                    
            CASE ELSE 
            'Nothing for now   
         END SELECT
         
         'Button clicks
         IF HIWORD(wParam) = BN_CLICKED THEN
            SELECT CASE LOWORD(wParam)
               CASE MyCloseBut
                  EndDialog cast(HWND,hDlg), 1
               CASE MyFirstBut
                  hbrush = GetStockObject(WHITE_BRUSH)
                  SelectObject EZP.memDC ,  hbrush
                  PatBlt EZP.memDC, 0, 0, EZP.CanvasRight, EZP.CanvasBottom,  PATCOPY
                  DeleteObject hbrush
        
                  EZP.CurrentPage = 1
                  EZP_ReadCurrentPage(EZP.FileHandle,EZP.SorP)
                  
                  SendMessage(GetDlgItem(Cast(hWnd,hDlg),MyBookmarkCombo),CB_SETCURSEL , -1, 0)
                  
                  InvalidateRect cast(HWND,hDlg), BYVAL NULL, 0
                  UpdateWindow cast(HWND,hDlg)
                                   
               CASE MyBackBut
                  hbrush = GetStockObject(WHITE_BRUSH)
                  SelectObject EZP.memDC ,  hbrush
                  PatBlt EZP.memDC, 0, 0, EZP.CanvasRight, EZP.CanvasBottom,  PATCOPY
                  DeleteObject hbrush
                  
                  EZP.CurrentPage = MAX(EZP.CurrentPage - 1,1)
                  EZP_ReadCurrentPage(EZP.FileHandle,EZP.SorP)
                  
                  SendMessage(GetDlgItem(Cast(hWnd,hDlg),MyBookmarkCombo),CB_SETCURSEL , -1, 0)
                  
                  InvalidateRect cast(HWND,hDlg), BYVAL NULL, 0
                  UpdateWindow cast(HWND,hDlg)
                  
               CASE MyPageStatic
                 ' beep
               CASE MyNextBut
                  hbrush = GetStockObject(WHITE_BRUSH)
                  SelectObject EZP.memDC ,  hbrush
                  PatBlt EZP.memDC, 0, 0, EZP.CanvasRight, EZP.CanvasBottom,  PATCOPY
                  DeleteObject hbrush
                  
                  EZP.CurrentPage = MIN(EZP.CurrentPage + 1,EZP.NumPages)
                  EZP_ReadCurrentPage(EZP.FileHandle,EZP.SorP)
                  
                  SendMessage(GetDlgItem(Cast(hWnd,hDlg),MyBookmarkCombo),CB_SETCURSEL , -1, 0)
                   
                  InvalidateRect cast(HWND,hDlg), BYVAL NULL, 0
                  UpdateWindow cast(HWND,hDlg)
               CASE MyLastBut
                  hbrush = GetStockObject(WHITE_BRUSH)
                  SelectObject EZP.memDC ,  hbrush
                  PatBlt EZP.memDC, 0, 0, EZP.CanvasRight, EZP.CanvasBottom,  PATCOPY
                  DeleteObject hbrush
        
                  EZP.CurrentPage = EZP.NumPages
                  EZP_ReadCurrentPage(EZP.FileHandle,EZP.SorP)
                  
                  SendMessage(GetDlgItem(Cast(hWnd,hDlg),MyBookmarkCombo),CB_SETCURSEL , -1, 0)
                  
                  InvalidateRect cast(HWND,hDlg), BYVAL NULL, 0
                  UpdateWindow cast(HWND,hDlg)
               
               CASE MyPrintBut
         
                   XPrint.ShowPrinterDialogEx(@MyWStr,0,EZP.PrintDlgFreedom)
                   
                   EZP.PrintNumCopies = XPrint.nCopies
                   EZP.PrintNumPageRanges = XPrint.NumPageRanges
                   CopyMemory(varptr(EZP.PrintPageRanges(0)),XPrint.PageRangePtr,10 * sizeof(tagPRINTPAGERANGE))
                   print MyWStr
                   if len(MyWStr) > 0 then EZP_PrintDoc(EZP.FileHandle,1)
                    
            END SELECT
           else
             ' print "Menu Item   "  & str(cast(long,wParam)) 
         END IF

       case WM_SIZE
          'Calculate the size needed for the off screen bitmap
          'Recalc the scrollbars
          'See if paper needs to be scrolled
          'if x or y is negative, they need to be scrolled
          GetClientRect(Cast(hWnd,hDlg), @wRect)
          EZP.y = EZP.CanvasBottom - (wRect.Bottom - EZP_DIAGVIEWTOP - 16)
          EZP.x = EZP.CanvasRight - (wRect.Right - 16)
          
          if EZP.Zoom = EZP_ZOOM100 THEN
             EZP.zoomfactor = 1 
             if EZP.y > 0 THEN
                EZP.VScrollPos = 1
                EnableScrollBar(cast(HWND,hDlg),SB_VERT , ESB_ENABLE_BOTH)
                EZP_SetSBar cast(HWND,hDlg),EZP.CanvasBottom/10, EZP.CanvasBottom, EZP.VScrollPos,  SB_VERT
               else
                EnableScrollBar(cast(HWND,hDlg),SB_VERT , ESB_DISABLE_BOTH)   
             END IF  
             
             if EZP.x > 0 THEN
                EZP.HScrollPos = 1
                EnableScrollBar(cast(HWND,hDlg),SB_HORZ , ESB_ENABLE_BOTH)
                EZP_SetSBar cast(HWND,hDlg),EZP.CanvasRight/10, EZP.CanvasRight, EZP.HScrollPos,  SB_VERT
               else
                EnableScrollBar(cast(HWND,hDlg),SB_HORZ , ESB_DISABLE_BOTH)   
             END IF      
          END IF
          
          if EZP.Zoom = EZP_ZOOMFIT THEN
              EZP.VScrollPos = 1
              EZP.HScrollPos = 1
              '30 is for size of Scrollbar and then some
              tempscalefactorx = (wRect.Right - 30)/EZP.CanvasRight  :' print wRect.Right, EZP.CanvasRight
              tempscalefactory = (wRect.Bottom-EZP_DIAGVIEWTOP -30)/EZP.CanvasBottom   :' print wRect.Bottom, EZP.CanvasBottom
              if tempscalefactorx >= tempscalefactory THEN
                EZP.zoomfactor = tempscalefactory 
               else
                EZP.zoomfactor = tempscalefactorx 
              END IF 
              EnableScrollBar(cast(HWND,hDlg),SB_BOTH , ESB_DISABLE_BOTH) 'ESB_ENABLE_BOTH       
          END IF
          
          if EZP.Zoom = EZP_ZOOMWIDTH THEN
             EZP.VScrollPos = 1
             EZP.HScrollPos = 1
             'The horizontal scrollbar should always be disabled in theory
             tempscalefactorx = (wRect.Right-30)/EZP.CanvasRight
             EZP.zoomfactor = tempscalefactorx
             EnableScrollBar(cast(HWND,hDlg),SB_HORZ , ESB_DISABLE_BOTH)
             if EZP.y > 0 THEN
                EnableScrollBar(cast(HWND,hDlg),SB_VERT , ESB_ENABLE_BOTH)
                EZP_SetSBar cast(HWND,hDlg),EZP.CanvasBottom/10, EZP.CanvasBottom, EZP.VScrollPos,  SB_VERT
               else
                EnableScrollBar(cast(HWND,hDlg),SB_VERT , ESB_DISABLE_BOTH)   
             END IF         
          END IF
          
          if EZP.Zoom = EZP_ZOOM125 THEN
            EZP.VScrollPos = 1
            EZP.HScrollPos = 1
            EZP.zoomfactor = 1.25
            if EZP.y > 0 THEN
                EnableScrollBar(cast(HWND,hDlg),SB_VERT , ESB_ENABLE_BOTH)
                EZP_SetSBar cast(HWND,hDlg),EZP.CanvasBottom/10, EZP.CanvasBottom, EZP.VScrollPos,  SB_VERT
               else
                EnableScrollBar(cast(HWND,hDlg),SB_VERT , ESB_DISABLE_BOTH)   
             END IF  
             
             if EZP.x > 0 THEN
              
                EnableScrollBar(cast(HWND,hDlg),SB_HORZ , ESB_ENABLE_BOTH)
                EZP_SetSBar cast(HWND,hDlg),EZP.CanvasRight/10, EZP.CanvasRight, EZP.HScrollPos,  SB_VERT
               else
                EnableScrollBar(cast(HWND,hDlg),SB_HORZ , ESB_DISABLE_BOTH)   
             END IF     
             
          end if
          
          if EZP.Zoom = EZP_ZOOM75 THEN
             EZP.VScrollPos = 1
             EZP.HScrollPos = 1
             EZP.zoomfactor = 0.75
              if EZP.y > 0 THEN
                EnableScrollBar(cast(HWND,hDlg),SB_VERT , ESB_ENABLE_BOTH)
                EZP_SetSBar cast(HWND,hDlg),EZP.CanvasBottom/10, EZP.CanvasBottom, EZP.VScrollPos,  SB_VERT
               else
                EnableScrollBar(cast(HWND,hDlg),SB_VERT , ESB_DISABLE_BOTH)   
             END IF  
             
             if EZP.x > 0 THEN
                EnableScrollBar(cast(HWND,hDlg),SB_HORZ , ESB_ENABLE_BOTH)
                EZP_SetSBar cast(HWND,hDlg),EZP.CanvasRight/10, EZP.CanvasRight, EZP.HScrollPos,  SB_VERT
               else
                EnableScrollBar(cast(HWND,hDlg),SB_HORZ , ESB_DISABLE_BOTH)   
             END IF    
             
          end if
              
          
          InvalidateRect cast(HWND,hDlg), BYVAL NULL, 0
          UpdateWindow cast(HWND,hDlg)
          
         
'       case WM_CTLCOLORSTATIC
'          if lParam = GetDlgItem(Cast(hWnd,hDlg),MyPageStatic) THEN
'             SetBkColor(cast(HDC,wParam),GetStockObject(LTGRAY_BRUSH))
'             Function = GetStockObject(LTGRAY_BRUSH)
'          END IF
         
       case WM_NOTIFY  
         
       CASE WM_DESTROY
            if EZP.RClickMenu <> 0 THEN DestroyMenu EZP.RClickMenu  : EZP.RClickMenu = 0
            DeleteObject myfont
            
       CASE WM_VSCROLL
           dim  iVscrollInc as LONG
           
           
'           WM_VSCROLL 
'           nScrollCode = (int) LOWORD(wParam); // scroll bar value 
'           nPos = (short int) HIWORD(wParam); // scroll box position 
'           hwndScrollBar = (HWND) lParam; // handle of scroll bar 
 
        SELECT CASE LOWORD(wParam)
           'CASE SB_TOP        : iVscrollInc = -1
           'CASE SB_BOTTOM     : iVscrollInc = 1
           CASE SB_LINEUP     : iVscrollInc = -EZP.CanvasBottom/100
           CASE SB_LINEDOWN   : iVscrollInc = EZP.CanvasBottom/100
           CASE SB_PAGEUP     : iVscrollInc = -EZP.CanvasBottom/10
           CASE SB_PAGEDOWN   : iVscrollInc =  EZP.CanvasBottom/10
           CASE SB_THUMBTRACK : iVscrollInc = HIWORD(wParam) - EZP.VScrollPos                          ' getScrollInfo enables 32-bit scroll positions
           CASE ELSE : iVscrollInc = 0
        END SELECT
 
        IF iVscrollInc <> 0 THEN
           EZP.VScrollPos = EZP.VScrollPos + iVScrollInc
           if EZP.VScrollPos < 0 then EZP.VScrollPos = 0
           if EZP.VScrollPos > EZP.CanvasBottom then EZP.VScrollPos = EZP.CanvasBottom
           EZP_SetSBar cast(HWND,hDlg),EZP.CanvasBottom/10, EZP.CanvasBottom, EZP.VScrollPos,  SB_VERT
          
           InvalidateRect cast(HWND,hDlg), BYVAL  NULL, 0 
           GetClientRect(Cast(hWnd,hDlg), @wRect) : wRect.Bottom = EZP_DIAGVIEWTOP
           ValidateRect(Cast(hWnd,hDlg), @wRect)
           UpdateWindow cast(HWND,hDlg)
        END IF

    CASE WM_HSCROLL
       dim  hScrlInc as LONG
       
        SELECT CASE LOWORD(wParam)
           CASE SB_LINELEFT      : hScrlInc = -EZP.CanvasRight/20
           CASE SB_LINERIGHT     : hScrlInc =  EZP.CanvasRight/20
           CASE SB_PAGELEFT      : hScrlInc = -EZP.CanvasRight/10
           CASE SB_PAGERIGHT     : hScrlInc =  EZP.CanvasRight/10
           CASE SB_THUMBTRACK    : hScrlInc = HIWORD(wParam) - EZP.HScrollPos
           CASE ELSE             : hScrlInc = 0
        END SELECT
 
          
           
        IF hScrlInc <> 0 THEN
           EZP.HscrollPos = EZP.HscrollPos + hScrlInc 
           if EZP.HScrollPos < 0 then EZP.HScrollPos = 0
           if EZP.HScrollPos > EZP.CanvasRight then EZP.HScrollPos = EZP.CanvasRight
           EZP_SetSBar cast(HWND,hDlg),EZP.CanvasRight/10, EZP.CanvasRight, EZP.HScrollPos,  SB_HORZ
           
           InvalidateRect cast(HWND,hDlg), BYVAL  NULL, 0
           GetClientRect(Cast(hWnd,hDlg), @wRect) : wRect.Bottom = EZP_DIAGVIEWTOP
           ValidateRect(Cast(hWnd,hDlg), @wRect)
           UpdateWindow cast(HWND,hDlg)
        END IF     
          
         
       case WM_CONTEXTMENU
          if EZP.RClickMenu <> 0 THEN
                 TrackPopupMenu(EZP.RClickMenu, TPM_LEFTALIGN OR TPM_LEFTBUTTON,LOWORD(lParam), HIWORD(lParam), 0, cast(HWND,hDlg), BYVAL Null)
          END IF   
         
       CASE  WM_PAINT 'Draw only the list items that needs to be shown
           Dim RectDelta as LONG
          
           hdc = BeginPaint(cast(HWND,hDlg), @ps)

              GetClientRect(Cast(hWnd,hDlg), @wRect)
              
              hbrush = GetStockObject(LTGRAY_BRUSH)
              SelectObject hDC,hBrush
              PatBlt hDC, 0, 0, wRect.Right, wRect.Bottom,  PATCOPY
              DeleteObject hbrush
                            
              ExcludeClipRect(hDC,0,0,wRect.Right,EZP_DIAGVIEWTOP)'Top controls
              dim yyy as Long
              
              wRect.Top = EZP_DIAGVIEWTOP
              yyy = EZP.VScrollPos
              
              'if canvas is smaller then window width then center it
              wRect.Left = (wRect.Right\2 - (EZP.CanvasRight * EZP.zoomfactor)\2) + EZP.HScrollPos
              
              MyWstr=Wstr(EZP.CurrentPage) & "/" & Wstr(EZP.NumPages)
              SetWindowTextW (GetDlgItem(Cast(hWnd,hDlg),MyPageStatic),varptr(MyWStr))
              
          
              IF EZP.Zoomfactor = 1 THEN
                  BitBlt hDC, wRect.left, wRect.top, wRect.right, wRect.bottom, EZP.memDC, 0, yyy,  SRCCOPY
                ELSE
                  StretchBlt hDC, wRect.left, wRect.top, EZP.Canvasright*EZP.zoomfactor, EZP.CanvasBottom * EZP.zoomfactor,_
                  EZP.memDC, 0, yyy,EZP.CanvasRight,EZP.CanvasBottom , SRCCOPY
              END IF
             
           EndPaint(cast(HWND,hDlg), @ps)

      CASE WM_CLOSE
         EndDialog cast(HWND,hDlg), 0

   END SELECT
  function = 0
END FUNCTION

function EZP_NewDialog(byval hParent as HWND, byval lpDialogProc as DLGPROC,byval hInstance as hInstance,byval cx as long,byval cy as long) as INT_PTR
  dim rval as INT_PTR
  dim strTitle as string
  dim lpdt as LPDLGTEMPLATE
  
  'Allocation of memory for DLGTEMPLATE to zero it because some unknown data is tacked on the end
  dim dt as DLGTEMPLATE ptr = callocate(1,sizeof(DLGTEMPLATE) + 300)
   
  dt->style = DS_MODALFRAME OR DS_CENTER OR WS_VISIBLE OR WS_OVERLAPPEDWINDOW or WS_HSCROLL or WS_VSCROLL
  dt->dwExtendedStyle = 0
  dt->cx = cx
  dt->cy = cy
  dt->cdit = 0
  dt->x = 0
  dt->y = 0
  
  
  lpdt = cast(LPDLGTEMPLATE,dt)
  rval = DialogBoxIndirect(hInstance,lpdt,hParent,lpDialogProc)
  
  'if rval = -1 THEN
  '   print GetLastError
  'END IF
                                                     
  DeAllocate dt
  return rval
 
end function
